@@ -25,6 +25,7 @@
 #include <set>
 #include <map>
 #include <vector>
+#include <sstream>
 #include "classad/classad_stl.h"
 #include "classad/exprTree.h"
 
@@ -37,21 +38,37 @@ typedef std::map<const ClassAd*, References> PortReferences;
 #include "classad/rectangle.h"
 #endif
 
-#ifdef WIN32
-typedef classad_hash_map<std::string, ExprTree*, StringCaseIgnHash> AttrList;
-#else
-typedef classad_hash_map<std::string, ExprTree*, StringCaseIgnHash, CaseIgnEqStr> AttrList;
-#endif
+typedef classad_unordered<std::string, ExprTree*, ClassadAttrNameHash, CaseIgnEqStr> AttrList;
 typedef std::set<std::string, CaseIgnLTStr> DirtyAttrList;
 
 void ClassAdLibraryVersion(int &major, int &minor, int &patch);
 void ClassAdLibraryVersion(std::string &version_string);
 
+// Should parsed expressions be cached and shared between multiple ads.
+// The default is false.
+void ClassAdSetExpressionCaching(bool do_caching);
+bool ClassAdGetExpressionCaching();
+
 // This flag is only meant for use in Condor, which is transitioning
 // from an older version of ClassAds with slightly different evaluation
 // semantics. It will be removed without warning in a future release.
 extern bool _useOldClassAdSemantics;
 
+template <class T>
+void val_str(std::string & szOut, const T & tValue)
+{
+  std::stringstream foo;
+  foo<<tValue;
+  szOut = foo.str();
+}
+template<bool>
+void val_str(std::string & szOut, const bool & tValue)
+{
+  std::stringstream foo;
+  foo <<(tValue?"true":"false");
+  szOut = foo.str();
+}
+
 /// The ClassAd object represents a parsed %ClassAd.
 class ClassAd : public ExprTree
 {
@@ -90,7 +107,10 @@ class ClassAd : public ExprTree
 			@return true if the operation succeeded, false otherwise.
 			@see ExprTree::setParentScope
 		*/
-		bool Insert( const std::string &attrName, ExprTree *expr );
+		bool Insert( const std::string& attrName, ExprTree *& pRef, bool cache=true);
+		bool Insert( const std::string& attrName, ClassAd *& expr, bool cache=true );
+		bool Insert( const std::string& serialized_nvp);
+
 
 		/** Inserts an attribute into a nested classAd.  The scope expression is
 		 		evaluated to obtain a nested classad, and the attribute is 
@@ -115,6 +135,10 @@ class ClassAd : public ExprTree
 e		*/
 		bool InsertAttr( const std::string &attrName,int value, 
 				Value::NumberFactor f=Value::NO_FACTOR );
+		bool InsertAttr( const std::string &attrName,long value, 
+				Value::NumberFactor f=Value::NO_FACTOR );
+		bool InsertAttr( const std::string &attrName,long long value, 
+				Value::NumberFactor f=Value::NO_FACTOR );
 
 		/** Inserts an attribute into a nested classad.  The scope expression 
 		 		is evaluated to obtain a nested classad, and the attribute is
@@ -129,6 +153,10 @@ e		*/
 		*/
 		bool DeepInsertAttr( ExprTree *scopeExpr, const std::string &attrName,
 				int value, Value::NumberFactor f=Value::NO_FACTOR );
+		bool DeepInsertAttr( ExprTree *scopeExpr, const std::string &attrName,
+				long value, Value::NumberFactor f=Value::NO_FACTOR );
+		bool DeepInsertAttr( ExprTree *scopeExpr, const std::string &attrName,
+				long long value, Value::NumberFactor f=Value::NO_FACTOR );
 
 		/** Inserts an attribute into the ClassAd.  The real value is
 				converted into a Literal expression, and then inserted into
@@ -349,9 +377,13 @@ e		*/
 		/** Evaluates an attribute to an integer.
 			@param attr The name of the attribute.
 			@param intValue The value of the attribute.
+			If the type of intValue is smaller than a long long, the
+			value may be truncated.
 			@return true if attrName evaluated to an integer, false otherwise.
 		*/
 		bool EvaluateAttrInt( const std::string &attr, int& intValue ) const;
+		bool EvaluateAttrInt( const std::string &attr, long& intValue ) const;
+		bool EvaluateAttrInt( const std::string &attr, long long& intValue ) const;
 
 		/** Evaluates an attribute to a real.
 			@param attr The name of the attribute.
@@ -362,14 +394,22 @@ e		*/
 
 		/** Evaluates an attribute to an integer. If the attribute evaluated to 
 				a real, it is truncated to an integer.
+				If the value is a boolean, it is converted to 0 (for False)
+				or 1 (for True).
 			@param attr The name of the attribute.
 			@param intValue The value of the attribute.
+			If the type of intValue is smaller than a long long, the
+			value may be truncated.
 			@return true if attrName evaluated to an number, false otherwise.
 		*/
 		bool EvaluateAttrNumber( const std::string &attr, int& intValue ) const;
+		bool EvaluateAttrNumber( const std::string &attr, long& intValue ) const;
+		bool EvaluateAttrNumber( const std::string &attr, long long& intValue ) const;
 
 		/** Evaluates an attribute to a real.  If the attribute evaluated to an 
 				integer, it is promoted to a real.
+				If the value is a boolean, it is converted to 0.0 (for False)
+				or 1.0 (for True).
 			@param attr The name of the attribute.
 			@param realValue The value of the attribute.
 			@return true if attrName evaluated to a number, false otherwise.
@@ -412,7 +452,17 @@ e		*/
 			@return true if attrName evaluated to a ClassAd, false 
 				otherwise.
 		*/
-        bool EvaluateAttrClassAd( const std::string &attr, ClassAd *&classad ) const;
+		// This interface is disabled, because it cannot support dynamically allocated
+		// classad values (in case such a thing is ever added, similar to SLIST_VALUE).
+		// Instead, use EvaluateAttr().
+		// Waiting to hear if anybody cares ...
+		// If anybody does, we can make this set a shared_ptr instead, but that
+		// has performance implications that depend on whether all ClassAds
+		// are managed via shared_ptr or only ones dynamically created
+		// during evaluation (because a fresh copy has to be made for
+		// objects not already managed via shared_ptr).  So let's avoid depending
+		// on this interface until we need it.
+        //bool EvaluateAttrClassAd( const std::string &attr, ClassAd *&classad ) const;
 
 		/** Evaluates an attribute to an ExprList.  A pointer to the ExprList is 
 				returned. You do not own the ExprList--do not free it.
@@ -421,7 +471,16 @@ e		*/
 			@return true if attrName evaluated to a ExprList, false 
 				otherwise.
 		*/
-        bool EvaluateAttrList( const std::string &attr, ExprList *&l ) const;
+		// This interface is disabled, because it cannot support dynamically allocated
+		// list values (SLIST_VALUE).  Instead, use EvaluateAttr().
+		// Waiting to hear if anybody cares ...
+		// If anybody does, we can make this set a shared_ptr instead, but that
+		// has performance implications that depend on whether all ExprLists
+		// in ClassAds are managed via shared_ptr or only ones dynamically created
+		// during evaluation  (because a fresh copy has to be made for
+		// objects not already managed via shared_ptr).  So let's avoid depending
+		// on this interface until we need it.
+        //bool EvaluateAttrList( const std::string &attr, ExprList *&l ) const;
 		//@}
 
 		/**@name STL-like Iterators */
@@ -462,14 +521,6 @@ e		*/
         int size(void) const { return attrList.size(); }
 		//@}
 
-		/**@name Miscellaneous */
-		//@{
-		/** Factory method to make a classad
-		 * 	@param vec A vector of (name,expression) pairs to make a classad
-		 * 	@return The constructed classad
-		 */
-		static ClassAd *MakeClassAd( std::vector< std::pair< std::string, ExprTree* > > &vec );
-
 		/** Deconstructor to get the components of a classad
 		 * 	@param vec A vector of (name,expression) pairs which are the
 		 * 		attributes of the classad
@@ -587,6 +638,11 @@ e		*/
          *  @param new_chain_parent_ad the parent ad we are chained too.
          */
 	    void		ChainToAd(ClassAd *new_chain_parent_ad);
+		
+		/** If there is a chained parent remove redundant entries.
+		 */
+		int 		PruneChildAd();
+		
         /** If we are chained to a parent ad, remove the chain. 
          */
 		void		Unchain(void);
@@ -636,7 +692,12 @@ e		*/
          *  @param name The attribute name
          *  @return true if the attribute is dirty, false otherwise
          */
-		bool        IsAttributeDirty(const std::string &name);
+		bool        IsAttributeDirty(const std::string &name) const;
+
+		/* Needed for backward compatibility
+		 * Remove it the next time we have to bump the ClassAds SO version.
+		 */
+		bool        IsAttributeDirty(const std::string &name) {return ((const ClassAd*)this)->IsAttributeDirty(name);}
 
 		typedef DirtyAttrList::iterator dirtyIterator;
         /** Return an interator to the first dirty attribute so all dirty attributes 
