@@ -127,12 +127,13 @@ BasicAnalyze( ClassAd *request, ClassAd *offer ) {
   if (!result_as_struct) { return; }
 
   char remote_user[128];
-  EvalResult eval_result;
+  classad::Value eval_result;
+  bool val;
 
-  bool satisfied_std_rank = EvalExprTree(std_rank_condition, offer, request, &eval_result) && eval_result.type == LX_INTEGER && eval_result.i == TRUE;
-  bool satisfied_preempt_prio = EvalExprTree( preempt_prio_condition, offer, request, &eval_result ) && eval_result.type == LX_INTEGER && eval_result.i == TRUE; 
-  bool satisfied_preempt_rank = EvalExprTree( preempt_rank_condition, offer, request, &eval_result ) && eval_result.type == LX_INTEGER && eval_result.i == TRUE;
-  bool satisfied_preempt_req = EvalExprTree( preemption_req, offer, request, &eval_result ) && eval_result.type == LX_INTEGER && eval_result.i == TRUE;
+  bool satisfied_std_rank = EvalExprTree(std_rank_condition, offer, request, eval_result) && eval_result.IsBooleanValue(val) && val;
+  bool satisfied_preempt_prio = EvalExprTree( preempt_prio_condition, offer, request, eval_result ) && eval_result.IsBooleanValue(val) && val;
+  bool satisfied_preempt_rank = EvalExprTree( preempt_rank_condition, offer, request, eval_result ) && eval_result.IsBooleanValue(val) && val;
+  bool satisfied_preempt_req = EvalExprTree( preemption_req, offer, request, eval_result ) && eval_result.IsBooleanValue(val) && val;
 
   if (!IsAHalfMatch(request, offer)) {
     result_add_explanation(classad_analysis::MACHINES_REJECTED_BY_JOB_REQS, offer);
@@ -245,12 +246,14 @@ result_add_machine(classad::ClassAd resource) {
 
 
 bool ClassAdAnalyzer::
-AnalyzeJobReqToBuffer( ClassAd *request, ClassAdList &offers, string &buffer )
+AnalyzeJobReqToBuffer( ClassAd *request, ClassAdList &offers, string &buffer, std::string &pretty_req )
 {
 	ResourceGroup     rg;
     classad::ClassAd  *explicit_classad;
     bool              success;
 
+	pretty_req = "";
+
 		// create a ResourceGroup object for offer ClassAds
 	if( !MakeResourceGroup( offers, rg ) ) {
 		buffer += "Unable to process machine ClassAds";
@@ -273,7 +276,7 @@ AnalyzeJobReqToBuffer( ClassAd *request, ClassAdList &offers, string &buffer )
       }
     }
 
-	success = AnalyzeJobReqToBuffer( explicit_classad, rg, buffer );
+	success = AnalyzeJobReqToBuffer( explicit_classad, rg, buffer, pretty_req );
 
     delete explicit_classad;
     return success;
@@ -323,7 +326,7 @@ AnalyzeJobAttrsToBuffer( ClassAd *request, ClassAdList &offers,
 
 
 bool ClassAdAnalyzer::
-AnalyzeJobReqToBuffer( classad::ClassAd *request, ResourceGroup &offers, string &buffer)
+AnalyzeJobReqToBuffer( classad::ClassAd *request, ResourceGroup &offers, string &buffer, string &pretty_req)
 {
 	if( !request ) {
 			// request is NULL;
@@ -373,15 +376,15 @@ AnalyzeJobReqToBuffer( classad::ClassAd *request, ResourceGroup &offers, string
 		t++;
 	}
 		// Print formatted req expression
-	buffer += "\n";
-	buffer += "The ";
-	buffer += ATTR_REQUIREMENTS; 
-	buffer += " expression for your job is:";
-	buffer += "\n";
-	buffer += "\n";
-	buffer += temp_buffer;
-	buffer += "\n";
-	buffer += "\n";
+	pretty_req += "\n";
+	pretty_req += "The ";
+	pretty_req += ATTR_REQUIREMENTS; 
+	pretty_req += " expression for your job is:";
+	pretty_req += "\n";
+	pretty_req += "\n";
+	pretty_req += temp_buffer;
+	pretty_req += "\n";
+	pretty_req += "\n";
 
 		// Try to flatten Requirements expression
 	mad.ReplaceLeftAd( request );
@@ -608,7 +611,7 @@ AnalyzeJobAttrsToBuffer( classad::ClassAd *request, ResourceGroup &offers,
 	char suggest[64];
 
 	if( !( AnalyzeAttributes( request, offers, adExplain ) ) ) {
-		cerr << "error in AnalyzeAttributes" << endl << endl;
+		errstm << "error in AnalyzeAttributes" << endl << endl;
 	}
 
 		// get information from ClassAdExplain
@@ -732,16 +735,16 @@ AnalyzeExprToBuffer( classad::ClassAd *mainAd, classad::ClassAd *contextAd, stri
 	contextList.Append( copyContextAd );
 
 	if( !( rg.Init( contextList ) ) ) {
-		cerr << "problem adding job ad to ResourceGroup\n";
+		errstm << "problem adding job ad to ResourceGroup\n";
 	}
 	if( !( expr = mainAd->Lookup( attr ) ) ) {
-		cerr << "error looking up " << attr << " expression\n";
+		errstm << "error looking up " << attr << " expression\n";
         delete mp;
 		return false;
 	}
 	
 	if( !( mainAd->FlattenAndInline( expr, val, flatExpr ) ) ) {
-		cerr << "error flattening machine ad\n";
+		errstm << "error flattening machine ad\n";
         delete mp;
 		return false;
 	}
@@ -756,22 +759,22 @@ AnalyzeExprToBuffer( classad::ClassAd *mainAd, classad::ClassAd *contextAd, stri
 	}
 
 	if( !PruneDisjunction( flatExpr, prunedExpr ) ) {
-		cerr << "error pruning expression:\n";
+		errstm << "error pruning expression:\n";
 		pp.Unparse( tempBuff_s, flatExpr );
-		cerr << tempBuff_s << "\n";
+		errstm << tempBuff_s << "\n";
         delete mp;
 		return false;
 	}
 
 	if( !( BoolExpr::ExprToMultiProfile( prunedExpr, mp ) ) ) {
-		cerr << "error in ExprToMultiProfile\n";
+		errstm << "error in ExprToMultiProfile\n";
         delete mp;
 		return false;
 	}		
 	
 		// Do analysis
 	if( !SuggestCondition( mp, rg ) ) {
-		cerr << "error in SuggestCondition\n";
+		errstm << "error in SuggestCondition\n";
 	}
 	
 		// Print results
@@ -843,19 +846,19 @@ BuildBoolTable( MultiProfile *mp, ResourceGroup &rg, BoolTable &result )
 	int numContexts = 0;
 
 	if( !mp->GetNumberOfProfiles( numProfs ) ) {
-		cerr << "BuildBoolTable: error calling GetNumberOfProfiles" << endl;
+		errstm << "BuildBoolTable: error calling GetNumberOfProfiles" << endl;
 	}
 
 	if( !rg.GetNumberOfClassAds( numContexts ) ) {
-		cerr << "BuildBoolTable: error calling GetNumberOfClassAds" << endl;
+		errstm << "BuildBoolTable: error calling GetNumberOfClassAds" << endl;
 	}
 
 	if( !rg.GetClassAds( contexts ) ) {
-		cerr << "BuildBoolTable: error calling GetClassAds" << endl;
+		errstm << "BuildBoolTable: error calling GetClassAds" << endl;
 	}
 
 	if( !result.Init( numContexts, numProfs ) ) {
-		cerr << "BuildBoolTable: error calling BoolTable::Init" << endl;
+		errstm << "BuildBoolTable: error calling BoolTable::Init" << endl;
 	}
 
 	contexts.Rewind( );
@@ -960,7 +963,7 @@ bool ClassAdAnalyzer::
 SuggestCondition( MultiProfile *mp, ResourceGroup &rg )
 {
 	if( mp == NULL ) {
-		cerr << "SuggestCondition: tried to pass null MultiProfile"
+		errstm << "SuggestCondition: tried to pass null MultiProfile"
 			 << endl;
 		return false;
 	}
@@ -1001,7 +1004,7 @@ SuggestCondition( MultiProfile *mp, ResourceGroup &rg )
 	mp->Rewind( );
 	while( mp->NextProfile( currentProfile ) ) {
 		if( !SuggestConditionModify( currentProfile, rg ) ) {
-			cerr << "error in SuggestConditionModify" << endl;
+			errstm << "error in SuggestConditionModify" << endl;
 			return false;
 		}
 //		if( !SuggestConditionRemove( currentProfile, rg ) ) {
@@ -1094,7 +1097,7 @@ SuggestConditionRemove( Profile *p, ResourceGroup &rg )
 		// find first ABV with max frequency & max total true	
 		// set up ConditionExplains using ABV
 	if( !AnnotatedBoolVector::MostFreqABV( abvList, bestABV ) ) {
-		cerr << "Analysis::SuggestConditionRemove(): error - bad ABV" << endl;
+		errstm << "Analysis::SuggestConditionRemove(): error - bad ABV" << endl;
 		abvList.Rewind( );
 		while( abvList.Next( abv ) ) {
 			delete abv;
@@ -1495,7 +1498,7 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
 
 		// get list of ClassAds from ResourceGroup
 	if( !( rg.GetClassAds( offerList ) ) ) {
-		cerr << "CA::AA: error with GetClassAds" << endl << endl;
+		errstm << "CA::AA: error with GetClassAds" << endl << endl;
 		return false;
 	}
 
@@ -1524,7 +1527,7 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
 		currReq = new MultiProfile( );
 
 		if( !( reqExpr = offer->Lookup( ATTR_REQUIREMENTS ) ) ) {
-			cerr << "error looking up requirements" << endl << endl;
+			errstm << "error looking up requirements" << endl << endl;
             delete currReq;
 			reqList.Rewind( );
 			while( reqList.Next( currReq ) ) {
@@ -1535,7 +1538,7 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
 
 		
 		if( !( offer->FlattenAndInline( reqExpr, val, flatReqExpr ) ) ) {
-			cerr << "error flattening request" << endl << endl;
+			errstm << "error flattening request" << endl << endl;
             delete currReq;
 			reqList.Rewind( );
 			while( reqList.Next( currReq ) ) {
@@ -1548,9 +1551,9 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
 				// we have a non-literal boolean expression 
 
 			if( !( PruneDisjunction( flatReqExpr, prunedReqExpr ) ) ) {
-				cerr << "error pruning expression:" << endl;
+				errstm << "error pruning expression:" << endl;
 				pp.Unparse( buffer, flatReqExpr );
-				cerr << buffer << endl << endl;
+				errstm << buffer << endl << endl;
 				buffer = "";
 				delete flatReqExpr;
                 delete currReq;
@@ -1562,7 +1565,7 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
 			}
 			
 			if( !( BoolExpr::ExprToMultiProfile( prunedReqExpr, currReq ) ) ) {
-				cerr << "error in ExprToMultiProfile" << endl << endl;
+				errstm << "error in ExprToMultiProfile" << endl << endl;
 				delete flatReqExpr;
 				delete prunedReqExpr;
 				delete currReq;
@@ -1582,7 +1585,7 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
 
 				// we have a literal boolean expression
 			if( !( BoolExpr::ValToMultiProfile( val, currReq ) ) ) {
-				cerr << "error in ValToMultiProfile" << endl << endl;
+				errstm << "error in ValToMultiProfile" << endl << endl;
 				delete currReq;
 				reqList.Rewind( );
 				while( reqList.Next( currReq ) ) {
@@ -1681,7 +1684,7 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
 
 				string currAttr;
 				if( !currCondition->GetAttr( currAttr ) ) {
-					cerr << "AA error: couldn't get attribute" << endl;
+					errstm << "AA error: couldn't get attribute" << endl;
 					exit(1);
 				}
 
@@ -1711,7 +1714,7 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
 
 				BoolValue conditionValue;
 				if( !currCondition->EvalInContext( mad, ad, conditionValue) ) {
-					cerr << "AA error: BoolExpr::EvalInContext failed" << endl;
+					errstm << "AA error: BoolExpr::EvalInContext failed" << endl;
 					exit(1);
 				}
 
@@ -1721,7 +1724,7 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
 					BoolValue oldValue = ( *tempBools )[attrNum];
 					And( oldValue, conditionValue, newValue );
 					( *tempBools )[attrNum] = newValue;
-					if( ( ( *tempIntervals )[attrNum] == NULL ) ) {
+					if( ( *tempIntervals )[attrNum] == NULL ) {
 						( *tempIntervals )[attrNum] = new ValueRange( );
 					}
 					AddConstraint( ( *tempIntervals )[attrNum],
@@ -1747,7 +1750,7 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
 					}
 					( *tempBools )[numRefdAttrs - 1] = conditionValue;
 
-					if( ( ( *tempIntervals )[attrNum] == NULL ) ) {
+					if( ( *tempIntervals )[attrNum] == NULL ) {
 						( *tempIntervals )[attrNum] = new ValueRange( );
 					}
 					AddConstraint( ( *tempIntervals )[attrNum], 
@@ -2076,21 +2079,21 @@ AddConstraint( ValueRange *&vr, Condition *condition )
 	classad::PrettyPrint pp;
 	string buffer;
 	if( condition == NULL ) {
-		cerr << "Error: passed NULL Condition pointer to AddConstraint"
+		errstm << "Error: passed NULL Condition pointer to AddConstraint"
 			 << endl;
 		return false;
 	}
 
 	if( vr == NULL ) {
-		cerr << "Error: passed NULL ValueRange pointer to AddConstraint"
+		errstm << "Error: passed NULL ValueRange pointer to AddConstraint"
 			 << endl;
 		return false;
 	}
 
 	if( condition->IsComplex( ) && condition->HasMultipleAttrs( ) ) {
-		cerr << "AddConstraint: can't process complex Condition:" << endl;
+		errstm << "AddConstraint: can't process complex Condition:" << endl;
 		condition->ToString( buffer );
-		cerr << buffer << endl;
+		errstm << buffer << endl;
 		return false;
 	}
 
@@ -2135,15 +2138,15 @@ AddConstraint( ValueRange *&vr, Condition *condition )
 				twoVals = true;
 			}
 			else {
-				cerr << "AddConstraint: can't process complex Condition"<<endl;
+				errstm << "AddConstraint: can't process complex Condition"<<endl;
 				pp.Unparse( buffer, val1 );
-				cerr << "val1 is " << buffer << endl;
+				errstm << "val1 is " << buffer << endl;
 				buffer = "";
 				pp.Unparse( buffer, val2 );
-				cerr << "val2 is " << buffer << endl;
+				errstm << "val2 is " << buffer << endl;
 				buffer = "";
                 condition->ToString( buffer );
-                cerr << buffer << endl;
+                errstm << buffer << endl;
                 return false;
 			}
 //			}
@@ -2227,7 +2230,7 @@ AddConstraint( ValueRange *&vr, Condition *condition )
 				return true;
 			}
 			else {
-				cerr << "AddConstraint: error: boolean value expected"
+				errstm << "AddConstraint: error: boolean value expected"
 					 << endl;
 				return false;
 			}
@@ -2291,7 +2294,7 @@ AddConstraint( ValueRange *&vr, Condition *condition )
 		default: {
 			string expr;
 			condition->ToString(expr);
-			cerr << "AddConstraint: Condition value not literal: '" << val << "' in '" << expr << "'" << endl;
+			errstm << "AddConstraint: Condition value not literal: '" << val << "' in '" << expr << "'" << endl;
 			return false;
 		}
 		}
@@ -2459,7 +2462,7 @@ bool ClassAdAnalyzer::
 PruneDisjunction( classad::ExprTree *expr, classad::ExprTree *&result )
 {
 	if( !expr ) {
-		cerr << "PD error: null expr" << endl; 
+		errstm << "PD error: null expr" << endl; 
 		return false;
 	}
 
@@ -2486,7 +2489,7 @@ PruneDisjunction( classad::ExprTree *expr, classad::ExprTree *&result )
 		}
 		else if( !( result=classad::Operation::MakeOperation( classad::Operation::PARENTHESES_OP,
 													 result, NULL, NULL ) ) ) {
-			cerr << "PD error: can't make Operation" << endl;
+			errstm << "PD error: can't make Operation" << endl;
 			return false;
 		}
 		else {
@@ -2511,7 +2514,7 @@ PruneDisjunction( classad::ExprTree *expr, classad::ExprTree *&result )
 		!newLeft || !newRight ||
 		!( result = classad::Operation::MakeOperation( classad::Operation::LOGICAL_OR_OP,
 											  newLeft, newRight, NULL ) ) ) {
-		cerr << "PD error: can't make Operation" << endl;
+		errstm << "PD error: can't make Operation" << endl;
 		return false;
 	}
 	return true;
@@ -2520,7 +2523,7 @@ PruneDisjunction( classad::ExprTree *expr, classad::ExprTree *&result )
 bool ClassAdAnalyzer::
 PruneConjunction( classad::ExprTree *expr, classad::ExprTree *&result ) {
 	if( !expr ) {
-		cerr << "PC error: null expr" << endl; 
+		errstm << "PC error: null expr" << endl; 
 		return false;
 	}
 
@@ -2547,7 +2550,7 @@ PruneConjunction( classad::ExprTree *expr, classad::ExprTree *&result ) {
 		}
 		else if( !( result=classad::Operation::MakeOperation( classad::Operation::PARENTHESES_OP,
 													 result, NULL, NULL ) ) ) {
-			cerr << "PC error: can't make Operation" << endl;
+			errstm << "PC error: can't make Operation" << endl;
 			return false;
 		}
 		else {
@@ -2576,7 +2579,7 @@ PruneConjunction( classad::ExprTree *expr, classad::ExprTree *&result ) {
 		!newLeft || !newRight  ||
 		!( result = classad::Operation::MakeOperation( classad::Operation::LOGICAL_AND_OP,
 											  newLeft, newRight, NULL ) ) ) {
-		cerr << "PC error: can't Make Operation" << endl;
+		errstm << "PC error: can't Make Operation" << endl;
 		return false;
 	}
 	return true;
@@ -2586,7 +2589,7 @@ bool ClassAdAnalyzer::
 PruneAtom( classad::ExprTree *expr, classad::ExprTree *&result )
 {
 	if( !expr ) {
-		cerr << "PA error: null expr" << endl; 
+		errstm << "PA error: null expr" << endl; 
 		return false;
 	}
 
@@ -2608,12 +2611,12 @@ PruneAtom( classad::ExprTree *expr, classad::ExprTree *&result )
 
 	if( op == classad::Operation::PARENTHESES_OP ) {
 		if( !PruneAtom( left, result ) ) {
-			cerr << "PA error: problem with expression in parens" << endl;
+			errstm << "PA error: problem with expression in parens" << endl;
 			return false;
 		}
 		else if( !( result=classad::Operation::MakeOperation(classad::Operation::PARENTHESES_OP, 
 													result, NULL, NULL ) ) ) {
-			cerr << "PA error: can't make Operation" << endl;
+			errstm << "PA error: can't make Operation" << endl;
 			return false;
 		}
 		else {
@@ -2630,14 +2633,14 @@ PruneAtom( classad::ExprTree *expr, classad::ExprTree *&result )
 	}
 
 	if( left == NULL || right == NULL ) {
-		cerr << "PA error: NULL ptr in expr" << endl;
+		errstm << "PA error: NULL ptr in expr" << endl;
 			// error: NULL ptr in expr
 		return false;
 	}
 
 	if( !( result = classad::Operation::MakeOperation( op, left->Copy( ), right->Copy(),
 											  NULL ) ) ) {
-		cerr << "PA error: can't make Operation" << endl;
+		errstm << "PA error: can't make Operation" << endl;
 		return false;
 	}
 
@@ -2649,7 +2652,7 @@ bool ClassAdAnalyzer::
 InDNF( classad::ExprTree * tree )
 {
 	if( tree == NULL ) {
-		cerr << "InDNF: tried to pass null pointer" << endl;
+		errstm << "InDNF: tried to pass null pointer" << endl;
 		return false;
 	}
 
@@ -2689,7 +2692,7 @@ bool ClassAdAnalyzer::
 IsAtomicBooleanFormula( classad::Operation *tree )
 {
 	if( tree == NULL ) {
-		cerr << "IsAtomicBooleanFormula: tried to pass null pointer" << endl;
+		errstm << "IsAtomicBooleanFormula: tried to pass null pointer" << endl;
 		return false;
 	}
 
@@ -2702,7 +2705,7 @@ bool ClassAdAnalyzer::
 PropagateNegation( classad::ExprTree *tree, classad::ExprTree *&result )
 {
 	if( tree == NULL ) {
-		cerr << "PropagateNegation: tried to pass null pointer" << endl;
+		errstm << "PropagateNegation: tried to pass null pointer" << endl;
 		return false;
 	}
 
@@ -2715,7 +2718,7 @@ bool ClassAdAnalyzer::
 ToDNF( classad::ExprTree *tree, classad::ExprTree *&result )
 {
 	if( tree == NULL ) {
-		cerr << "ToDNF: tried to pass null pointer" << endl;
+		errstm << "ToDNF: tried to pass null pointer" << endl;
 		return false;
 	}
 		// FINISH THIS CODE
