@@ -48,6 +48,7 @@
 #include "protocol.h"
 #include "internet_obsolete.h"
 #include "condor_sockfunc.h"
+#include "selector.h"
 using namespace std;
 
 XferSummary	xfer_summary;
@@ -178,7 +179,7 @@ void Server::Init()
 	num_replicate_xfers = 0;
 
 	config();
-	dprintf_config( get_mySubSystem()->getName(), get_param_functions() );
+	dprintf_config( get_mySubSystem()->getName() );
 
 	set_condor_priv();
 
@@ -420,8 +421,7 @@ void Server::SetUpPeers()
 
 void Server::Execute()
 {
-	fd_set         req_sds;
-	int            num_sds_ready = 0;
+	Selector       selector;
 	time_t         current_time;
 	time_t         last_reclaim_time;
 	time_t		   last_clean_time;
@@ -453,6 +453,11 @@ void Server::Execute()
 	// this ensures I don't count the time it took to remove them.
 	last_remove_stale_time = time(NULL);
 
+	selector.add_fd( store_req_sd, Selector::IO_READ );
+	selector.add_fd( restore_req_sd, Selector::IO_READ );
+	selector.add_fd( service_req_sd, Selector::IO_READ );
+	selector.add_fd( replicate_req_sd, Selector::IO_READ );
+
 	while (more) {                          // Continues until SIGUSR2 signal
 		poll.tv_sec = reclaim_interval - ((unsigned int)current_time -
 										  (unsigned int)last_reclaim_time);
@@ -484,49 +489,40 @@ void Server::Execute()
 			poll.tv_sec = remove_stale_ckptfile_interval;
 		}
 
-		errno = 0;
-		FD_ZERO(&req_sds);
-		FD_SET(store_req_sd, &req_sds);
-		FD_SET(restore_req_sd, &req_sds);
-		FD_SET(service_req_sd, &req_sds);
-		FD_SET(replicate_req_sd, &req_sds);
 		// Can reduce the number of calls to the reclaimer by blocking until a
 		//   request comes in (change the &poll argument to NULL).  The 
 		//   philosophy is that one does not need to reclaim a child process
 		//   until another is ready to start
 		UnblockSignals();
-		while( (more) && 
-			   ((num_sds_ready=select(max_req_sd_plus1,
-									  (SELECT_FDSET_PTR)&req_sds, 
-									  NULL, NULL, &poll)) > 0)) {
-		        BlockSignals();
+		selector.set_timeout(poll);
+		selector.execute();
+		while( (more) && selector.has_ready() ) {
+
+			BlockSignals();
 	   
-			if (FD_ISSET(service_req_sd, &req_sds))
+			if (selector.fd_ready(service_req_sd, Selector::IO_READ))
 				HandleRequest(service_req_sd, SERVICE_REQ);
-			if (FD_ISSET(store_req_sd, &req_sds))
+			if (selector.fd_ready(store_req_sd, Selector::IO_READ))
 				HandleRequest(store_req_sd, STORE_REQ);
-			if (FD_ISSET(restore_req_sd, &req_sds))
+			if (selector.fd_ready(restore_req_sd, Selector::IO_READ))
 				HandleRequest(restore_req_sd, RESTORE_REQ);
-			if (FD_ISSET(replicate_req_sd, &req_sds))
+			if (selector.fd_ready(replicate_req_sd, Selector::IO_READ))
 				HandleRequest(replicate_req_sd, REPLICATE_REQ);
-			FD_ZERO(&req_sds);
-			FD_SET(store_req_sd, &req_sds);
-			FD_SET(restore_req_sd, &req_sds);
-			FD_SET(service_req_sd, &req_sds);	  
-			FD_SET(replicate_req_sd, &req_sds);
 
 			UnblockSignals();
+			selector.execute();
 		}
 		BlockSignals();
-		if (num_sds_ready < 0) {
-			if (errno == ECHILD) {
+		if (selector.failed()) {
+			int select_errno = selector.select_errno();
+			if (select_errno == ECHILD) {
 				// Note: we shouldn't really get an ECHILD here, but
 				// we did (see condor-admin 14075).
 				dprintf(D_ALWAYS, "WARNING: got ECHILD from select()\n");
-			} else if (errno != EINTR) {
+			} else {
 				dprintf(D_ALWAYS, 
 						"ERROR: cannot select from request ports, errno = "
-						"%d (%s)\n", errno, strerror(errno));
+						"%d (%s)\n", select_errno, strerror(select_errno));
 				exit(SELECT_ERROR);
 			}
 		}
@@ -555,7 +551,7 @@ void Server::Execute()
 			if( kill(ppid,0) < 0 ) {
 				if( errno == ESRCH ) {
 					MyString msg;
-					msg.sprintf("Parent process %d has gone away", ppid);
+					msg.formatstr("Parent process %d has gone away", ppid);
 					NoMore(msg.Value());
 				}
 			}
@@ -1016,7 +1012,7 @@ void Server::ProcessServiceReq(int             req_id,
 										  service_req.owner_name, 
 										  service_req.file_name);
 				MyString key;
-				key.sprintf("%s/%s/%s", inet_ntoa(shadow_IP), 
+				key.formatstr("%s/%s/%s", inet_ntoa(shadow_IP), 
 						service_req.owner_name, service_req.file_name);
 				if (CkptClassAds) {
 					CkptClassAds->DestroyClassAd(key.Value());
@@ -1580,7 +1576,7 @@ void Server::ProcessStoreReq(int            req_id,
 				store_req.filename[len-4] = '\0';
 			}
 			MyString keybuf;
-			keybuf.sprintf( "%s/%s/%s", inet_ntoa(shadow_IP), store_req.owner,
+			keybuf.formatstr( "%s/%s/%s", inet_ntoa(shadow_IP), store_req.owner,
 					store_req.filename);
 			char const *key = keybuf.Value();
 			ClassAd *ad;
@@ -1588,11 +1584,11 @@ void Server::ProcessStoreReq(int            req_id,
 				if (!CkptClassAds->LookupClassAd(key, ad)) {
 					MyString buf;
 					CkptClassAds->NewClassAd(key, CKPT_FILE_ADTYPE, "0");
-					buf.sprintf( "\"%s\"", store_req.owner);
+					buf.formatstr( "\"%s\"", store_req.owner);
 					CkptClassAds->SetAttribute(key, ATTR_OWNER, buf.Value());
-					buf.sprintf( "\"%s\"", inet_ntoa(shadow_IP));
+					buf.formatstr( "\"%s\"", inet_ntoa(shadow_IP));
 					CkptClassAds->SetAttribute(key, ATTR_SHADOW_IP_ADDR, buf.Value());
-					buf.sprintf( "\"%s\"", store_req.filename);
+					buf.formatstr( "\"%s\"", store_req.filename);
 					CkptClassAds->SetAttribute(key, ATTR_FILE_NAME, buf.Value());
 				}
 				char size[40];
@@ -2230,7 +2226,7 @@ void Server::RemoveStaleCheckpointFiles(const char *directory)
 	next_time_to_remove_stale_ckpt_files =
 		time(NULL) + remove_stale_ckptfile_interval;
 
-	str.sprintf("Next stale checkpoint file check in %lu seconds.",
+	str.formatstr("Next stale checkpoint file check in %lu seconds.",
 		(unsigned long)remove_stale_ckptfile_interval);
 
 	Log(str.Value());
@@ -2252,14 +2248,14 @@ void Server::RemoveStaleCheckpointFilesRecurse(const char *path,
 	}
 
 	if (realpath(path, real_path) == 0) {
-		str.sprintf("Server::RemoveStaleCheckpointFilesRecurse(): Could "
+		str.formatstr("Server::RemoveStaleCheckpointFilesRecurse(): Could "
 			"not resolve %s into a real path: %d(%s). Ignoring.\n",
 			path, errno, strerror(errno));
 		return;
 	}
 
 	if (realpath(ckpt_server_dir, real_ckpt_server_dir) == 0) {
-		str.sprintf("Server::RemoveStaleCheckpointFilesRecurse(): Could "
+		str.formatstr("Server::RemoveStaleCheckpointFilesRecurse(): Could "
 			"not resolve %s into a real path: %d(%s). Strange..ignoring "
 			"remove request for file under this directory.\n",
 			path, errno, strerror(errno));
@@ -2282,7 +2278,7 @@ void Server::RemoveStaleCheckpointFilesRecurse(const char *path,
 		((strlen(real_path) > strlen(real_ckpt_server_dir)) && 
 			path[strlen(real_ckpt_server_dir)] != '/'))
 	{
-		str.sprintf(
+		str.formatstr(
 			"WARNING: "
 			"Server::RemoveStaleCheckpointFilesRecurse(): "
 			"path name %s, whose real path is %s, appears to be outside "
