@@ -30,9 +30,6 @@
 
 //-------------------------------------------------------------
 
-// about self
-DECL_SUBSYSTEM("COLLECTOR", SUBSYSTEM_TYPE_COLLECTOR );
-
 // the heart of the collector ...
 CollectorDaemon* Daemon;
 
@@ -99,15 +96,14 @@ void main_shutdown_graceful()
 	DC_Exit(0);
 }
 
-
-void
-main_pre_dc_init( int argc, char* argv[] )
+int
+main( int argc, char **argv )
 {
+	set_mySubSystem("COLLECTOR", SUBSYSTEM_TYPE_COLLECTOR );
+
+	dc_main_init = main_init;
+	dc_main_config = main_config;
+	dc_main_shutdown_fast = main_shutdown_fast;
+	dc_main_shutdown_graceful = main_shutdown_graceful;
+	return dc_main( argc, argv );
 }
-
-
-void
-main_pre_command_sock_init( )
-{
-}
-
