@@ -44,8 +44,9 @@ typedef enum {
 	VIEW_STATE_SHUTDOWN = 6,
 	VIEW_STATE_DELETE = 7,
 	VIEW_STATE_BACKFILL = 8,
+	VIEW_STATE_DRAINED = 9,
 
-	VIEW_STATE_MAX = VIEW_STATE_BACKFILL,
+	VIEW_STATE_MAX = VIEW_STATE_DRAINED,
 	VIEW_STATE_MAX_OFFSET = VIEW_STATE_MAX - 1,
 } ViewStates;
 struct GeneralRecord {
@@ -132,7 +133,7 @@ private:
 	// misc variables
 
 	static int HistoryTimer;
-	static int KeepHistory;
+	static bool KeepHistory;
 
 	// Utility functions
 
