@@ -113,15 +113,15 @@ CredDaemon::initialize_classad()
 {
 	m_classad.Clear();
 
-	m_classad.SetMyTypeName(CREDD_ADTYPE);
-	m_classad.SetTargetTypeName("");
+	SetMyTypeName(m_classad, CREDD_ADTYPE);
+	SetTargetTypeName(m_classad, "");
 
 	MyString line;
 
-	line.sprintf("%s = \"%s\"", ATTR_NAME, m_name );
+	line.formatstr("%s = \"%s\"", ATTR_NAME, m_name );
 	m_classad.Insert(line.Value());
 
-	line.sprintf ("%s = \"%s\"", ATTR_CREDD_IP_ADDR,
+	line.formatstr ("%s = \"%s\"", ATTR_CREDD_IP_ADDR,
 			daemonCore->InfoCommandSinfulString() );
 	m_classad.Insert(line.Value());
 
@@ -140,11 +140,11 @@ void
 CredDaemon::invalidate_ad()
 {
 	ClassAd query_ad;
-	query_ad.SetMyTypeName(QUERY_ADTYPE);
-	query_ad.SetTargetTypeName(CREDD_ADTYPE);
+	SetMyTypeName(query_ad, QUERY_ADTYPE);
+	SetTargetTypeName(query_ad, CREDD_ADTYPE);
 
 	MyString line;
-	line.sprintf("%s = TARGET.%s == \"%s\"", ATTR_REQUIREMENTS, ATTR_NAME, m_name);
+	line.formatstr("%s = TARGET.%s == \"%s\"", ATTR_REQUIREMENTS, ATTR_NAME, m_name);
     query_ad.Insert(line.Value());
 	query_ad.Assign(ATTR_NAME,m_name);
 
