@@ -74,8 +74,7 @@ int main(int argc, char **argv)
 				break;
 			case 'd':
 					// dprintf to console
-				Termlog = 1;
-				dprintf_config ("TOOL", get_param_functions());
+				dprintf_set_tool_debug("TOOL", 0);
 				break;
 			case 'n':
 				if( !(--argc) || !(*(++ptr)) ) {
@@ -116,7 +115,7 @@ int main(int argc, char **argv)
 								number,
 								errorstack)) {
 		fprintf (stderr, "Unable to retrieve credentials (%s)\n",
-				 errorstack.getFullText(true));
+				 errorstack.getFullText(true).c_str());
 		return 1;
 	}
 	 
