@@ -55,6 +55,7 @@ Daemon::common_init() {
 	_is_configured = true;
 	_addr = NULL;
 	_name = NULL;
+	_alias = NULL;
 	_pool = NULL;
 	_version = NULL;
 	_platform = NULL;
@@ -200,6 +201,7 @@ Daemon::deepCopy( const Daemon &copy )
 		// which is exactly what we want everywhere in this method.
 
 	New_name( strnewp(copy._name) );
+	New_alias( strnewp(copy._alias) );
 	New_hostname( strnewp(copy._hostname) );
 	New_full_hostname( strnewp(copy._full_hostname) );
 	New_addr( strnewp(copy._addr) );
@@ -250,12 +252,13 @@ Daemon::deepCopy( const Daemon &copy )
 
 Daemon::~Daemon() 
 {
-	if( DebugFlags & D_HOSTNAME ) {
+	if( IsDebugLevel( D_HOSTNAME ) ) {
 		dprintf( D_HOSTNAME, "Destroying Daemon object:\n" );
 		display( D_HOSTNAME );
 		dprintf( D_HOSTNAME, " --- End of Daemon object info ---\n" );
 	}
 	if( _name ) delete [] _name;
+	if( _alias ) delete [] _alias;
 	if( _pool ) delete [] _pool;
 	if( _addr ) delete [] _addr;
 	if( _error ) delete [] _error;
@@ -370,26 +373,26 @@ Daemon::idStr( void )
 	} else {
 		dt_str = daemonString(_type);
 	}
-	MyString buf;
+	std::string buf;
 	if( _is_local ) {
 		ASSERT( dt_str );
-		buf.sprintf( "local %s", dt_str );
+		formatstr( buf, "local %s", dt_str );
 	} else if( _name ) {
 		ASSERT( dt_str );
-		buf.sprintf( "%s %s", dt_str, _name );
+		formatstr( buf, "%s %s", dt_str, _name );
 	} else if( _addr ) {
 		ASSERT( dt_str );
 		Sinful sinful(_addr);
 		sinful.clearParams(); // too much info is ugly
-		buf.sprintf( "%s at %s", dt_str,
+		formatstr( buf, "%s at %s", dt_str,
 					 sinful.getSinful() ? sinful.getSinful() : _addr );
 		if( _full_hostname ) {
-			buf.sprintf_cat( " (%s)", _full_hostname );
+			formatstr_cat( buf, " (%s)", _full_hostname );
 		}
 	} else {
 		return "unknown daemon";
 	}
-	_id_str = strnewp( buf.Value() );
+	_id_str = strnewp( buf.c_str() );
 	return _id_str;
 }
 
@@ -533,7 +536,7 @@ Daemon::connectSock(Sock *sock, int sec, CondorError* errstack, bool non_blockin
 
 
 StartCommandResult
-Daemon::startCommand( int cmd, Sock* sock, int timeout, CondorError *errstack, StartCommandCallbackType *callback_fn, void *misc_data, bool nonblocking, char const *cmd_description, char *, SecMan *sec_man, bool raw_protocol, char const *sec_session_id )
+Daemon::startCommand( int cmd, Sock* sock, int timeout, CondorError *errstack, int subcmd, StartCommandCallbackType *callback_fn, void *misc_data, bool nonblocking, char const *cmd_description, char *, SecMan *sec_man, bool raw_protocol, char const *sec_session_id )
 {
 	// This function may be either blocking or non-blocking, depending
 	// on the flag that is passed in.  All versions of Daemon::startCommand()
@@ -555,7 +558,7 @@ Daemon::startCommand( int cmd, Sock* sock, int timeout, CondorError *errstack, S
 		sock->timeout( timeout );
 	}
 
-	start_command_result = sec_man->startCommand(cmd, sock, raw_protocol, errstack, 0, callback_fn, misc_data, nonblocking, cmd_description, sec_session_id);
+	start_command_result = sec_man->startCommand(cmd, sock, raw_protocol, errstack, subcmd, callback_fn, misc_data, nonblocking, cmd_description, sec_session_id);
 
 	if(callback_fn) {
 		// SecMan::startCommand() called the callback function, so we just return here
@@ -586,7 +589,7 @@ Daemon::makeConnectedSocket( Stream::stream_type st,
 }
 
 StartCommandResult
-Daemon::startCommand( int cmd, Stream::stream_type st,Sock **sock,int timeout, CondorError *errstack, StartCommandCallbackType *callback_fn, void *misc_data, bool nonblocking, char const *cmd_description, bool raw_protocol, char const *sec_session_id )
+Daemon::startCommand( int cmd, Stream::stream_type st,Sock **sock,int timeout, CondorError *errstack, int subcmd, StartCommandCallbackType *callback_fn, void *misc_data, bool nonblocking, char const *cmd_description, bool raw_protocol, char const *sec_session_id )
 {
 	// This function may be either blocking or non-blocking, depending on
 	// the flag that was passed in.
@@ -612,6 +615,7 @@ Daemon::startCommand( int cmd, Stream::stream_type st,Sock **sock,int timeout, C
 						 *sock,
 						 timeout,
 						 errstack,
+						 subcmd,
 						 callback_fn,
 						 misc_data,
 						 nonblocking,
@@ -622,13 +626,60 @@ Daemon::startCommand( int cmd, Stream::stream_type st,Sock **sock,int timeout, C
 						 sec_session_id);
 }
 
+
+bool
+Daemon::startSubCommand( int cmd, int subcmd, Sock* sock, int timeout, CondorError *errstack, char const *cmd_description,bool raw_protocol, char const *sec_session_id )
+{
+	// This is a blocking version of startCommand().
+	const bool nonblocking = false;
+	StartCommandResult rc = startCommand(cmd,sock,timeout,errstack,subcmd,NULL,NULL,nonblocking,cmd_description,_version,&_sec_man,raw_protocol,sec_session_id);
+	switch(rc) {
+	case StartCommandSucceeded:
+		return true;
+	case StartCommandFailed:
+		return false;
+	case StartCommandInProgress:
+	case StartCommandWouldBlock: //impossible!
+	case StartCommandContinue: //impossible!
+		break;
+	}
+	EXCEPT("startCommand(nonblocking=false) returned an unexpected result: %d\n",rc);
+	return false;
+}
+
+
+Sock*
+Daemon::startSubCommand( int cmd, int subcmd, Stream::stream_type st, int timeout, CondorError* errstack, char const *cmd_description, bool raw_protocol, char const *sec_session_id )
+{
+	// This is a blocking version of startCommand.
+	const bool nonblocking = false;
+	Sock *sock = NULL;
+	StartCommandResult rc = startCommand(cmd,st,&sock,timeout,errstack,subcmd,NULL,NULL,nonblocking,cmd_description,raw_protocol,sec_session_id);
+	switch(rc) {
+	case StartCommandSucceeded:
+		return sock;
+	case StartCommandFailed:
+		if(sock) {
+			delete sock;
+		}
+		return NULL;
+	case StartCommandInProgress:
+	case StartCommandWouldBlock: //impossible!
+	case StartCommandContinue: //impossible!
+		break;
+	}
+	EXCEPT("startCommand(blocking=true) returned an unexpected result: %d\n",rc);
+	return NULL;
+}
+
+
 Sock*
 Daemon::startCommand( int cmd, Stream::stream_type st, int timeout, CondorError* errstack, char const *cmd_description, bool raw_protocol, char const *sec_session_id )
 {
 	// This is a blocking version of startCommand.
 	const bool nonblocking = false;
 	Sock *sock = NULL;
-	StartCommandResult rc = startCommand(cmd,st,&sock,timeout,errstack,NULL,NULL,nonblocking,cmd_description,raw_protocol,sec_session_id);
+	StartCommandResult rc = startCommand(cmd,st,&sock,timeout,errstack,0,NULL,NULL,nonblocking,cmd_description,raw_protocol,sec_session_id);
 	switch(rc) {
 	case StartCommandSucceeded:
 		return sock;
@@ -654,7 +705,7 @@ Daemon::startCommand_nonblocking( int cmd, Stream::stream_type st, int timeout,
 	Sock *sock = NULL;
 	// We require that callback_fn be non-NULL. The startCommand() we call
 	// here does that check.
-	return startCommand(cmd,st,&sock,timeout,errstack,callback_fn,misc_data,nonblocking,cmd_description,raw_protocol,sec_session_id);
+	return startCommand(cmd,st,&sock,timeout,errstack,0,callback_fn,misc_data,nonblocking,cmd_description,raw_protocol,sec_session_id);
 }
 
 StartCommandResult
@@ -662,7 +713,7 @@ Daemon::startCommand_nonblocking( int cmd, Sock* sock, int timeout, CondorError
 {
 	// This is the nonblocking version of startCommand().
 	const bool nonblocking = true;
-	return startCommand(cmd,sock,timeout,errstack,callback_fn,misc_data,nonblocking,cmd_description,_version,&_sec_man,raw_protocol,sec_session_id);
+	return startCommand(cmd,sock,timeout,errstack,0,callback_fn,misc_data,nonblocking,cmd_description,_version,&_sec_man,raw_protocol,sec_session_id);
 }
 
 bool
@@ -670,7 +721,7 @@ Daemon::startCommand( int cmd, Sock* sock, int timeout, CondorError *errstack, c
 {
 	// This is a blocking version of startCommand().
 	const bool nonblocking = false;
-	StartCommandResult rc = startCommand(cmd,sock,timeout,errstack,NULL,NULL,nonblocking,cmd_description,_version,&_sec_man,raw_protocol,sec_session_id);
+	StartCommandResult rc = startCommand(cmd,sock,timeout,errstack,0,NULL,NULL,nonblocking,cmd_description,_version,&_sec_man,raw_protocol,sec_session_id);
 	switch(rc) {
 	case StartCommandSucceeded:
 		return true;
@@ -693,10 +744,10 @@ Daemon::sendCommand( int cmd, Sock* sock, int sec, CondorError* errstack, char c
 		return false;
 	}
 	if( ! sock->end_of_message() ) {
-		MyString err_buf;
-		err_buf.sprintf( "Can't send eom for %d to %s", cmd,  
+		std::string err_buf;
+		formatstr( err_buf, "Can't send eom for %d to %s", cmd,  
 				 idStr() );
-		newError( CA_COMMUNICATION_ERROR, err_buf.Value() );
+		newError( CA_COMMUNICATION_ERROR, err_buf.c_str() );
 		return false;
 	}
 	return true;
@@ -711,10 +762,10 @@ Daemon::sendCommand( int cmd, Stream::stream_type st, int sec, CondorError* errs
 		return false;
 	}
 	if( ! tmp->end_of_message() ) {
-		MyString err_buf;
-		err_buf.sprintf( "Can't send eom for %d to %s", cmd,  
+		std::string err_buf;
+		formatstr( err_buf, "Can't send eom for %d to %s", cmd,  
 				 idStr() );
-		newError( CA_COMMUNICATION_ERROR, err_buf.Value() );
+		newError( CA_COMMUNICATION_ERROR, err_buf.c_str() );
 		delete tmp;
 		return false;
 	}
@@ -756,19 +807,19 @@ Daemon::sendCACmd( ClassAd* req, ClassAd* reply, ReliSock* cmd_sock,
 		return false;
 	}
 	
-	req->SetMyTypeName( COMMAND_ADTYPE );
-	req->SetTargetTypeName( REPLY_ADTYPE );
+	SetMyTypeName( *req, COMMAND_ADTYPE );
+	SetTargetTypeName( *req, REPLY_ADTYPE );
 
 	if( timeout >= 0 ) {
 		cmd_sock->timeout( timeout );
 	}
 
 	if( ! connectSock(cmd_sock) ) {
-		MyString err_msg = "Failed to connect to ";
+		std::string err_msg = "Failed to connect to ";
 		err_msg += daemonString(_type);
 		err_msg += " ";
 		err_msg += _addr;
-		newError( CA_CONNECT_FAILED, err_msg.Value() );
+		newError( CA_CONNECT_FAILED, err_msg.c_str() );
 		return false;
 	}
 
@@ -780,21 +831,21 @@ Daemon::sendCACmd( ClassAd* req, ClassAd* reply, ReliSock* cmd_sock,
 	}
 	CondorError errstack;
 	if( ! startCommand(cmd, cmd_sock, 20, &errstack, NULL, false, sec_session_id) ) {
-		MyString err_msg = "Failed to send command (";
+		std::string err_msg = "Failed to send command (";
 		if( cmd == CA_CMD ) {
 			err_msg += "CA_CMD";
 		} else {
 			err_msg += "CA_AUTH_CMD";
 		}
 		err_msg += "): ";
-		err_msg += errstack.getFullText();
-		newError( CA_COMMUNICATION_ERROR, err_msg.Value() );
+		err_msg += errstack.getFullText().c_str();
+		newError( CA_COMMUNICATION_ERROR, err_msg.c_str() );
 		return false;
 	}
 	if( force_auth ) {
 		CondorError e;
 		if( ! forceAuthentication(cmd_sock, &e) ) {
-			newError( CA_NOT_AUTHENTICATED, e.getFullText() );
+			newError( CA_NOT_AUTHENTICATED, e.getFullText().c_str() );
 			return false;
 		}
 	}
@@ -806,7 +857,7 @@ Daemon::sendCACmd( ClassAd* req, ClassAd* reply, ReliSock* cmd_sock,
 		cmd_sock->timeout( timeout );
 	}
 
-	if( ! req->put(*cmd_sock) ) { 
+	if( ! putClassAd(cmd_sock, *req) ) {
 		newError( CA_COMMUNICATION_ERROR,
 				  "Failed to send request ClassAd" );
 		return false;
@@ -819,7 +870,7 @@ Daemon::sendCACmd( ClassAd* req, ClassAd* reply, ReliSock* cmd_sock,
 
 		// Now, try to get the reply
 	cmd_sock->decode();
-	if( ! reply->initFromStream(*cmd_sock) ) {
+	if( ! getClassAd(cmd_sock, *reply) ) {
 		newError( CA_COMMUNICATION_ERROR, "Failed to read reply ClassAd" );
 		return false;
 	}
@@ -831,10 +882,10 @@ Daemon::sendCACmd( ClassAd* req, ClassAd* reply, ReliSock* cmd_sock,
 		// Finally, interpret the results
 	char* result_str = NULL;
 	if( ! reply->LookupString(ATTR_RESULT, &result_str) ) {
-		MyString err_msg = "Reply ClassAd does not have ";
+		std::string err_msg = "Reply ClassAd does not have ";
 		err_msg += ATTR_RESULT;
 		err_msg += " attribute";
-		newError( CA_INVALID_REPLY, err_msg.Value() );
+		newError( CA_INVALID_REPLY, err_msg.c_str() );
 		return false;
 	}
 	CAResult result = getCAResultNum( result_str );
@@ -858,12 +909,12 @@ Daemon::sendCACmd( ClassAd* req, ClassAd* reply, ReliSock* cmd_sock,
 		}
 			// otherwise, it's a known failure, but there's no error
 			// string to help us...
-		MyString err_msg = "Reply ClassAd returned '";
+		std::string err_msg = "Reply ClassAd returned '";
 		err_msg += result_str;
 		err_msg += "' but does not have the ";
 		err_msg += ATTR_ERROR_STRING;
 		err_msg += " attribute";
-		newError( result, err_msg.Value() );
+		newError( result, err_msg.c_str() );
 		free( result_str );
 		return false;
 	}
@@ -1030,7 +1081,7 @@ Daemon::setSubsystem( const char* subsys )
 bool
 Daemon::getDaemonInfo( AdTypes adtype, bool query_collector )
 {
-	MyString			buf;
+	std::string			buf;
 	char				*tmp, *my_name;
 	char				*host = NULL;
 	bool				nameHasPort = false;
@@ -1049,14 +1100,14 @@ Daemon::getDaemonInfo( AdTypes adtype, bool query_collector )
 		// If we were not passed a name or an addr, check the
 		// config file for a subsystem_HOST, e.g. SCHEDD_HOST=XXXX
 	if( ! _name  && !_pool ) {
-		buf.sprintf( "%s_HOST", _subsys );
-		char *specified_host = param( buf.Value() );
+		formatstr( buf, "%s_HOST", _subsys );
+		char *specified_host = param( buf.c_str() );
 		if ( specified_host ) {
 				// Found an entry.  Use this name.
 			_name = strnewp( specified_host );
 			dprintf( D_HOSTNAME, 
 					 "No name given, but %s defined to \"%s\"\n",
-					 buf.Value(), specified_host );
+					 buf.c_str(), specified_host );
 			free(specified_host);
 		}
 	}
@@ -1084,7 +1135,7 @@ Daemon::getDaemonInfo( AdTypes adtype, bool query_collector )
 
 		if(host && hostaddr.from_ip_string(host) ) {
 			buf = generate_sinful(host, _port);
-			New_addr( strnewp(buf.Value()) );
+			New_addr( strnewp(buf.c_str()) );
 			dprintf( D_HOSTNAME,
 					"Host info \"%s\" is an IP address\n", host );
 		} else {
@@ -1095,8 +1146,8 @@ Daemon::getDaemonInfo( AdTypes adtype, bool query_collector )
 						 "finding IP address\n", host );
 				if (!get_fqdn_and_ip_from_hostname(host, fqdn, hostaddr)) {
 					// With a hostname, this is a fatal Daemon error.
-					buf.sprintf( "unknown host %s", host );
-					newError( CA_LOCATE_FAILED, buf.Value() );
+					formatstr( buf, "unknown host %s", host );
+					newError( CA_LOCATE_FAILED, buf.c_str() );
 					if (host) free( host );
 
 						// We assume this is a transient DNS failure.  Therefore,
@@ -1108,10 +1159,13 @@ Daemon::getDaemonInfo( AdTypes adtype, bool query_collector )
 				}
 			} else return false;
 			buf = generate_sinful(hostaddr.to_ip_string().Value(), _port);
-			dprintf( D_HOSTNAME, "Found IP address and port %s\n", buf.Value() );
-			New_addr( strnewp(buf.Value()) );
+			dprintf( D_HOSTNAME, "Found IP address and port %s\n", buf.c_str() );
 			if (fqdn.Length() > 0)
 				New_full_hostname(strnewp(fqdn.Value()));
+			if( host ) {
+				New_alias( strnewp(host) );
+			}
+			New_addr( strnewp(buf.c_str()) );
 		}
 
 		if (host) free( host );
@@ -1129,14 +1183,15 @@ Daemon::getDaemonInfo( AdTypes adtype, bool query_collector )
 				// we failed to contruct the daemon name.  the only
 				// possible reason for this is being given faulty
 				// hostname.  This is a fatal error.
-			MyString err_msg = "unknown host ";
+			std::string err_msg = "unknown host ";
 			err_msg += get_host_part( _name );
-			newError( CA_LOCATE_FAILED, err_msg.Value() );
+			newError( CA_LOCATE_FAILED, err_msg.c_str() );
 			return false;
 		}
-			// if it worked, we've not got the proper values for the
+			// if it worked, we've now got the proper values for the
 			// name (and the full hostname, since that's just the
 			// "host part" of the "name"...
+		New_alias( strnewp(get_host_part( _name )) );
 		New_name( tmp );
 		dprintf( D_HOSTNAME, "Using \"%s\" for name in Daemon object\n",
 				 tmp );
@@ -1222,13 +1277,13 @@ Daemon::getDaemonInfo( AdTypes adtype, bool query_collector )
 				  machine all reporting to the same collector.
 				  -Derek Wright 2005-03-09
 				*/
-			buf.sprintf( "%s == \"%s\"", ATTR_MACHINE, _full_hostname ); 
-			query.addANDConstraint( buf.Value() );
+			formatstr( buf, "%s == \"%s\"", ATTR_MACHINE, _full_hostname ); 
+			query.addANDConstraint( buf.c_str() );
 		} else if ( _type == DT_GENERIC ) {
 			query.setGenericQueryType(_subsys);
 		} else if ( _name ) {
-			buf.sprintf( "%s == \"%s\"", ATTR_NAME, _name ); 
-			query.addANDConstraint( buf.Value() );
+			formatstr( buf, "%s == \"%s\"", ATTR_NAME, _name ); 
+			query.addANDConstraint( buf.c_str() );
 		} else {
 			if ( ( _type != DT_NEGOTIATOR ) && ( _type != DT_LEASE_MANAGER) ) {
 					// If we're not querying for negotiator
@@ -1245,7 +1300,7 @@ Daemon::getDaemonInfo( AdTypes adtype, bool query_collector )
 		CondorError errstack;
 		if (collectors->query (query, ads) != Q_OK) {
 			delete collectors;
-			newError( CA_LOCATE_FAILED, errstack.getFullText() );
+			newError( CA_LOCATE_FAILED, errstack.getFullText().c_str() );
 			return false;
 		};
 		delete collectors;
@@ -1255,9 +1310,9 @@ Daemon::getDaemonInfo( AdTypes adtype, bool query_collector )
 		if(!scan) {
 			dprintf( D_ALWAYS, "Can't find address for %s %s\n",
 					 daemonString(_type), _name ? _name : "" );
-			buf.sprintf( "Can't find address for %s %s", 
+			formatstr( buf, "Can't find address for %s %s", 
 						 daemonString(_type), _name ? _name : "" );
-			newError( CA_LOCATE_FAILED, buf.Value() );
+			newError( CA_LOCATE_FAILED, buf.c_str() );
 			return false; 
 		}
 
@@ -1288,7 +1343,7 @@ Daemon::getDaemonInfo( AdTypes adtype, bool query_collector )
 bool
 Daemon::getCmInfo( const char* subsys )
 {
-	MyString buf;
+	std::string buf;
 	char* host = NULL;
 
 	setSubsystem( subsys );
@@ -1339,9 +1394,9 @@ Daemon::getCmInfo( const char* subsys )
 			// this is just a fancy wrapper for param()...
 		char *hostnames = getCmHostFromConfig( subsys );
 		if(!hostnames) {
-			buf.sprintf("%s address or hostname not specified in config file",
+			formatstr( buf, "%s address or hostname not specified in config file",
 					 subsys ); 
-			newError( CA_LOCATE_FAILED, buf.Value() );
+			newError( CA_LOCATE_FAILED, buf.c_str() );
 			_is_configured = false;
 			return false;
 		}
@@ -1368,9 +1423,9 @@ Daemon::getCmInfo( const char* subsys )
 	}
 
 	if( ! host || !host[0]) {
-		buf.sprintf("%s address or hostname not specified in config file",
+		formatstr( buf, "%s address or hostname not specified in config file",
 				 subsys ); 
-		newError( CA_LOCATE_FAILED, buf.Value() );
+		newError( CA_LOCATE_FAILED, buf.c_str() );
 		_is_configured = false;
 		if( host ) free( host );
 
@@ -1387,7 +1442,7 @@ bool
 Daemon::findCmDaemon( const char* cm_name )
 {
 	char* host = NULL;
-	MyString buf;
+	std::string buf;
 	condor_sockaddr saddr;
 
 	dprintf( D_HOSTNAME, "Using name \"%s\" to find daemon\n", cm_name ); 
@@ -1396,9 +1451,9 @@ Daemon::findCmDaemon( const char* cm_name )
 
 	if( !sinful.valid() || !sinful.getHost() ) {
 		dprintf( D_ALWAYS, "Invalid address: %s\n", cm_name );
-		buf.sprintf( "%s address or hostname not specified in config file",
+		formatstr( buf, "%s address or hostname not specified in config file",
 				 _subsys ); 
-		newError( CA_LOCATE_FAILED, buf.Value() );
+		newError( CA_LOCATE_FAILED, buf.c_str() );
 		_is_configured = false;
 		return false;
 	}
@@ -1441,9 +1496,9 @@ Daemon::findCmDaemon( const char* cm_name )
 
 
 	if ( !host ) {
-		buf.sprintf( "%s address or hostname not specified in config file",
+		formatstr( buf, "%s address or hostname not specified in config file",
 				 _subsys ); 
-		newError( CA_LOCATE_FAILED, buf.Value() );
+		newError( CA_LOCATE_FAILED, buf.c_str() );
 		_is_configured = false;
 		return false;
 	}
@@ -1461,8 +1516,8 @@ Daemon::findCmDaemon( const char* cm_name )
 		int ret = get_fqdn_and_ip_from_hostname(host, fqdn, saddr);
 		if (!ret) {
 				// With a hostname, this is a fatal Daemon error.
-			buf.sprintf( "unknown host %s", host );
-			newError( CA_LOCATE_FAILED, buf.Value() );
+			formatstr( buf, "unknown host %s", host );
+			newError( CA_LOCATE_FAILED, buf.c_str() );
 			free( host );
 
 				// We assume this is a transient DNS failure.  Therefore,
@@ -1475,8 +1530,11 @@ Daemon::findCmDaemon( const char* cm_name )
 		sinful.setHost(saddr.to_ip_string().Value());
 		dprintf( D_HOSTNAME, "Found IP address and port %s\n",
 				 sinful.getSinful() ? sinful.getSinful() : "NULL" );
-		New_addr( strnewp( sinful.getSinful() ) );
 		New_full_hostname(strnewp(fqdn.Value()));
+		if( host ) {
+			New_alias( strnewp(host) );
+		}
+		New_addr( strnewp( sinful.getSinful() ) );
 	}
 
 		// If the pool was set, we want to use _name for that, too. 
@@ -1538,9 +1596,9 @@ Daemon::initHostname( void )
 		New_full_hostname( NULL );
 		dprintf(D_HOSTNAME, "get_full_hostname() failed for address %s",
 				saddr.to_ip_string().Value());
-		MyString err_msg = "can't find host info for ";
+		std::string err_msg = "can't find host info for ";
 		err_msg += _addr;
-		newError( CA_LOCATE_FAILED, err_msg.Value() );
+		newError( CA_LOCATE_FAILED, err_msg.c_str() );
 		return false;
 	}
 
@@ -1683,18 +1741,18 @@ Daemon::readAddressFile( const char* subsys )
 {
 	char* addr_file;
 	FILE* addr_fp;
-	MyString param_name;
+	std::string param_name;
 	MyString buf;
 	bool rval = false;
 
-	param_name.sprintf( "%s_ADDRESS_FILE", subsys );
-	addr_file = param( param_name.Value() );
+	formatstr( param_name, "%s_ADDRESS_FILE", subsys );
+	addr_file = param( param_name.c_str() );
 	if( ! addr_file ) {
 		return false;
 	}
 
 	dprintf( D_HOSTNAME, "Finding address for local daemon, "
-			 "%s is \"%s\"\n", param_name.Value(), addr_file );
+			 "%s is \"%s\"\n", param_name.c_str(), addr_file );
 
 	if( ! (addr_fp = safe_fopen_wrapper_follow(addr_file, "r")) ) {
 		dprintf( D_HOSTNAME,
@@ -1749,17 +1807,16 @@ Daemon::readLocalClassAd( const char* subsys )
 	char* addr_file;
 	FILE* addr_fp;
 	ClassAd *adFromFile;
-	MyString param_name;
-	MyString buf;
+	std::string param_name;
 
-	param_name.sprintf( "%s_DAEMON_AD_FILE", subsys );
-	addr_file = param( param_name.Value() );
+	formatstr( param_name, "%s_DAEMON_AD_FILE", subsys );
+	addr_file = param( param_name.c_str() );
 	if( ! addr_file ) {
 		return false;
 	}
 
 	dprintf( D_HOSTNAME, "Finding classad for local daemon, "
-			 "%s is \"%s\"\n", param_name.Value(), addr_file );
+			 "%s is \"%s\"\n", param_name.c_str(), addr_file );
 
 	if( ! (addr_fp = safe_fopen_wrapper_follow(addr_file, "r")) ) {
 		dprintf( D_HOSTNAME,
@@ -1802,9 +1859,9 @@ Daemon::hasUDPCommandPort()
 bool 
 Daemon::getInfoFromAd( const ClassAd* ad )
 {
-	MyString buf = "";
-	MyString buf2 = "";
-	MyString addr_attr_name = "";
+	std::string buf = "";
+	std::string buf2 = "";
+	std::string addr_attr_name = "";
 		// TODO Which attributes should trigger a failure if we don't find
 		// them in the ad? Just _addr?
 	bool ret_val = true;
@@ -1815,28 +1872,28 @@ Daemon::getInfoFromAd( const ClassAd* ad )
 	initStringFromAd( ad, ATTR_NAME, &_name );
 
 		// construct the IP_ADDR attribute
-	buf.sprintf( "%sIpAddr", _subsys );
-	if ( ad->LookupString( buf.Value(), buf2 ) ) {
-		New_addr( strnewp( buf2.Value() ) );
+	formatstr( buf, "%sIpAddr", _subsys );
+	if ( ad->LookupString( buf.c_str(), buf2 ) ) {
+		New_addr( strnewp( buf2.c_str() ) );
 		found_addr = true;
 		addr_attr_name = buf;
 	}
 	else if ( ad->LookupString( ATTR_MY_ADDRESS, buf2 ) ) {
-		New_addr( strnewp( buf2.Value() ) );
+		New_addr( strnewp( buf2.c_str() ) );
 		found_addr = true;
 		addr_attr_name = ATTR_MY_ADDRESS;
 	}
 
 	if ( found_addr ) {
 		dprintf( D_HOSTNAME, "Found %s in ClassAd, using \"%s\"\n",
-				 addr_attr_name.Value(), _addr);
+				 addr_attr_name.c_str(), _addr);
 		_tried_locate = true;
 	} else {
 		dprintf( D_ALWAYS, "Can't find address in classad for %s %s\n",
 				 daemonString(_type), _name ? _name : "" );
-		buf.sprintf( "Can't find address in classad for %s %s",
+		formatstr( buf, "Can't find address in classad for %s %s",
 					 daemonString(_type), _name ? _name : "" );
-		newError( CA_LOCATE_FAILED, buf.Value() );
+		newError( CA_LOCATE_FAILED, buf.c_str() );
 
 		ret_val = false;
 	}
@@ -1874,15 +1931,15 @@ Daemon::initStringFromAd( const ClassAd* ad, const char* attrname, char** value
 		EXCEPT( "Daemon::initStringFromAd() called with NULL value!" );
 	}
 	char* tmp = NULL;
-	MyString buf;
 	if( ! ad->LookupString(attrname, &tmp) ) {
+		std::string buf;
 		dprintf( D_ALWAYS, "Can't find %s in classad for %s %s\n",
 				 attrname, daemonString(_type),
 				 _name ? _name : "" );
-		buf.sprintf( "Can't find %s in classad for %s %s",
+		formatstr( buf, "Can't find %s in classad for %s %s",
 					 attrname, daemonString(_type),
 					 _name ? _name : "" );
-		newError( CA_LOCATE_FAILED, buf.Value() );
+		newError( CA_LOCATE_FAILED, buf.c_str() );
 		return false;
 	}
 	if( *value ) {
@@ -1946,13 +2003,13 @@ Daemon::New_addr( char* str )
 					using_private = true;
 					if( priv_addr ) {
 						// replace address with private address
-						MyString buf;
+						std::string buf;
 						if( *priv_addr != '<' ) {
 								// [TODO]
 								// if priv address is an IPv6 address,
 								// it should be <[%s]> form
-							buf.sprintf("<%s>",priv_addr);
-							priv_addr = buf.Value();
+							formatstr(buf,"<%s>",priv_addr);
+							priv_addr = buf.c_str();
 						}
 						delete [] _addr;
 						_addr = strnewp( priv_addr );
@@ -1992,8 +2049,31 @@ Daemon::New_addr( char* str )
 			// This address explicitly specifies that UDP is not supported
 			m_has_udp_command_port = false;
 		}
+		if( !sinful.getAlias() && _alias ) {
+			size_t len = strlen(_alias);
+				// If _alias is not equivalent to the canonical hostname,
+				// then stash it in the sinful address.  This is important
+				// in cases where we later verify that the certificate
+				// presented by the host we are connecting to matches
+				// the hostname we requested.
+			if( !_full_hostname || (strcmp(_alias,_full_hostname)!=0 && (strncmp(_alias,_full_hostname,len)!=0 || _full_hostname[len]!='.')) )
+			{
+				sinful.setAlias(_alias);
+				delete [] _addr;
+				_addr = strnewp( sinful.getSinful() );
+			}
+		}
 	}
 
+	if( _addr ) {
+		dprintf( D_HOSTNAME, "Daemon client (%s) address determined: "
+				 "name: \"%s\", pool: \"%s\", alias: \"%s\", addr: \"%s\"\n",
+				 daemonString(_type),
+				 _name ? _name : "NULL",
+				 _pool ? _pool : "NULL",
+				 _alias ? _alias : "NULL",
+				 _addr ? _addr : "NULL" );
+	}
 	return;
 }
 
@@ -2028,6 +2108,15 @@ Daemon::New_name( char* str )
 	return str;
 }
 
+const char*
+Daemon::New_alias( char *str )
+{
+	if( _alias ) {
+		delete [] _alias;
+	}
+	_alias = str;
+	return str;
+}
 
 char*
 Daemon::New_pool( char* str )
@@ -2118,18 +2207,18 @@ Daemon::setCmdStr( const char* cmd )
 char*
 getCmHostFromConfig( const char * subsys )
 { 
-	MyString buf;
+	std::string buf;
 	char* host = NULL;
 
 		// Try the config file for a subsys-specific hostname 
-	buf.sprintf( "%s_HOST", subsys );
-	host = param( buf.Value() );
+	formatstr( buf, "%s_HOST", subsys );
+	host = param( buf.c_str() );
 	if( host ) {
 		if( host[0] ) {
-			dprintf( D_HOSTNAME, "%s is set to \"%s\"\n", buf.Value(), 
+			dprintf( D_HOSTNAME, "%s is set to \"%s\"\n", buf.c_str(), 
 					 host ); 
 			if(host[0] == ':') {
-				dprintf( D_ALWAYS, "Warning: Configuration file sets '%s=%s'.  This does not look like a valid host name with optional port.\n", buf.Value(), host);
+				dprintf( D_ALWAYS, "Warning: Configuration file sets '%s=%s'.  This does not look like a valid host name with optional port.\n", buf.c_str(), host);
 			}
 			return host;
 		} else {
@@ -2138,11 +2227,11 @@ getCmHostFromConfig( const char * subsys )
 	}
 
 		// Try the config file for a subsys-specific IP addr 
-	buf.sprintf ("%s_IP_ADDR", subsys );
-	host = param( buf.Value() );
+	formatstr( buf, "%s_IP_ADDR", subsys );
+	host = param( buf.c_str() );
 	if( host ) {
 		if( host[0] ) {
-			dprintf( D_HOSTNAME, "%s is set to \"%s\"\n", buf.Value(), host );
+			dprintf( D_HOSTNAME, "%s is set to \"%s\"\n", buf.c_str(), host );
 			return host;
 		} else {
 			free( host );
@@ -2153,7 +2242,7 @@ getCmHostFromConfig( const char * subsys )
 	host = param( "CM_IP_ADDR" );
 	if( host ) {
 		if(  host[0] ) {
-			dprintf( D_HOSTNAME, "%s is set to \"%s\"\n", buf.Value(), 
+			dprintf( D_HOSTNAME, "%s is set to \"%s\"\n", buf.c_str(), 
 					 host ); 
 			return host;
 		} else {
