@@ -230,7 +230,7 @@ CollectorList::resortLocal( const char *preferred_collector )
 		}
 
 		tmp_preferred_collector = strdup(_hostname);
-		preferred_collector = preferred_collector; // So we know to free later
+		preferred_collector = tmp_preferred_collector; // So we know to free later
 	}
 
 
@@ -298,14 +298,11 @@ CollectorList::query(CondorQuery & cQuery, ClassAdList & adList, CondorError *er
 	}
 	
 
-        while ( vCollectors.size() ) 
-        {
-            // choose a random collector in the list to query.
-            unsigned int idx = get_random_int() % vCollectors.size() ;
-            daemon = vCollectors[idx];
+	while ( vCollectors.size() ) {
+		// choose a random collector in the list to query.
+		unsigned int idx = get_random_int() % vCollectors.size() ;
+		daemon = vCollectors[idx];
 
-	    if ( !daemon->isBlacklisted() )
-	    {
 		if ( ! daemon->addr() ) {
 			if ( daemon->name() ) {
 				dprintf( D_ALWAYS,
@@ -316,46 +313,37 @@ CollectorList::query(CondorQuery & cQuery, ClassAdList & adList, CondorError *er
 						 "Can't resolve nameless collector; skipping\n" );
 			}
 			problems_resolving = true;
-			
+		} else if ( daemon->isBlacklisted() ) {
+			dprintf( D_ALWAYS,"Collector %s blacklisted; skipping\n",
+					 daemon->name() );
+		} else {
+			dprintf (D_FULLDEBUG,
+					 "Trying to query collector %s\n",
+					 daemon->addr());
+
+			if( num_collectors > 1 ) {
+				daemon->blacklistMonitorQueryStarted();
+			}
+
+			result = cQuery.fetchAds (adList, daemon->addr(), errstack);
+
+			if( num_collectors > 1 ) {
+				daemon->blacklistMonitorQueryFinished( result == Q_OK );
+			}
+
+			if (result == Q_OK) {
+				return result;
+			}
 		}
-		else
-                {
 
-                    dprintf (D_FULLDEBUG, 
-                                    "Trying to query collector %s\n", 
-                                    daemon->addr());
-
-                    if( num_collectors > 1 ) {
-                            daemon->blacklistMonitorQueryStarted();
-                    }
-
-                    result = 
-                            cQuery.fetchAds (adList, daemon->addr(), errstack);
-
-                    if( num_collectors > 1 ) {
-                            daemon->blacklistMonitorQueryFinished( result == Q_OK );
-                    }
-
-                    if (result == Q_OK) {
-                            return result;
-                    }
-                }
-	    }
-	    else
-	    {
-	      dprintf( D_ALWAYS,"Collector %s blacklisted; skipping\n", daemon->name() );
-	    }
-
-	    // if you got here remove it from the list of potential candidates.
-	    vCollectors.erase( vCollectors.begin()+idx );
+		// if you got here remove it from the list of potential candidates.
+		vCollectors.erase( vCollectors.begin()+idx );
 	}
-			
+
 	// only push an error if the error stack exists and is currently empty
 	if(problems_resolving && errstack && !errstack->code(0)) {
-		MyString errmsg;
 		char* tmplist = getCmHostFromConfig( "COLLECTOR" );
-		errmsg.sprintf ("Unable to resolve COLLECTOR_HOST (%s).",tmplist?tmplist:"(null)");
-		errstack->push("CONDOR_STATUS",1,errmsg.Value());
+		errstack->pushf("CONDOR_STATUS",1,"Unable to resolve COLLECTOR_HOST (%s).",tmplist?tmplist:"(null)");
 	}
 
 		// If we've gotten here, there are no good collectors
