@@ -45,6 +45,8 @@ static int ZZZ_always_increase() {
 	return ZZZZZ++;
 }
 
+const std::string DaemonCommandProtocol::WaitForSocketDataString = "DaemonCommandProtocol::WaitForSocketData";
+
 DaemonCommandProtocol::DaemonCommandProtocol(Stream *sock,bool is_command_sock):
 #ifdef HAVE_EXT_GSOAP
 	m_is_http_post(false),
@@ -175,7 +177,7 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::WaitForSocke
 		m_sock,
 		m_sock->peer_description(),
 		(SocketHandlercpp)&DaemonCommandProtocol::SocketCallback,
-		"DaemonCommandProtocol::WaitForSocketData",
+		WaitForSocketDataString.c_str(),
 		this,
 		ALLOW);
 
@@ -570,7 +572,7 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ReadCommand(
 
 		dprintf (D_SECURITY, "DC_AUTHENTICATE: received DC_AUTHENTICATE from %s\n", m_sock->peer_description());
 
-		if( !m_auth_info.initFromStream(*m_sock)) {
+		if( !getClassAd(m_sock, m_auth_info)) {
 			dprintf (D_ALWAYS, "ERROR: DC_AUTHENTICATE unable to "
 					 "receive auth_info from %s!\n", m_sock->peer_description());
 			m_result = FALSE;
@@ -584,9 +586,9 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ReadCommand(
 			return CommandProtocolFinished;
 		}
 
-		if (DebugFlags & D_FULLDEBUG) {
+		if (IsDebugVerbose(D_SECURITY)) {
 			dprintf (D_SECURITY, "DC_AUTHENTICATE: received following ClassAd:\n");
-			m_auth_info.dPrint (D_SECURITY);
+			dPrintAd (D_SECURITY, m_auth_info);
 		}
 
 		MyString peer_version;
@@ -600,7 +602,7 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ReadCommand(
 		m_auth_cmd = 0;
 		m_auth_info.LookupInteger(ATTR_SEC_COMMAND, m_real_cmd);
 
-		if (m_real_cmd == DC_AUTHENTICATE) {
+		if ((m_real_cmd == DC_AUTHENTICATE) || (m_real_cmd == DC_SEC_QUERY)) {
 			// we'll set m_auth_cmd temporarily to
 			m_auth_info.LookupInteger(ATTR_SEC_AUTH_COMMAND, m_auth_cmd);
 		} else {
@@ -695,7 +697,7 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ReadCommand(
 				} else {
 					// the session->id() and the_sid strings should be identical.
 
-					if (DebugFlags & D_SECURITY) {
+					if (IsDebugLevel(D_SECURITY)) {
 						char *return_addr = NULL;
 						if(session->policy()) {
 							session->policy()->LookupString(ATTR_SEC_SERVER_COMMAND_SOCK,&return_addr);
@@ -718,22 +720,37 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ReadCommand(
 				if (session->policy()) {
 					// copy this to the HandleReq() scope
 					m_policy = new ClassAd(*session->policy());
-					if (DebugFlags & D_FULLDEBUG) {
+					if (IsDebugVerbose(D_SECURITY)) {
 						dprintf (D_SECURITY, "DC_AUTHENTICATE: Cached Session:\n");
-						m_policy->dPrint (D_SECURITY);
+						dPrintAd (D_SECURITY, *m_policy);
 					}
 				}
 
-				// grab the user out of the policy.
+				// grab some attributes out of the policy.
 				if (m_policy) {
-					char *the_user  = NULL;
-					m_policy->LookupString( ATTR_SEC_USER, &the_user);
-					if (the_user) {
+					char *tmp  = NULL;
+					m_policy->LookupString( ATTR_SEC_USER, &tmp);
+					if (tmp) {
 						// copy this to the HandleReq() scope
-						m_user = the_user;
-						free( the_user );
-						the_user = NULL;
+						m_user = tmp;
+						free( tmp );
+						tmp = NULL;
 					}
+					m_policy->LookupString( ATTR_SEC_AUTHENTICATED_NAME, &tmp);
+					if (tmp) {
+						// copy this to the HandleReq() scope
+						m_sock->setAuthenticatedName(tmp);
+						free( tmp );
+						tmp = NULL;
+					}
+					m_policy->LookupString( ATTR_SEC_AUTHENTICATION_METHODS, &tmp);
+					if (tmp) {
+						// copy this to the HandleReq() scope
+						m_sock->setAuthenticationMethodUsed(tmp);
+						free( tmp );
+						tmp = NULL;
+					}
+
 
 					bool tried_authentication=false;
 					m_policy->LookupBool(ATTR_SEC_TRIED_AUTHENTICATION,tried_authentication);
@@ -761,9 +778,9 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ReadCommand(
 					return CommandProtocolFinished;
 				}
 
-				if (DebugFlags & D_FULLDEBUG) {
+				if (IsDebugVerbose(D_SECURITY)) {
 					dprintf ( D_SECURITY, "DC_AUTHENTICATE: our_policy:\n" );
-					our_policy.dPrint(D_SECURITY);
+					dPrintAd(D_SECURITY, our_policy);
 				}
 
 				// reconcile.  if unable, close socket.
@@ -775,9 +792,9 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ReadCommand(
 					m_result = FALSE;
 					return CommandProtocolFinished;
 				} else {
-					if (DebugFlags & D_FULLDEBUG) {
+					if (IsDebugVerbose(D_SECURITY)) {
 						dprintf ( D_SECURITY, "DC_AUTHENTICATE: the_policy:\n" );
-						m_policy->dPrint(D_SECURITY);
+						dPrintAd(D_SECURITY, *m_policy);
 					}
 				}
 
@@ -793,7 +810,7 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ReadCommand(
 
 					// generate a unique ID.
 					MyString tmpStr;
-					tmpStr.sprintf( "%s:%i:%i:%i", 
+					tmpStr.formatstr( "%s:%i:%i:%i", 
 									get_local_hostname().Value(), daemonCore->mypid,
 							 (int)time(0), ZZZ_always_increase() );
 					assert (m_sid == NULL);
@@ -855,15 +872,15 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ReadCommand(
 
 				// if they asked, tell them
 				if (m_is_tcp && (m_sec_man->sec_lookup_feat_act(m_auth_info, ATTR_SEC_ENACT) == SecMan::SEC_FEAT_ACT_NO)) {
-					if (DebugFlags & D_FULLDEBUG) {
+					if (IsDebugVerbose(D_SECURITY)) {
 						dprintf (D_SECURITY, "SECMAN: Sending following response ClassAd:\n");
-						m_policy->dPrint( D_SECURITY );
+						dPrintAd( D_SECURITY, *m_policy );
 					}
 					m_sock->encode();
-					if (!m_policy->put(*m_sock) ||
+					if (!putClassAd(m_sock, *m_policy) ||
 						!m_sock->end_of_message()) {
 						dprintf (D_ALWAYS, "SECMAN: Error sending response classad to %s!\n", m_sock->peer_description());
-						m_auth_info.dPrint (D_ALWAYS);
+						dPrintAd (D_ALWAYS, m_auth_info);
 						m_result = FALSE;
 						return CommandProtocolFinished;
 					}
@@ -916,7 +933,7 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ReadCommand(
 				// 6.6.1 or higher, we will force will_authenticate to
 				// SEC_FEAT_ACT_NO.
 
-				if ((will_authenticate == SecMan::SEC_FEAT_ACT_YES)) {
+				if ( will_authenticate == SecMan::SEC_FEAT_ACT_YES ) {
 					if ((!m_new_session)) {
 						char * remote_version = NULL;
 						m_policy->LookupString(ATTR_SEC_REMOTE_VERSION, &remote_version);
@@ -945,7 +962,7 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ReadCommand(
 
 
 				} else {
-					if (DebugFlags & D_FULLDEBUG) {
+					if (IsDebugVerbose(D_SECURITY)) {
 						dprintf (D_SECURITY, "DC_AUTHENTICATE: not authenticating.\n");
 					}
 					m_state = CommandProtocolPostAuthenticate;
@@ -977,7 +994,7 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::Authenticate
 		return CommandProtocolFinished;
 	}
 
-	if (DebugFlags & D_FULLDEBUG) {
+	if (IsDebugVerbose(D_SECURITY)) {
 		dprintf (D_SECURITY, "DC_AUTHENTICATE: authenticating RIGHT NOW.\n");
 	}
 
@@ -985,17 +1002,30 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::Authenticate
 	if( !daemonCore->CommandNumToTableIndex(m_auth_cmd,&cmd_index) ) {
 		dprintf(D_ALWAYS, "DC_AUTHENTICATE: UNREGISTERED COMMAND %d in Authenticate()\n",m_auth_cmd);
 		m_result = FALSE;
+		free( auth_methods );
 		return CommandProtocolFinished;
 	}
 
 	int auth_timeout = daemonCore->getSecMan()->getSecTimeout( m_comTable[cmd_index].perm );
 
+	m_sock->setAuthenticationMethodsTried(auth_methods);
+
 	char *method_used = NULL;
 	bool auth_success = m_sock->authenticate(m_key, auth_methods, &errstack, auth_timeout, &method_used);
 
 	if ( method_used ) {
 		m_policy->Assign(ATTR_SEC_AUTHENTICATION_METHODS, method_used);
 	}
+	if ( m_sock->getAuthenticatedName() ) {
+		m_policy->Assign(ATTR_SEC_AUTHENTICATED_NAME, m_sock->getAuthenticatedName() );
+	}
+
+	if (!auth_success) {
+		// call the auditing callback to record the authentication failure
+		if (daemonCore->audit_log_callback_fn) {
+			(*(daemonCore->audit_log_callback_fn))( m_auth_cmd, (*m_sock), true );
+		}
+	}
 
 	free( auth_methods );
 	free( method_used );
@@ -1010,7 +1040,7 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::Authenticate
 		if( !auth_success ) {
 			dprintf( D_ALWAYS,
 					 "DC_AUTHENTICATE: reason for authentication failure: %s\n",
-					 errstack.getFullText() );
+					 errstack.getFullText().c_str() );
 		}
 		m_result = FALSE;
 		return CommandProtocolFinished;
@@ -1037,7 +1067,7 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::Authenticate
 			dprintf( D_ALWAYS,
 					 "DC_AUTHENTICATE: required authentication of %s failed: %s\n",
 					 m_sock->peer_ip_str(),
-					 errstack.getFullText() );
+					 errstack.getFullText().c_str() );
 			m_result = FALSE;
 			return CommandProtocolFinished;
 		}
@@ -1155,19 +1185,19 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::PostAuthenti
 		m_sec_man->sec_copy_attribute( *m_policy, pa_ad, ATTR_SEC_SID );
 		m_sec_man->sec_copy_attribute( *m_policy, pa_ad, ATTR_SEC_VALID_COMMANDS );
 
-		if (DebugFlags & D_FULLDEBUG) {
+		if (IsDebugVerbose(D_SECURITY)) {
 			dprintf (D_SECURITY, "DC_AUTHENTICATE: sending session ad:\n");
-			pa_ad.dPrint( D_SECURITY );
+			dPrintAd( D_SECURITY, pa_ad );
 		}
 
 		m_sock->encode();
-		if (! pa_ad.put(*m_sock) ||
+		if (! putClassAd(m_sock, pa_ad) ||
 			! m_sock->end_of_message() ) {
 			dprintf (D_ALWAYS, "DC_AUTHENTICATE: unable to send session %s info to %s!\n", m_sid, m_sock->peer_description());
 			m_result = FALSE;
 			return CommandProtocolFinished;
 		} else {
-			if (DebugFlags & D_FULLDEBUG) {
+			if (IsDebugVerbose(D_SECURITY)) {
 				dprintf (D_SECURITY, "DC_AUTHENTICATE: sent session %s info!\n", m_sid);
 			}
 		}
@@ -1209,8 +1239,8 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::PostAuthenti
 		KeyCacheEntry tmp_key(m_sid, NULL, m_key, m_policy, expiration_time, session_lease );
 		m_sec_man->session_cache->insert(tmp_key);
 		dprintf (D_SECURITY, "DC_AUTHENTICATE: added incoming session id %s to cache for %i seconds (lease is %ds, return address is %s).\n", m_sid, durint, session_lease, return_addr ? return_addr : "unknown");
-		if (DebugFlags & D_FULLDEBUG) {
-			m_policy->dPrint(D_SECURITY);
+		if (IsDebugVerbose(D_SECURITY)) {
+			dPrintAd(D_SECURITY, *m_policy);
 		}
 
 		free( dur );
@@ -1231,13 +1261,20 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ExecCommand(
 	int cmd_index = 0;
 
 	if (m_req == DC_AUTHENTICATE) {
+
+		m_result = TRUE;
+
 		if (m_real_cmd == DC_AUTHENTICATE) {
-			m_result = TRUE;
 			return CommandProtocolFinished;
 		}
 
-		m_req = m_real_cmd;
-		m_result = TRUE;
+		if (m_real_cmd == DC_SEC_QUERY) {
+			// continue spoofing another command.  just before calling the command
+			// handler we will check m_real_cmd again for DC_SEC_QUERY and abort.
+			m_req = m_auth_cmd;
+		} else {
+			m_req = m_real_cmd;
+		}
 
 		if( !daemonCore->CommandNumToTableIndex(m_auth_cmd,&cmd_index) ) {
 			dprintf(D_ALWAYS, "DC_AUTHENTICATE: UNREGISTERED COMMAND %d in ExecCommand()\n",m_auth_cmd);
@@ -1360,7 +1397,7 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ExecCommand(
 		}
 
 		MyString command_desc;
-		command_desc.sprintf("command %d (%s)",m_req,m_comTable[cmd_index].command_descrip);
+		command_desc.formatstr("command %d (%s)",m_req,m_comTable[cmd_index].command_descrip);
 
 		if( m_comTable[cmd_index].force_authentication &&
 			!m_sock->isMappedFQU() )
@@ -1380,6 +1417,32 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ExecCommand(
 						  m_user.Value() );
 		}
 
+		// special case for DC_SEC_QUERY.  we are not going to call a command
+		// handler, but we need to respond whether or not the authorization
+		// succeeded.
+		if ( m_real_cmd == DC_SEC_QUERY ) {
+
+			// send another classad saying what happened
+			ClassAd q_response;
+			q_response.Assign( ATTR_SEC_AUTHORIZATION_SUCCEEDED, (m_perm == USER_AUTH_SUCCESS) );
+
+			if (!putClassAd(m_sock, q_response) ||
+				!m_sock->end_of_message()) {
+				dprintf (D_ALWAYS, "SECMAN: Error sending DC_SEC_QUERY classad to %s!\n", m_sock->peer_description());
+				dPrintAd (D_ALWAYS, q_response);
+				m_result = FALSE;
+				return CommandProtocolFinished;
+			}
+
+			dprintf (D_ALWAYS, "SECMAN: Succesfully sent DC_SEC_QUERY classad to %s!\n", m_sock->peer_description());
+			dPrintAd (D_ALWAYS, q_response);
+
+			// now, having informed the client about the authorization status,
+			// successfully abort before actually calling any command handler.
+			m_result = TRUE;
+			return CommandProtocolFinished;
+		}
+
 		if( m_perm != USER_AUTH_SUCCESS )
 		{
 			// Permission check FAILED
@@ -1425,6 +1488,19 @@ DaemonCommandProtocol::CommandProtocolResult DaemonCommandProtocol::ExecCommand(
 		}
 	}
 */
+
+	if ( m_real_cmd == DC_SEC_QUERY ) {
+		// send another classad saying what happened
+		// abort before actually calling any command handler.
+		m_result = TRUE;
+		return CommandProtocolFinished;
+	}
+
+	// call the auditing callback to record the status of this connection
+	if (daemonCore->audit_log_callback_fn) {
+		(*(daemonCore->audit_log_callback_fn))( m_req, (*m_sock), (m_perm != USER_AUTH_SUCCESS) );
+	}
+
 	if ( m_reqFound == TRUE ) {
 		// Handlers should start out w/ parallel mode disabled by default
 		ScopedEnableParallel(false);
