@@ -17,13 +17,11 @@
  *
  ***************************************************************/
 
-#include "condor_timer_manager.h"
 #include "condor_io.h"
 
-#include "stdsoap2.h"
+struct soap;
 
-void init_soap(struct soap *soap);
-
-int handle_soap_ssl_socket(Service *, Stream *stream);
-
-int get_handler(struct soap *soap);
+void dc_soap_init(struct soap *&soap);
+void dc_soap_free(struct soap *soap);
+int dc_soap_serve(struct soap *soap);
+struct soap *dc_soap_accept(Sock *socket, const struct soap *soap);
