@@ -22,6 +22,7 @@
 
 #include "dag.h"
 #include "string_list.h"
+#include "dagman_classad.h"
 
 	// Don't change these values!  Doing so would break some DAGs.
 enum exit_value {
@@ -39,12 +40,24 @@ class Dagman {
   public:
 	Dagman();
 	~Dagman();
+
     inline void CleanUp () { 
 		if ( dag != NULL ) {
 			delete dag; 
 			dag = NULL;
 		}
+		delete _dagmanClassad;
+		_dagmanClassad = NULL;
 	}
+
+		// Check (based on the version from the .condor.sub file, etc.),
+		// whether we should fall back to non-default log mode.
+	void CheckLogFileMode( const CondorVersionInfo &submitFileVersion );
+
+		// Disable use of the default node log (use the log files from
+		// the submit files instead).
+	void DisableDefaultLog();
+
     Dag * dag;
     int maxIdle;  // Maximum number of idle DAG nodes
     int maxJobs;  // Maximum number of Jobs to run at once
@@ -204,6 +217,8 @@ class Dagman {
 		// Default priority that DAGman uses for nodes.
 	int _defaultPriority;
 	int _claim_hold_time;
+
+	DagmanClassad *_dagmanClassad;
 };
 
 #endif	// ifndef DAGMAN_MAIN_H
