@@ -32,8 +32,6 @@
 #include "CondorError.h"
 #include <set>
 
-using namespace std;
-
 class ThrottleByCategory;
 class Dag;
 
@@ -63,7 +61,9 @@ typedef int JobID_t;
      in the Condor system.  If it completes successfully and the job
      has a POST script defined it changes to POSTRUN while that script
      executes, otherwise if it completes successfully it is DONE, or
-     is in the ERROR state if it completes abnormally.<p>
+     is in the ERROR state if it completes abnormally.  Note that final
+     nodes are a special case -- they are created with a state of
+     NOT_READY<p>
 
      The DAG class will control the job by modifying and viewing its
      three queues.  Once the WAITING queue becomes empty, the job
@@ -191,7 +191,7 @@ class Job {
 	Script * _scriptPost;
 
     ///
-    inline set<JobID_t> & GetQueueRef (const queue_t queue) {
+    inline std::set<JobID_t> & GetQueueRef (const queue_t queue) {
         return _queues[queue];
     }
 
@@ -220,9 +220,11 @@ class Job {
 
 	/** Check whether the submit file for this job has a log file
 	    defined.
+		@param usingDefault is true if DAGman is watching the
+			default node log
 		@return true iff the submit file defines a log file
 	*/
-	bool CheckForLogFile() const;
+	bool CheckForLogFile(bool usingDefault) const;
 
     /** Returns true if a queue is empty (has no jobs)
         @param queue Selects which queue to look at
@@ -355,7 +357,7 @@ class Job {
 	*/
 	bool MonitorLogFile( ReadMultipleUserLogs &condorLogReader,
 				ReadMultipleUserLogs &storkLogReader, bool nfsIsError,
-				bool recovery, const char *defaultNodeLog );
+				bool recovery, const char *defaultNodeLog, bool usingDefault );
 
 	/** Unmonitor this node's Condor or Stork log file with the
 		multiple log reader.  (Must be called after everything is done
@@ -367,7 +369,7 @@ class Job {
 				ReadMultipleUserLogs &storkLogReader );
 
 		// Whether this node is using the default node log file.
-	bool UsingDefaultLog() { return _useDefaultLog; }
+	bool UsingDefaultLog() const { return _useDefaultLog; }
 
 	/** Get the log file for this node.
 		@return the name of this node's log file.
@@ -419,7 +421,14 @@ class Job {
 	bool HasPreSkip() const { return _preskip != PRE_SKIP_INVALID; }
 	int GetPreSkip() const;
 	
+	int GetCluster() const { return _CondorID._cluster; }
+	int GetProc() const { return _CondorID._proc; }
+	int GetSubProc() const { return _CondorID._subproc; }
+	bool SetCondorID(const CondorID& cid);
+	const CondorID& GetID() const { return _CondorID; }
+private:
     /** */ CondorID _CondorID;
+public:
 
     // maximum number of times to retry this node
     int retry_max;
@@ -463,14 +472,12 @@ class Job {
 	//DFS ordering of the node
 	int _dfsOrder; 
 
-	// DAG definition files can now pass named variables into job submit files.
-	// For lack of a pair<> class, I made two lists, and these lists work together
-	// closely. The order of their items defines the mapping between names
-	// and values of these named variables, i.e., for the first named variable, its
-	// name is the first item in varNamesFromDag, and its value is the first item
-	// in varValsFromDag.
-	List<MyString> *varNamesFromDag;
-	List<MyString> *varValsFromDag;
+	struct NodeVar {
+		MyString _name;
+		MyString _value;
+	};
+
+	List<NodeVar> *varsFromDag;
 
 		// Count of the number of job procs currently in the batch system
 		// queue for this node.
@@ -491,8 +498,25 @@ class Job {
 		// (Note: we may need to track the hold state of each proc in a
 		// cluster separately to correctly deal with multi-proc clusters.)
 	int _jobProcsOnHold;
+	bool UseDefaultLog() const { return append_default_log; }
 
+		/** Mark a job with ProcId == proc as being on hold
+ 			Returns false if the job is already on hold
+		*/
+ 
+	bool Hold(int proc);
+	
+		/** Mark a job with ProcId == proc as being released
+ 		    Returns false if the job is not on hold
+		*/
+	bool Release(int proc);
 private:
+
+		/** _onHold[proc] is nonzero if the condor job 
+ 			with ProcId == proc is on hold, and zero
+			otherwise
+		*/
+	std::vector<unsigned char> _onHold;	
 		// Mark this node as failed because of an error in monitoring
 		// the log file.
   	void LogMonitorFailed();
@@ -537,7 +561,7 @@ private:
 		waiting -> Jobs on which the current Job is waiting for output
     */ 
 	
-	set<JobID_t> _queues[3];
+	std::set<JobID_t> _queues[3];
 
     /*	The ID of this job.  This serves as a primary key for Jobs, where each
 		Job's ID is unique from all the rest 
@@ -593,17 +617,16 @@ private:
 		// Skip the rest of the node (and consider it successful) if the
 		// PRE script exits with this value.  (-1 means undefined.)
 	int _preskip;
-	int _pre_status;
 
 	enum {
 		PRE_SKIP_INVALID = -1,
 		PRE_SKIP_MIN = 0,
-		PRE_SKIP_MAX = 0xff,
-		NO_PRE_VALUE = -1
+		PRE_SKIP_MAX = 0xff
 	};
 
 	// whether this is a final job
 	bool _final;
+	bool append_default_log;
 };
 
 /** A wrapper function for Job::Print which allows a NULL job pointer.
