@@ -31,6 +31,7 @@
 #include "condor_arglist.h"
 #include <map>
 #include <queue>
+#include <list>
 #include <string>
 
 
@@ -43,8 +44,8 @@ struct GahpProxyInfo
 
 typedef void (* unicore_gahp_callback_func_t)(const char *update_ad_string);
 
-static const char *GAHPCLIENT_DEFAULT_SERVER_ID = "DEFAULT";
-static const char *GAHPCLIENT_DEFAULT_SERVER_PATH = "DEFAULT";
+#define GAHPCLIENT_DEFAULT_SERVER_ID "DEFAULT"
+#define GAHPCLIENT_DEFAULT_SERVER_PATH "DEFAULT"
 
 // Additional error values that GAHP calls can return
 ///
@@ -168,6 +169,7 @@ class GahpServer : public Service {
 	int m_gahp_writefd;
 	int m_gahp_errorfd;
 	std::string m_gahp_error_buffer;
+	std::list<std::string> m_gahp_error_list;
 	bool m_gahp_startup_failed;
 	char m_gahp_version[150];
 	StringList * m_commands_supported;
@@ -183,6 +185,7 @@ class GahpServer : public Service {
 	char *binary_path;
 	ArgList binary_args;
 	char *my_id;
+	int m_ssh_forward_port;
 
 	char *globus_gass_server_url;
 	char *globus_gt2_gram_callback_contact;
@@ -305,9 +308,12 @@ class GahpClient : public Service {
 		bool isInitialized() { return server->is_initialized; }
 
 		const char *getErrorString();
+		const char *getGahpStderr();
 
 		const char *getVersion();
 
+		int getSshForwardPort() { return server->m_ssh_forward_port; }
+
 		//-----------------------------------------------------------
 		
 		/**@name Globus Methods
@@ -444,6 +450,16 @@ class GahpClient : public Service {
 		int
 		blah_job_refresh_proxy(const char *job_id, const char *proxy_file);
 
+		int
+		blah_download_sandbox(const char *sandbox_id, const ClassAd *job_ad,
+							  std::string &sandbox_path);
+
+		int
+		blah_upload_sandbox(const char *sandbox_id, const ClassAd *job_ad);
+
+		int
+		blah_destroy_sandbox(const char *sandbox_id, const ClassAd *job_ad);
+
 		int
 		nordugrid_submit(const char *hostname, const char *rsl, char *&job_id);
 
@@ -570,6 +586,12 @@ class GahpClient : public Service {
 							  StringList & returnStatus,
 							  char* & error_code );
 
+		int ec2_vm_status_all( std::string service_url,
+							   std::string publickeyfile,
+							   std::string privatekeyfile,
+							   StringList & returnStatus,
+							   char* & error_code );
+
 		int ec2_ping( std::string service_url,
 					  std::string publickeyfile,
 					  std::string privatekeyfile,
@@ -637,6 +659,44 @@ class GahpClient : public Service {
                               StringList & returnStatus,
                               char* & error_code );
 
+        // Is there a particular reason these aren't const references?
+        int ec2_spot_start( std::string service_url,
+                            std::string publickeyfile,
+                            std::string privatekeyfile,
+                            std::string ami_id,
+                            std::string spot_price,
+                            std::string keypair,
+                            std::string user_data,
+                            std::string user_data_file,
+                            std::string instance_type,
+                            std::string availability_zone,
+                            std::string vpc_subnet,
+                            std::string vpc_ip,
+                            std::string client_token,
+                            StringList & groupnames,
+                            char * & request_id,
+                            char * & error_code
+                          );
+        int ec2_spot_stop(  std::string service_url,
+                            std::string publickeyfile,
+                            std::string privatekeyfile,
+                            std::string request_id,
+                            char * & error_code
+                         );
+        int ec2_spot_status(    std::string service_url,
+                                std::string publickeyfile,
+                                std::string privatekeyfile,
+                                std::string request_id,
+                                StringList & returnStatus,
+                                char * & error_code
+                           );
+        int ec2_spot_status_all(    std::string service_url,
+                                    std::string publickeyfile,
+                                    std::string privatekeyfile,
+                                    StringList & returnStatus,
+                                    char * & error_code
+                               );
+
 		int
 		dcloud_submit( const char *service_url,
 					   const char *username,
@@ -706,6 +766,8 @@ class GahpClient : public Service {
 
 		//@}
 
+    void setErrorString( const std::string & newErrorString );
+
 	private:
 
 	/// Enum used by now_pending for the waiting queues
