@@ -32,7 +32,7 @@
 char *myUserName = NULL;
 
 // this appears at the bottom of this file
-extern "C" int display_dprintf_header(char **buf,int *bufpos,int *buflen);
+int display_dprintf_header(char **buf,int *bufpos,int *buflen);
 
 void
 usage( char *name )
@@ -181,8 +181,9 @@ main( int argc, char **argv )
 
 // This function is called by dprintf - always display our pid in our
 // log entries. 
-extern "C" 
+//extern "C" 
 int
+//display_dprintf_header(char **buf,int *bufpos,int *buflen)
 display_dprintf_header(char **buf,int *bufpos,int *buflen)
 {
 	static pid_t mypid = 0;
