@@ -34,9 +34,9 @@ const char * INFNBatchResource::HashName( const char * batch_type,
 		const char * resource_name )
 {
 	static std::string hash_name;
-	hash_name = batch_type;
+	formatstr( hash_name, "batch %s", batch_type );
 	if ( resource_name && resource_name[0] ) {
-		sprintf_cat( hash_name, " %s", resource_name );
+		formatstr_cat( hash_name, " %s", resource_name );
 	}
 	return hash_name.c_str();
 }
@@ -71,12 +71,20 @@ INFNBatchResource::INFNBatchResource( const char *batch_type,
 	: BaseResource( resource_name )
 {
 	m_batchType = batch_type;
+	m_gahpIsRemote = false;
+
+	m_remoteHostname = resource_name;
+	size_t pos = m_remoteHostname.find( '@' );
+	if ( pos != m_remoteHostname.npos ) {
+		m_remoteHostname.erase( 0, pos + 1 );
+	}
 	
 	gahp = NULL;
 
 	std::string gahp_name = batch_type;
 	if ( resource_name && *resource_name ) {
-		sprintf_cat( gahp_name, "/%s", resource_name );
+		formatstr_cat( gahp_name, "/%s", resource_name );
+		m_gahpIsRemote = true;
 	}
 
 	gahp = new GahpClient( gahp_name.c_str() );
@@ -101,7 +109,7 @@ void INFNBatchResource::Reconfig()
 
 const char *INFNBatchResource::ResourceType()
 {
-	return m_batchType.c_str();
+	return "batch";
 }
 
 const char *INFNBatchResource::GetHashName()
