@@ -55,6 +55,7 @@
 #define GM_EXIT_INFO			15
 #define GM_RECOVER_QUERY		16
 #define GM_START				17
+#define GM_STAGE_OUT_OLD		18
 
 static const char *GMStateNames[] = {
 	"GM_INIT",
@@ -74,7 +75,8 @@ static const char *GMStateNames[] = {
 	"GM_STAGE_OUT",
 	"GM_EXIT_INFO",
 	"GM_RECOVER_QUERY",
-	"GM_START"
+	"GM_START",
+	"GM_STAGE_OUT_OLD",
 };
 
 #define REMOTE_STATE_ACCEPTING		"ACCEPTING"
@@ -124,9 +126,6 @@ void NordugridJobReconfig()
 {
 	int tmp_int;
 
-	tmp_int = param_integer( "GRIDMANAGER_JOB_PROBE_INTERVAL", 5 * 60 );
-	NordugridJob::setProbeInterval( tmp_int );
-
 	tmp_int = param_integer( "GRIDMANAGER_GAHP_CALL_TIMEOUT", 5 * 60 );
 	NordugridJob::setGahpCallTimeout( tmp_int );
 
@@ -158,7 +157,6 @@ BaseJob *NordugridJobCreate( ClassAd *jobad )
 	return (BaseJob *)new NordugridJob( jobad );
 }
 
-int NordugridJob::probeInterval = 300;	// default value
 int NordugridJob::submitInterval = 300;	// default value
 int NordugridJob::gahpCallTimeout = 300;	// default value
 int NordugridJob::maxConnectFailures = 3;	// default value
@@ -198,7 +196,7 @@ NordugridJob::NordugridJob( ClassAd *classad )
 							 (TimerHandlercpp)&BaseJob::SetEvaluateState, this );
 	if ( jobProxy == NULL ) {
 		if ( error_string == "" ) {
-			sprintf( error_string, "%s is not set in the job ad",
+			formatstr( error_string, "%s is not set in the job ad",
 								  ATTR_X509_USER_PROXY );
 		}
 		goto error_exit;
@@ -228,7 +226,7 @@ NordugridJob::NordugridJob( ClassAd *classad )
 
 		token = GetNextToken( " ", false );
 		if ( !token || strcasecmp( token, "nordugrid" ) ) {
-			sprintf( error_string, "%s not of type nordugrid",
+			formatstr( error_string, "%s not of type nordugrid",
 								  ATTR_GRID_RESOURCE );
 			goto error_exit;
 		}
@@ -237,13 +235,13 @@ NordugridJob::NordugridJob( ClassAd *classad )
 		if ( token && *token ) {
 			resourceManagerString = strdup( token );
 		} else {
-			sprintf( error_string, "%s missing server name",
+			formatstr( error_string, "%s missing server name",
 								  ATTR_GRID_RESOURCE );
 			goto error_exit;
 		}
 
 	} else {
-		sprintf( error_string, "%s is not set in the job ad",
+		formatstr( error_string, "%s is not set in the job ad",
 							  ATTR_GRID_RESOURCE );
 		goto error_exit;
 	}
@@ -528,13 +526,14 @@ void NordugridJob::doEvaluateState()
 					probeNow = false;
 				}
 /*
-				if ( now >= lastProbeTime + probeInterval ) {
+				int probe_interval = myResource->GetJobPollInterval();
+				if ( now >= lastProbeTime + probe_interval ) {
 					gmState = GM_PROBE_JOB;
 					break;
 				}
 				unsigned int delay = 0;
-				if ( (lastProbeTime + probeInterval) > now ) {
-					delay = (lastProbeTime + probeInterval) - now;
+				if ( (lastProbeTime + probe_interval) > now ) {
+					delay = (lastProbeTime + probe_interval) - now;
 				}				
 				daemonCore->Reset_Timer( evaluateStateTid, delay );
 */
@@ -574,7 +573,7 @@ void NordugridJob::doEvaluateState()
 		case GM_EXIT_INFO: {
 			std::string filter;
 			StringList reply;
-			sprintf( filter, "nordugrid-job-globalid=gsiftp://%s:2811/jobs/%s",
+			formatstr( filter, "nordugrid-job-globalid=gsiftp://%s:2811/jobs/%s",
 							resourceManagerString, remoteJobId );
 
 			rc = gahp->nordugrid_ldap_query( resourceManagerString, "mds-vo-name=local,o=grid", filter.c_str(), "nordugrid-job-usedcputime,nordugrid-job-usedwalltime,nordugrid-job-exitcode", reply );
@@ -617,8 +616,8 @@ void NordugridJob::doEvaluateState()
 					jobAd->Assign( ATTR_ON_EXIT_BY_SIGNAL, false );
 					jobAd->Assign( ATTR_ON_EXIT_CODE, exit_code );
 				}
-				jobAd->Assign( ATTR_JOB_REMOTE_WALL_CLOCK, wallclock * 60 );
-				jobAd->Assign( ATTR_JOB_REMOTE_USER_CPU, cpu * 60 );
+				jobAd->Assign( ATTR_JOB_REMOTE_WALL_CLOCK, wallclock * 60.0 );
+				jobAd->Assign( ATTR_JOB_REMOTE_USER_CPU, cpu * 60.0 );
 				gmState = GM_STAGE_OUT;
 			}
 			} break;
@@ -639,6 +638,45 @@ void NordugridJob::doEvaluateState()
 				}
 				}
 			}
+			rc = gahp->nordugrid_stage_out2( resourceManagerString,
+											 remoteJobId,
+											 *stageList, *stageLocalList );
+			if ( rc == GAHPCLIENT_COMMAND_NOT_SUBMITTED ||
+				 rc == GAHPCLIENT_COMMAND_PENDING ) {
+				break;
+			} else if ( rc != 0 ) {
+				errorString = gahp->getErrorString();
+				dprintf( D_ALWAYS, "(%d.%d) file stage out failed: %s\n",
+						 procID.cluster, procID.proc, errorString.c_str() );
+				dprintf( D_ALWAYS, "(%d.%d) retrying with old stdout/err names\n",
+						 procID.cluster, procID.proc );
+				gmState = GM_STAGE_OUT_OLD;
+				//gmState = GM_CANCEL;
+			} else {
+				gmState = GM_DONE_SAVE;
+			}
+			} break;
+		case GM_STAGE_OUT_OLD: {
+			// CRUFT: Condor 8.0.5 introduced new names for the stdout and
+			//   stderr files on the nordugrid server. In case jobs were
+			//   queued during an upgrade from pre-8.0.5, trying using the
+			//   old names when transferring the output sandbox.
+			if ( stageList == NULL ) {
+				stageList = buildStageOutList( true );
+			}
+			if ( stageLocalList == NULL ) {
+				const char *file;
+				stageLocalList = buildStageOutLocalList( stageList, true );
+				stageList->rewind();
+				stageLocalList->rewind();
+				while ( (file = stageLocalList->next()) ) {
+					ASSERT( stageList->next() );
+					if ( IsUrl( file ) ) {
+						stageList->deleteCurrent();
+						stageLocalList->deleteCurrent();
+				}
+				}
+			}
 			rc = gahp->nordugrid_stage_out2( resourceManagerString,
 											 remoteJobId,
 											 *stageList, *stageLocalList );
@@ -895,7 +933,7 @@ void NordugridJob::SetRemoteJobId( const char *job_id )
 
 	std::string full_job_id;
 	if ( job_id ) {
-		sprintf( full_job_id, "nordugrid %s %s", resourceManagerString,
+		formatstr( full_job_id, "nordugrid %s %s", resourceManagerString,
 							 job_id );
 	}
 	BaseJob::SetRemoteJobId( full_job_id.c_str() );
@@ -911,6 +949,10 @@ std::string *NordugridJob::buildSubmitRSL()
 	std::string rsl_suffix;
 	std::string iwd;
 	std::string executable;
+	std::string remote_stdout_name;
+	std::string remote_stderr_name;
+
+	GetRemoteStdoutNames( remote_stdout_name, remote_stderr_name );
 
 	if ( jobAd->LookupString( ATTR_NORDUGRID_RSL, rsl_suffix ) &&
 						   rsl_suffix[0] == '&' ) {
@@ -926,7 +968,7 @@ std::string *NordugridJob::buildSubmitRSL()
 
 	//Start off the RSL
 	attr_value = param( "FULL_HOSTNAME" );
-	sprintf( *rsl, "&(savestate=yes)(action=request)(hostname=%s)", attr_value );
+	formatstr( *rsl, "&(savestate=yes)(action=request)(hostname=%s)", attr_value );
 	free( attr_value );
 	attr_value = NULL;
 
@@ -950,7 +992,7 @@ std::string *NordugridJob::buildSubmitRSL()
 		if(!args.AppendArgsFromClassAd(jobAd,&arg_errors)) {
 			dprintf(D_ALWAYS,"(%d.%d) Failed to read job arguments: %s\n",
 					procID.cluster, procID.proc, arg_errors.Value());
-			sprintf(errorString,"Failed to read job arguments: %s\n",
+			formatstr(errorString,"Failed to read job arguments: %s\n",
 					arg_errors.Value());
 			delete rsl;
 			return NULL;
@@ -962,7 +1004,7 @@ std::string *NordugridJob::buildSubmitRSL()
 					dprintf(D_ALWAYS,
 							"(%d.%d) Failed to get job arguments: %s\n",
 							procID.cluster,procID.proc,arg_errors.Value());
-					sprintf(errorString,"Failed to get job arguments: %s\n",
+					formatstr(errorString,"Failed to get job arguments: %s\n",
 							arg_errors.Value());
 					delete rsl;
 					return NULL;
@@ -1011,7 +1053,7 @@ std::string *NordugridJob::buildSubmitRSL()
 			*rsl += "(";
 			*rsl += condor_basename(file);
 			if ( IsUrl( file ) ) {
-				sprintf_cat( *rsl, " \"%s\")", file );
+				formatstr_cat( *rsl, " \"%s\")", file );
 			} else {
 				*rsl += " \"\")";
 			}
@@ -1024,7 +1066,8 @@ std::string *NordugridJob::buildSubmitRSL()
 	if ( jobAd->LookupString( ATTR_JOB_OUTPUT, &attr_value ) == 1) {
 		// only add to list if not NULL_FILE (i.e. /dev/null)
 		if ( ! nullFile(attr_value) ) {
-			*rsl += ")(stdout=" REMOTE_STDOUT_NAME;
+			*rsl += ")(stdout=";
+			*rsl += remote_stdout_name;
 		}
 		free( attr_value );
 		attr_value = NULL;
@@ -1033,7 +1076,8 @@ std::string *NordugridJob::buildSubmitRSL()
 	if ( jobAd->LookupString( ATTR_JOB_ERROR, &attr_value ) == 1) {
 		// only add to list if not NULL_FILE (i.e. /dev/null)
 		if ( ! nullFile(attr_value) ) {
-			*rsl += ")(stderr=" REMOTE_STDERR_NAME;
+			*rsl += ")(stderr=";
+			*rsl += remote_stderr_name;
 		}
 		free( attr_value );
 	}
@@ -1054,7 +1098,7 @@ std::string *NordugridJob::buildSubmitRSL()
 			*rsl += "(";
 			*rsl += condor_basename(file);
 			if ( IsUrl( local_file ) ) {
-				sprintf_cat( *rsl, " \"%s\")", local_file );
+				formatstr_cat( *rsl, " \"%s\")", local_file );
 			} else {
 				*rsl += " \"\")";
 			}
@@ -1118,9 +1162,9 @@ StringList *NordugridJob::buildStageInList()
 	tmp_list->rewind();
 	while ( ( filename = tmp_list->next() ) ) {
 		if ( filename[0] == '/' || IsUrl( filename ) ) {
-			sprintf( buf, "%s", filename );
+			formatstr( buf, "%s", filename );
 		} else {
-			sprintf( buf, "%s%s", iwd.c_str(), filename );
+			formatstr( buf, "%s%s", iwd.c_str(), filename );
 		}
 		stage_list->append( buf.c_str() );
 	}
@@ -1130,11 +1174,15 @@ StringList *NordugridJob::buildStageInList()
 	return stage_list;
 }
 
-StringList *NordugridJob::buildStageOutList()
+StringList *NordugridJob::buildStageOutList( bool old_stdout )
 {
 	StringList *stage_list = NULL;
 	std::string buf;
 	bool transfer = TRUE;
+	std::string remote_stdout_name;
+	std::string remote_stderr_name;
+
+	GetRemoteStdoutNames( remote_stdout_name, remote_stderr_name, old_stdout );
 
 	jobAd->LookupString( ATTR_TRANSFER_OUTPUT_FILES, buf );
 	stage_list = new StringList( buf.c_str(), "," );
@@ -1143,8 +1191,8 @@ StringList *NordugridJob::buildStageOutList()
 	if ( transfer && jobAd->LookupString( ATTR_JOB_OUTPUT, buf ) == 1) {
 		// only add to list if not NULL_FILE (i.e. /dev/null)
 		if ( ! nullFile(buf.c_str()) ) {
-			if ( !stage_list->file_contains( REMOTE_STDOUT_NAME ) ) {
-				stage_list->append( REMOTE_STDOUT_NAME );
+			if ( !stage_list->file_contains( remote_stdout_name.c_str() ) ) {
+				stage_list->append( remote_stdout_name.c_str() );
 			}
 		}
 	}
@@ -1154,8 +1202,8 @@ StringList *NordugridJob::buildStageOutList()
 	if ( transfer && jobAd->LookupString( ATTR_JOB_ERROR, buf ) == 1) {
 		// only add to list if not NULL_FILE (i.e. /dev/null)
 		if ( ! nullFile(buf.c_str()) ) {
-			if ( !stage_list->file_contains( REMOTE_STDERR_NAME ) ) {
-				stage_list->append( REMOTE_STDERR_NAME );
+			if ( !stage_list->file_contains( remote_stderr_name.c_str() ) ) {
+				stage_list->append( remote_stderr_name.c_str() );
 			}
 		}
 	}
@@ -1163,7 +1211,8 @@ StringList *NordugridJob::buildStageOutList()
 	return stage_list;
 }
 
-StringList *NordugridJob::buildStageOutLocalList( StringList *stage_list )
+StringList *NordugridJob::buildStageOutLocalList( StringList *stage_list,
+												  bool old_stdout )
 {
 	StringList *stage_local_list;
 	char *remaps = NULL;
@@ -1173,6 +1222,10 @@ StringList *NordugridJob::buildStageOutLocalList( StringList *stage_list )
 	std::string stderr_name = "";
 	std::string buff;
 	std::string iwd = "/";
+	std::string remote_stdout_name;
+	std::string remote_stderr_name;
+
+	GetRemoteStdoutNames( remote_stdout_name, remote_stderr_name, old_stdout );
 
 	if ( jobAd->LookupString( ATTR_JOB_IWD, iwd ) ) {
 		if ( iwd.length() > 1 && iwd[iwd.length() - 1] != '/' ) {
@@ -1191,9 +1244,9 @@ StringList *NordugridJob::buildStageOutLocalList( StringList *stage_list )
 	while ( (remote_name = stage_list->next()) ) {
 			// stdout and stderr don't get remapped, and their paths
 			// are evaluated locally
-		if ( strcmp( REMOTE_STDOUT_NAME, remote_name ) == 0 ) {
+		if ( strcmp( remote_stdout_name.c_str(), remote_name ) == 0 ) {
 			local_name = stdout_name.c_str();
-		} else if ( strcmp( REMOTE_STDERR_NAME, remote_name ) == 0 ) {
+		} else if ( strcmp( remote_stderr_name.c_str(), remote_name ) == 0 ) {
 			local_name = stderr_name.c_str();
 		} else if( remaps && filename_remap_find( remaps, remote_name,
 												  local_name ) ) {
@@ -1206,7 +1259,7 @@ StringList *NordugridJob::buildStageOutLocalList( StringList *stage_list )
 			 || IsUrl( local_name.Value() ) ) {
 			buff = local_name;
 		} else {
-			sprintf( buff, "%s%s", iwd.c_str(), local_name.Value() );
+			formatstr( buff, "%s%s", iwd.c_str(), local_name.Value() );
 		}
 		stage_local_list->append( buff.c_str() );
 	}
@@ -1246,3 +1299,25 @@ void NordugridJob::NotifyNewRemoteStatus( const char *status )
 		SetEvaluateState();
 	}
 }
+
+void NordugridJob::GetRemoteStdoutNames( std::string &std_out, std::string &std_err, bool use_old_names )
+{
+	if ( use_old_names ) {
+		std_out = REMOTE_STDOUT_NAME;
+		std_err = REMOTE_STDERR_NAME;
+	} else {
+		std::string gjid;
+		jobAd->LookupString( ATTR_GLOBAL_JOB_ID, gjid );
+
+		size_t loc = 0;
+		#define FILENAME_FILTER "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789.-_"
+		while( (loc = gjid.find_first_not_of( FILENAME_FILTER, loc )) != std::string::npos ) {
+			gjid[loc] = '_';
+		}
+
+		std_out = REMOTE_STDOUT_NAME ".";
+		std_out += gjid;
+		std_err = REMOTE_STDERR_NAME ".";
+		std_err += gjid;
+	}
+}
