@@ -77,7 +77,9 @@ struct Proxy {
 
 struct ProxySubject {
 	char *fqan;
+	char *first_fqan;
 	char *subject_name;
+	char *email;
 	bool has_voms_attrs;
 	Proxy *master_proxy;
 	List<Proxy> proxies;
