@@ -23,11 +23,6 @@
 
 #include "ReplicatorStateMachine.h"
 
-/* daemon core needs this variable to associate its entries
- * inside $CONDOR_CONFIG file
- */
-DECL_SUBSYSTEM( "Replication", SUBSYSTEM_TYPE_DAEMON );// used by Daemon Core
-
 // replication daemon single object
 ReplicatorStateMachine* stateMachine = NULL;
 
@@ -72,13 +67,14 @@ main_config()
 	stateMachine->reinitialize( );
 }
 
-void
-main_pre_dc_init( int argc, char* argv[] )
+int
+main( int argc, char **argv )
 {
-}
+	set_mySubSystem( "Replication", SUBSYSTEM_TYPE_DAEMON );// used by Daemon Core
 
-void
-main_pre_command_sock_init( )
-{
+	dc_main_init = main_init;
+	dc_main_config = main_config;
+	dc_main_shutdown_fast = main_shutdown_fast;
+	dc_main_shutdown_graceful = main_shutdown_graceful;
+	return dc_main( argc, argv );
 }
-
