@@ -101,9 +101,9 @@ ReplicatorStateMachine::finalizeDelta( )
         m_classAd = NULL;
     }
 
-    invalidate_ad.SetMyTypeName( QUERY_ADTYPE );
-    invalidate_ad.SetTargetTypeName( "Replication" );
-    line.sprintf( "TARGET.%s == \"%s\"", ATTR_NAME, m_name.Value( ) );
+    SetMyTypeName( invalidate_ad, QUERY_ADTYPE );
+    SetTargetTypeName( invalidate_ad, "Replication" );
+    line.formatstr( "TARGET.%s == \"%s\"", ATTR_NAME, m_name.Value( ) );
     invalidate_ad.AssignExpr( ATTR_REQUIREMENTS, line.Value( ) );
 	invalidate_ad.Assign( ATTR_NAME, m_name.Value() );
     daemonCore->sendUpdates( INVALIDATE_ADS_GENERIC, &invalidate_ad, NULL, false );
@@ -219,10 +219,10 @@ ReplicatorStateMachine::initializeClassAd()
 
     m_classAd = new ClassAd();
 
-    m_classAd->SetMyTypeName("Replication");
-    m_classAd->SetTargetTypeName("");
+    SetMyTypeName(*m_classAd, "Replication");
+    SetTargetTypeName(*m_classAd, "");
 
-    m_name.sprintf( "replication@%s -p %d", my_full_hostname( ),
+    m_name.formatstr( "replication@%s -p %d", my_full_hostname( ),
 				  daemonCore->InfoCommandPort( ) );
     m_classAd->Assign( ATTR_NAME, m_name.Value( ) );
     m_classAd->Assign( ATTR_MY_ADDRESS,
