@@ -50,14 +50,6 @@
 #include "cedar_enums.h"
 #include "../condor_sysapi/sysapi.h"
 
-/**********************************************************************
-** C++ specific stuff 
-**********************************************************************/
-#if defined(__cplusplus)
-
-
-#endif /* __cplusplus */
-
 /*********************************************************************
 ** On WinNT, we _must_ make redefine the assert() macro to be EXCEPT.
 ** One reason we must do this is the system assert() on NT is a
@@ -78,11 +70,11 @@
 /*********************************************************************
 ** On Windows Server 2003 SDK, it does not have inet_pton/inet_ntop
 ** Instead, they have windows version of it.
-** Here, it includes condor_win_ipv6.h which implements inet_pton/
+** Here, it includes condor_ipv6.Windows.h which implements inet_pton/
 ** inet_ntop using Winsock function.
 ** Currently, only inet_pton is implemented.
 *********************************************************************/
-#include "condor_win_ipv6.h"
+#include "condor_ipv6.WINDOWS.h"
 #endif
 
 
