@@ -28,11 +28,15 @@
 extern "C" {
 #endif
 
-struct hostent *
-condor_gethostbyname(const char *name);
 
-struct hostent *
-condor_gethostbyaddr(const char *addr, SOCKET_LENGTH_TYPE len, int type);
+// do not use gethostbyname and gethostbyaddr
+// use resolve_hsotname, get_hostname instead
+
+//struct hostent *
+//condor_gethostbyname(const char *name);
+//
+//struct hostent *
+//condor_gethostbyaddr(const char *addr, SOCKET_LENGTH_TYPE len, int type);
 
 int
 condor_gethostname(char *name, size_t namelen);
