@@ -50,7 +50,7 @@ const condor_mode_t NULL_FILE_PERMISSIONS = (condor_mode_t)0;
     We need to define a special code() method for certain integer arguments.
     To take advantage of overloading, we need make these arguments have a
     new type.  Since typedef doesn't actually create a new type in C++, we
-    use enum.  Note that we explicitly define MAXINT as an allowable value
+    use enum.  Note that we explicitly define INT_MAX as an allowable value
     for the enum to make sure sizeof(enum) == sizeof(int).
 */
 
@@ -292,7 +292,6 @@ public:
     ///
 	int code_array(gid_t *&array, int &len);
     ///
-	int code(struct utsname &);
 #endif // !defined(WIN32)
 	///
 	int code(condor_mode_t &);
@@ -361,7 +360,6 @@ public:
 	int code(struct utimbuf *x)		{ return code(*x); }
     ///
 	int code(struct rlimit *x)		{ return code(*x); }
-	int code(struct utsname *x)		{ return code(*x); }
 #endif // !defined(WIN32)
 
 #if HAS_64BIT_STRUCTS
