@@ -125,7 +125,7 @@ int Buf::flush(
 	rewind();
 
 /* DEBUG SESSION
-	if ((dbg_fd = safe_open_wrapper("trace.snd", O_WRONLY|O_APPEND|O_CREAT, 0700)) < 0){
+	if ((dbg_fd = safe_open_wrapper_follow("trace.snd", O_WRONLY|O_APPEND|O_CREAT, 0700)) < 0){
 		dprintf(D_ALWAYS, "IO: Error opening trace file\n");
 		exit(1);
 	}
@@ -170,7 +170,7 @@ int Buf::read(
 	_dta_sz += nr;
 
 /* DEBUG SESSION
-	if ((dbg_fd = safe_open_wrapper("trace.rcv", O_WRONLY|O_APPEND|O_CREAT, 0700)) < 0){
+	if ((dbg_fd = safe_open_wrapper_follow("trace.rcv", O_WRONLY|O_APPEND|O_CREAT, 0700)) < 0){
 		dprintf(D_ALWAYS, "IO: Error opening trace file\n");
 		exit(1);
 	}
