@@ -29,7 +29,6 @@
 
 #include "classad/classad_distribution.h"
 #include "classad_oldnew.h"
-//#include "conversion.h"
 using namespace std;
 
 #include <list>
@@ -195,7 +194,7 @@ Tests::cmdLine( int argc, const char *argv[] )
 
 		if (  (m_op == OP_NONE)  &&  arg.Match('d', "debug")  ) {
 			if ( arg.hasOpt() ) {
-				set_debug_flags( const_cast<char *>(arg.getOpt()) );
+				set_debug_flags( const_cast<char *>(arg.getOpt()), 0 );
 				argno = arg.ConsumeOpt( );
 			} else {
 				fprintf(stderr, "Value needed for '%s'\n", arg.Arg() );
@@ -813,7 +812,7 @@ Tests::getListConst( bool selected_only ) const
 int
 main(int argc, const char **argv)
 {
-	DebugFlags = D_ALWAYS;
+	set_debug_flags(NULL, D_ALWAYS);
 
 	set_mySubSystem( "TEST_LEASE_MANAGER", SUBSYSTEM_TYPE_TOOL );
 
@@ -822,8 +821,7 @@ main(int argc, const char **argv)
 	config();
 
 		// Set up the dprintf stuff...
-	Termlog = true;
-	dprintf_config("TEST_LEASE_MANAGER", get_param_functions());
+	dprintf_set_tool_debug("TEST_LEASE_MANAGER", 0);
 
 	Tests	tests;
 	int		status;
