@@ -20,6 +20,7 @@
 
 #include "condor_common.h"
 #include "condor_debug.h"
+#include "selector.h"
 #include "named_pipe_writer.unix.h"
 #include "named_pipe_watchdog.unix.h"
 
@@ -80,20 +81,20 @@ NamedPipeWriter::~NamedPipeWriter()
 void
 NamedPipeWriter::set_watchdog(NamedPipeWatchdog* watchdog)
 {
-	ASSERT(m_initialized);
+	assert(m_initialized);
 	m_watchdog = watchdog;
 }
 
 bool
 NamedPipeWriter::write_data(void* buffer, int len)
 {
-	ASSERT(m_initialized);
+	assert(m_initialized);
 
 	// if we're writing to a pipe that has multiple writers,
 	// we need to make sure our messages are no larger than
 	// PIPE_BUF to guarantee atomic writes
 	//
-	ASSERT(len <= PIPE_BUF);
+	assert(len <= PIPE_BUF);
 
 	// if we have a watchdog, we don't go right into a blocking
 	// write. instead, we select with both the real pipe and the
@@ -101,23 +102,19 @@ NamedPipeWriter::write_data(void* buffer, int len)
 	// crashes
 	//
 	if (m_watchdog != NULL) {
-		fd_set write_fd_set;
-		FD_ZERO(&write_fd_set);
-		FD_SET(m_pipe, &write_fd_set);
 		int watchdog_pipe = m_watchdog->get_file_descriptor();
-		fd_set read_fd_set;
-		FD_ZERO(&read_fd_set);
-		FD_SET(watchdog_pipe, &read_fd_set);
-		int max_fd = (m_pipe > watchdog_pipe) ? m_pipe : watchdog_pipe;
-		int ret = select(max_fd + 1, &read_fd_set, &write_fd_set, NULL, NULL);
-		if (ret == -1) {
+		Selector selector;
+		selector.add_fd( m_pipe, Selector::IO_WRITE );
+		selector.add_fd( watchdog_pipe, Selector::IO_READ );
+		selector.execute();
+		if ( selector.failed() || selector.signalled() ) {
 			dprintf(D_ALWAYS,
 			        "select error: %s (%d)\n",
-			        strerror(errno),
-			        errno);
+			        strerror(selector.select_errno()),
+			        selector.select_errno());
 			return false;
 		}
-		if (FD_ISSET(watchdog_pipe, &read_fd_set)) {
+		if ( selector.fd_ready( watchdog_pipe, Selector::IO_READ ) ) {
 			dprintf(D_ALWAYS,
 			        "error writing to named pipe: "
 			            "watchdog pipe has closed\n");
