@@ -75,10 +75,6 @@ main(int argc, char* argv[])
 		return 1;
 	}
 
-	config();
-	Termlog = 1;
-	dprintf_config("TOOL", get_param_functions());
-
 	int cmd_argc = argc - 1;
 	char** cmd_argv = argv + 1;
 
@@ -129,11 +125,8 @@ main(int argc, char* argv[])
 	// If a procd address wasn't specified on the command line, see if we
 	// have an entry in a config file to use.
 	if (procd_address == NULL) {
-		procd_address = param("PROCD_ADDRESS");
-		if (procd_address == NULL) {
-			fprintf(stderr, "error: PROCD_ADDRESS not defined\n");
-			return 1;
-		}
+		fprintf(stderr, "error: procd address (-A) not specified.\n");
+		return 1;
 	}
 
 	
