@@ -25,6 +25,7 @@
 #include "condor_fix_assert.h"
 #include "condor_secman.h"
 #include "condor_attributes.h"
+#include "condor_uid.h"
 
 #include "../condor_syscall_lib/syscall_param_sizes.h"
 
@@ -41,8 +42,10 @@
 
 extern char *CondorCertDir;
 
+extern int active_cluster_num;
+
 static bool QmgmtMayAccessAttribute( char const *attr_name ) {
-	return !ClassAd::ClassAdAttributeIsPrivate( attr_name );
+	return !ClassAdAttributeIsPrivate( attr_name );
 }
 
 int
@@ -65,7 +68,7 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 	
 			// Authenticate socket, if not already done by daemonCore
 		if( !syscall_sock->triedAuthentication() ) {
-			if( DebugFlags & D_SECURITY ) {
+			if( IsDebugLevel(D_SECURITY) ) {
 				MyString methods;
 				SecMan::getAuthenticationMethods( WRITE, &methods );
 				dprintf(D_SECURITY,"Calling authenticate(%s) in qmgmt_receivers\n", methods.Value());
@@ -74,7 +77,7 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 			if( ! SecMan::authenticate_sock(syscall_sock, WRITE, &errstack) ) {
 					// Failed to authenticate
 				dprintf( D_ALWAYS, "SCHEDD: authentication failed: %s\n", 
-						 errstack.getFullText() );
+						 errstack.getFullText().c_str() );
 				authenticated = false;
 			}
 		}
@@ -133,7 +136,12 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 		errno = 0;
 		rval = NewCluster( );
 		terrno = errno;
-		dprintf(D_SYSCALLS,"\tNewCluster: trval = %d, errno = %d\n",rval,terrno );
+		dprintf(D_SYSCALLS, 
+				"\tNewCluster: rval = %d, errno = %d\n",rval,terrno );
+		if ( rval > 0 ) {
+			dprintf( D_AUDIT, *syscall_sock, 
+					 "Submitting new job %d.0\n", rval );
+		}
 
 		syscall_sock->encode();
 		assert( syscall_sock->code(rval) );
@@ -142,7 +150,7 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 		}
 		assert( syscall_sock->end_of_message() );;
 
-		dprintf(D_FAILURE,"schedd: NewCluster rval %d errno %d\n",rval,terrno);
+		dprintf(D_FULLDEBUG,"schedd: NewCluster rval %d errno %d\n",rval,terrno);
 
 		return 0;
 	}
@@ -160,6 +168,10 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 		rval = NewProc( cluster_id );
 		terrno = errno;
 		dprintf( D_SYSCALLS, "\trval = %d, errno = %d\n", rval, terrno );
+		if ( rval > 0 ) {
+			dprintf( D_AUDIT, *syscall_sock, 
+					 "Submitting new job %d.%d\n", cluster_id, rval );
+		}
 
 		syscall_sock->encode();
 		assert( syscall_sock->code(rval) );
@@ -168,7 +180,7 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 		}
 		assert( syscall_sock->end_of_message() );;
 
-		dprintf(D_FAILURE,"schedd: NewProc rval %d errno %d\n",rval,terrno);
+		dprintf(D_FULLDEBUG,"schedd: NewProc rval %d errno %d\n",rval,terrno);
 
 		return 0;
 	}
@@ -197,7 +209,7 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 		}
 		assert( syscall_sock->end_of_message() );;
 
-		dprintf(D_FAILURE,"schedd: DestroyProc cluster %d proc %d rval %d errno %d\n",cluster_id,proc_id,rval,terrno);
+		dprintf(D_FULLDEBUG,"schedd: DestroyProc cluster %d proc %d rval %d errno %d\n",cluster_id,proc_id,rval,terrno);
 
 		return 0;
 	}
@@ -279,6 +291,13 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 			rval = SetAttributeByConstraint( constraint, attr_name, attr_value, flags );
 			terrno = errno;
 			dprintf( D_SYSCALLS, "\trval = %d, errno = %d\n", rval, terrno );
+			if ( rval == 0 ) {
+				dprintf( D_AUDIT, *syscall_sock,
+						 "Set Attribute By Constraint %s, "
+						 "%s = %s\n",
+						 constraint, attr_name, attr_value);
+			}
+
 		}
 
 		syscall_sock->encode();
@@ -302,6 +321,8 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 		char *attr_value=NULL;
 		int terrno;
 		SetAttributeFlags_t flags = 0;
+		const char *users_username;
+		const char *condor_username;
 
 		assert( syscall_sock->code(cluster_id) );
 		dprintf( D_SYSCALLS, "	cluster_id = %d\n", cluster_id );
@@ -312,8 +333,10 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 		if( request_num == CONDOR_SetAttribute2 ) {
 			assert( syscall_sock->code( flags ) );
 		}
+		users_username = syscall_sock->getOwner();
+		condor_username = get_condor_username();
 		if (attr_name) dprintf(D_SYSCALLS,"\tattr_name = %s\n",attr_name);
-		if (attr_value) dprintf(D_SYSCALLS,"\tattr_value = %s\n",attr_value);
+		if (attr_value) dprintf(D_SYSCALLS,"\tattr_value = %s\n",attr_value);		
 		assert( syscall_sock->end_of_message() );;
 
 		// ckireyev:
@@ -333,6 +356,17 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 			rval = SetAttribute( cluster_id, proc_id, attr_name, attr_value, flags );
 			terrno = errno;
 			dprintf( D_SYSCALLS, "\trval = %d, errno = %d\n", rval, terrno );
+				// If we're modifying a previously-submitted job AND either
+				// the client's username is not HTCondor's (i.e. not a
+				// daemon) OR the client says we should log...
+			if( (cluster_id != active_cluster_num) && (rval == 0) &&
+				( strcmp(users_username, condor_username) || (flags & SHOULDLOG) ) ) { 
+
+				dprintf( D_AUDIT, *syscall_sock, 
+						 "Set Attribute for job %d.%d, "
+						 "%s = %s\n",
+						 cluster_id, proc_id, attr_name, attr_value);
+			}
 		}
 
 		free( (char *)attr_value );
@@ -377,6 +411,10 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 		rval = SetTimerAttribute( cluster_id, proc_id, attr_name, duration );
 		terrno = errno;
 		dprintf( D_SYSCALLS, "\trval = %d, errno = %d\n", rval, terrno );
+		dprintf( D_AUDIT, *syscall_sock, 
+				 "Set Timer Attribute for job %d.%d, "
+				 "attr_name = %s, duration = %d\n",
+				 cluster_id, proc_id, attr_name, duration);
 
 		syscall_sock->encode();
 		assert( syscall_sock->code(rval) );
@@ -667,7 +705,7 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 			}
 		}
 		if( rval >= 0 ) {
-			assert( updates.put(*syscall_sock) );
+			assert( putClassAd(syscall_sock, updates) );
 		}
 		assert( syscall_sock->end_of_message() );;
 		return 0;
@@ -749,10 +787,7 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 			assert( syscall_sock->code(terrno) );
 		}
 		if( rval >= 0 ) {
-			ad->SetPrivateAttributesInvisible( true );
-			assert( ad->put(*syscall_sock) );
-				// no need to unhide the private attributes, since this
-				// ad is being thrown away
+			assert( putClassAd(syscall_sock, *ad, true) );
 		}
 		// Here we must really, truely delete the ad.  Why? Because
 		// when GetJobAd is called with the third bool argument set
@@ -785,9 +820,7 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 			assert( syscall_sock->code(terrno) );
 		}
 		if( rval >= 0 ) {
-			ad->SetPrivateAttributesInvisible( true );
-			assert( ad->put(*syscall_sock) );
-			ad->SetPrivateAttributesInvisible( false );
+			assert( putClassAd(syscall_sock, *ad, true) );
 		}
 		FreeJobAd(ad);
 		free( (char *)constraint );
@@ -817,9 +850,7 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 			assert( syscall_sock->code(terrno) );
 		}
 		if( rval >= 0 ) {
-			ad->SetPrivateAttributesInvisible( true );
-			assert( ad->put(*syscall_sock) );
-			ad->SetPrivateAttributesInvisible( false );
+			assert( putClassAd(syscall_sock, *ad, true) );
 		}
 		FreeJobAd(ad);
 		assert( syscall_sock->end_of_message() );;
@@ -856,9 +887,7 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 			assert( syscall_sock->code(terrno) );
 		}
 		if( rval >= 0 ) {
-			ad->SetPrivateAttributesInvisible( true );
-			assert( ad->put(*syscall_sock) );
-			ad->SetPrivateAttributesInvisible( false );
+			assert( putClassAd(syscall_sock, *ad, true) );
 		}
 		FreeJobAd(ad);
 		free( (char *)constraint );
@@ -895,9 +924,7 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 			assert( syscall_sock->code(terrno) );
 		}
 		if( rval >= 0 ) {
-			ad->SetPrivateAttributesInvisible( true );
-			assert( ad->put(*syscall_sock) );
-			ad->SetPrivateAttributesInvisible( false );
+			assert( putClassAd(syscall_sock, *ad, true) );
 		}
 		FreeJobAd(ad);
 		free( (char *)constraint );
@@ -934,7 +961,7 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 		int terrno;
 
 		ClassAd ad;
-		assert( ad.initFromStream(*syscall_sock) );
+		assert( getClassAd(syscall_sock, ad) );
 		assert( syscall_sock->end_of_message() );;
 
 		errno = 0;
@@ -1007,13 +1034,9 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 						}
 					}
 
-					ad->SetPrivateAttributesInvisible( true );
-					assert( ad->put(*syscall_sock,&internals) );
-					ad->SetPrivateAttributesInvisible( false );
+					assert( putClassAd(syscall_sock, *ad, true, &internals) );
 				} else {
-					ad->SetPrivateAttributesInvisible( true );
-					assert( ad->put(*syscall_sock) );
-					ad->SetPrivateAttributesInvisible( false );
+					assert( putClassAd(syscall_sock, *ad, true) );
 				}
 				FreeJobAd(ad);
 			}
