@@ -36,6 +36,9 @@ use Condor;
 use CondorUtils;
 use CondorPersonal;
 
+use base 'Exporter';
+
+our @EXPORT = qw(runCondorTool runToolNTimes);
 my %securityoptions =
 (
 "NEVER" => "1",
@@ -406,6 +409,27 @@ sub RegisterRelease
 
     $test{$handle}{"RegisterRelease"} = $function_ref;
 }
+sub RegisterDisconnected
+{
+    my $handle = shift || croak "missing handle argument";
+    my $function_ref = shift || croak "missing function reference argument";
+
+    $test{$handle}{"RegisterDisconnected"} = $function_ref;
+}
+sub RegisterReconnected
+{
+    my $handle = shift || croak "missing handle argument";
+    my $function_ref = shift || croak "missing function reference argument";
+
+    $test{$handle}{"RegisterReconnected"} = $function_ref;
+}
+sub RegisterReconnectFailed
+{
+    my $handle = shift || croak "missing handle argument";
+    my $function_ref = shift || croak "missing function reference argument";
+
+    $test{$handle}{"RegisterReconnectFailed"} = $function_ref;
+}
 sub RegisterJobErr
 {
     my $handle = shift || croak "missing handle argument";
@@ -413,6 +437,13 @@ sub RegisterJobErr
 
     $test{$handle}{"RegisterJobErr"} = $function_ref;
 }
+sub RegisterULog
+{
+    my $handle = shift || croak "missing handle argument";
+    my $function_ref = shift || croak "missing function reference argument";
+
+    $test{$handle}{"RegisterULog"} = $function_ref;
+}
 
 sub RegisterTimed
 {
@@ -591,12 +622,31 @@ END
 		$want_checkpoint = 0;
 	}
 
-	return DoTest($name, $submit_file, $want_checkpoint, $args{dagman_args});
+	my $undead = undef;
+	return DoTest($name, $submit_file, $want_checkpoint, $undead, $args{dagman_args});
 }
  
+###################################################################################
+##
+## We want to pass a call back function to secure the cluster id after DoTest
+## submits the job. We were expecting 3 defined values and one possible. We are adding
+## another possible which will have stack location alllowing an undefined then
+## followed by a defined on for dag args. We want this to work for dag tests too
+## so we will expand the 4 dag args to 5 with an undef at poision 3 , the fourth element
+##
+###################################################################################
+
 sub RunDagTest
 {
-	DoTest(@_);
+	my $undead = undef;
+	my $count = 0;
+	$count = @_;
+	if($count == 5) {
+		DoTest(@_);
+	} else {
+		my @newrgs = ($_[0],$_[1],$_[2],$undead,$_[3]);
+		DoTest(@newrgs);
+	}
 }
 
 sub DoTest
@@ -604,6 +654,7 @@ sub DoTest
     $handle              = shift || croak "missing handle argument";
     $submit_file      = shift || croak "missing submit file argument";
     my $wants_checkpoint = shift;
+	my $clusterIDcallback = shift;
 	my $dagman_args = 	shift;
 
     my $status           = -1;
@@ -681,6 +732,10 @@ sub DoTest
     	$cluster = Condor::TestSubmitDagman( $submit_file, $dagman_args );
 	}
     
+	if(defined $clusterIDcallback) {
+		&$clusterIDcallback($cluster);
+	}
+
     # if condor_submit failed for some reason return an error
     if($cluster == 0){
 		print "Why is cluster 0 in RunTest??????\n";
@@ -948,6 +1003,18 @@ sub CheckRegistrations
 	} );
     }
 
+    if( defined $test{$handle}{"RegisterULog"} )
+    {
+	Condor::RegisterULog( $test{$handle}{"RegisterULog"} );
+    }
+    else
+    {
+	Condor::RegisterULog( sub {
+	    my %info = @_;
+	    die "$handle: FAILURE (job ulog)\n";
+	} );
+    }
+
     # if we wanted to know about requeues.....
     if( defined $test{$handle}{"RegisterEvictedWithRequeue"} )
     {
@@ -1238,6 +1305,7 @@ sub getJobStatus
 # upon failure and return 0 on failure.
 #
 
+
 sub runCondorTool
 {
 	my $trymultiplier = 1;
@@ -1249,32 +1317,53 @@ sub runCondorTool
 	my $arrayref = shift;
 	# use unused third arg to skip the noise like the time
 	my $quiet = shift;
-	my $options = shift;
+	my $options = shift; #hash ref
 	my $count = 0;
+	my %altoptions = ();
+	my $failconcerns = 1;
+
+	if(exists ${$options}{expect_result}) {
+		$failconcerns = 0;
+	}
+
+	# provide an expect_result=>ANY as a hash reference options
+	$altoptions{expect_result} = \&ANY;
+	if(defined $options) {
+		#simply use altoptions
+		${$options}{expect_result} = \&ANY;
+	} else {
+		$options = \%altoptions;
+	}
+	#Condor::DebugLevel(4);
 
 	# clean array before filling
 
-	my $attempts = 6;
+	my $attempts = 15;
 	$count = 0;
 	my $hashref;
 	while( $count < $attempts) {
+		#print "runCondorTool: Attempt: <$count>\n";
+
+		# Add a message to runcmd output
+		${$options}{emit_string} = "runCondorTool: Attempt: $count";
 		@{$arrayref} = (); #empty return array...
 		my @tmparray;
-		debug( "Try command <$cmd>\n",4);
+		debug( "Try command: $cmd\n",4);
 		#open(PULL, "_condor_TOOL_TIMEOUT_MULTIPLIER=4 $cmd 2>$catch |");
 
-		$hashref = runcmd("_condor_TOOL_TIMEOUT_MULTIPLIER=4 $cmd", $options);
+		$hashref = runcmd("_condor_TOOL_TIMEOUT_MULTIPLIER=10 $cmd", $options);
 		my @output =  @{${$hashref}{"stdout"}};
 		my @error =  @{${$hashref}{"stderr"}};
 
 		$status = ${$hashref}{"exitcode"};
+		#print "runCondorTool: Status was <$status>\n";
 		debug("Status is $status after command\n",4);
-		if( $status != 0 ) {
+		if(( $status != 0 ) && ($failconcerns == 1)){
 				#print "************* std out ***************\n";
 				#print "************* std err ***************\n";
-				print "************* GetQueue() ***************\n";
+				#print "************* GetQueue() ***************\n";
 				GetQueue();
-				print "************* GetQueue() DONE ***************\n";
+				#print "************* GetQueue() DONE ***************\n";
 		} else {
 
 			my $line = "";
@@ -1300,17 +1389,72 @@ sub runCondorTool
 			my $current_time = time;
 			$delta_time = $current_time - $start_time;
 			debug("runCondorTool: its been $delta_time since call\n",4);
+			#Condor::DebugLevel(2);
 			return(1);
 		}
 		$count = $count + 1;
-		debug("runCondorTool: iteration<$count> failed sleep 10 * $count \n",1);
+		debug("runCondorTool: iteration: $count failed sleep 10 * $count \n",1);
+		my $delaynow = 10*$count;
+		if(!defined $quiet) {
+			print "runCondorTool: this delay: $delaynow\n";
+		}
 		sleep((10*$count));
 	}
 	debug( "runCondorTool: $cmd failed!\n",1);
+	#Condor::DebugLevel(2);
 
 	return(0);
 }
 
+sub runToolNTimes
+{
+	my $cmd = shift;
+    my $goal = shift;
+    my $wantoutput = shift;
+
+    my $count = 0;
+    my $stop = 0;
+    my @cmdout = ();
+    my @date = ();
+	my @outarrray;
+	my $cmdstatus = 0;
+    $stop = $goal;
+
+    while($count < $stop) {
+        @cmdout = ();
+		@outarrray = ();
+        @date = ();
+        @date = `date`;
+        CondorUtils::fullchomp $date[0];
+        #print "$date[0] $cmd $count\n";
+        #@cmdout = `$cmd`;
+        if(defined $wantoutput) {
+			if($wantoutput == 0) {
+				#print "runToolNTimes quiet mode requested for: $cmd\n";
+				$cmdstatus = runCondorTool($cmd, \@outarrray, 2, {emit_output=>0});
+			} else {
+				# be verbose about it
+				$cmdstatus = runCondorTool($cmd, \@outarrray, 2);
+			}
+		} else {
+			# be verbose about it
+			$cmdstatus = runCondorTool($cmd, \@outarrray, 2);
+		}
+		if(!$cmdstatus) {
+			print "runCondorTool: $cmd attempt: $count SHOULD NOT fail!\n";
+		}
+
+        if(defined $wantoutput) {
+			if($wantoutput != 0) {
+            	foreach my $line (@outarrray) {
+                	print "$line";
+            	}
+			}
+        }
+        $count += 1;
+    }
+	return($cmdstatus);
+}
 
 # Lets be able to drop some extra information if runCondorTool
 # can not do what it is supposed to do....... short and full
@@ -1587,7 +1731,7 @@ sub getFqdnHost {
 #
 # SearchCondorLog
 #
-# Serach a log for a regexp pattern
+# Search a log for a regexp pattern
 #
 ##############################################################################
 
@@ -1599,7 +1743,7 @@ sub SearchCondorLog
     my $logloc = `condor_config_val ${daemon}_log`;
     CondorUtils::fullchomp($logloc);
 
-    CondorTest::debug("Search this log <$logloc> for <$regexp>\n",2);
+    CondorTest::debug("Search this log <$logloc> for <$regexp>\n",3);
     open(LOG,"<$logloc") || die "Can not open logfile<$logloc>: $!\n";
     while(<LOG>) {
         if( $_ =~ /$regexp/) {
@@ -1610,6 +1754,216 @@ sub SearchCondorLog
     return(0);
 }
 
+##############################################################################
+#
+# SearchCondorLogMultiple`
+#
+# Search a log for a regexp pattern N times on some interval
+# 
+# May 2103 extension by bt
+#
+# Find N new patterns in the file lets you wait for a new say negotiator
+# 	cycle to start. ($findnew [true/false]
+#
+# Find the next pattern after this pattern (next $findafter after regexp) 
+#
+# Find number of matches between two patterns and use call back for count.
+#	Count $findbetween which come after $regexp but before $findafter
+#
+#	see test job_services_during_neg_cycle.run
+#
+#
+##############################################################################
+
+sub SearchCondorLogMultiple
+{
+    my $daemon = shift;
+    my $regexp = shift;
+    my $instances = shift;
+    my $timeout = shift;
+    my $findnew = shift;
+    my $findcallback = shift;
+    my $findafter = shift;
+    my $findbetween = shift;
+    my $currentcount = 0;
+    my $found = 0;
+    my $tried = 0;
+    my $goal = 0;
+    my $retryregexp = "";
+
+    my $logloc = `condor_config_val ${daemon}_log`;
+    CondorUtils::fullchomp($logloc);
+    CondorTest::debug("Search this log: $logloc for: $regexp instances = $instances\n",2);
+    CondorTest::debug("Timeout = $timeout\n",2);
+
+    # do we want to see X new events
+    if($findnew eq "true") {
+        # find current event count
+        open(LOG,"<$logloc") || die "Can not open logfile: $logloc: $!\n";
+        while(<LOG>) {
+            if( $_ =~ /$regexp/) {
+                CondorTest::debug("FOUND IT! $_\n",2);
+                $currentcount += 1;
+                #print "FOUND IT! $_";
+            } else {
+                #CondorTest::debug(".",2);
+                #print "Skipping: $_";
+            }
+        }
+        close(LOG);
+        $goal = $currentcount + $instances;
+        CondorTest::debug("Raised request to $goal since current count is $currentcount\n",2);
+    } else {
+        $goal = $instances;
+        #print "SearchCondorLogMultiple: goal: $goal\n";
+    }
+
+	my $count = 0;
+    my $begin = 0;
+    my $foundanything = 0;
+    my $showdots = 0;
+    #my $tolerance = 5;
+    my $done = 0;
+    while($found < $goal) {
+        CondorTest::debug("Searching Try $tried\n",2);
+        $found = 0;
+        open(LOG,"<$logloc") || die "Can not open logfile: $logloc: $!\n";
+        while(<LOG>) {
+            CondorUtils::fullchomp($_);
+            if(defined $findbetween) {
+                # start looking for between string after first pattern
+                # and stop when you find after string. call match callback
+                # with actual count.
+                if( $_ =~ /$regexp/) {
+                    CondorTest::debug("Found start: $_\n",2);
+                    $begin = 1;
+                    $goal = 100000;
+                } elsif( $_ =~ /$findafter/) {
+                    CondorTest::debug("Found done: $_\n",2);
+                    $done = 1;
+                    if(defined $findcallback) {
+                         &$findcallback($count);
+                    }
+                    $found = $goal;
+                    last;
+                } elsif($_ =~ /$findbetween/) {
+                    if($begin == 1) {
+                        $count += 1;
+
+                        CondorTest::debug("Found Match: $_\n",2);
+                    }
+                } else {
+                    #print ".";
+                }
+            } elsif( $_ =~ /$regexp/) {
+                CondorTest::debug("FOUND IT! $_\n",2);
+                $found += 1;
+                $foundanything += 1;
+                if($showdots == 1) {
+                    print "instances $instances found $found goal $goal: $_\n";
+                }
+                if((defined $findcallback) and (!(defined $findafter)) and
+                     ($found == $goal)) {
+                     #print "Found after: $_\n";
+                    &$findcallback($_);
+                }
+                if((defined $findcallback) and (defined $findafter) and
+                     ($found == $goal) and ($findafter eq $regexp)) {
+                    &$findcallback($_);
+                    #print "find calllback and exit\n";
+                    return(1);
+                }
+                if((defined $findafter) and ($found == $goal)) {
+                    # change the pattern we are looking for. really only
+                    # works well when looking for one particular item.
+                    # undef the second pattern so we get a crack at the callback
+                    $found = 0;
+                    $goal = 1;
+                    #print "Found $regexp now looking for $findafter\n";
+                    #$showdots = 1;
+                    # if we retry we want to start looking for the after
+                    # pattern when we get through the first search.
+                    $retryregexp = $regexp;
+                    $regexp = $findafter;
+                    #$findafter = undef;
+                }
+            } else {
+                #CondorTest::debug(".",2);
+                    if($showdots == 1) {
+                            print ".";
+                }
+            }
+        } # log done
+		close(LOG);
+        CondorTest::debug("Found: $found want: $goal\n",2);
+        if($found < $goal) {
+            if($retryregexp ne "") {
+                $regexp = $retryregexp;
+            }
+            sleep 1;
+        } else {
+            #Done
+            last;
+        }
+        $tried += 1;
+        if($tried >= $timeout) {
+                CondorTest::debug("SearchCondorLogMultiple: About to fail from timeout!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n",1);
+                if(defined $findcallback) {
+                    &$findcallback("HitRetryLimit");
+                }
+                last;
+        }
+    }
+    if($found < $goal) {
+        return(0);
+    } else {
+        return(1);
+    }
+}
+
+
+##############################################################################
+##
+## SearchCondorSpecialLog
+##
+## Serach a log for a regexp pattern
+## Log not a daemon log but in log location(NEGOTIATOR_MATCH_LOG)
+##
+###############################################################################
+
+
+sub SearchCondorSpecialLog
+{
+    my $logname = shift;
+    my $regexp = shift;
+    my $allmatch = shift;
+
+    my $matches = 0;
+    my $logloc = `condor_config_val log`;
+    CondorUtils::fullchomp($logloc);
+    $logloc = "$logloc/$logname";
+
+    #print "SearchCondorSpecialLog: $logname/$regexp/$allmatch\n";
+    CondorTest::debug("Search this log <$logloc> for <$regexp>\n",2);
+    open(LOG,"<$logloc") || die "Can not open logfile<$logloc>: $!\n";
+    while(<LOG>) {
+        if( $_ =~ /$regexp/) {
+            CondorTest::debug("FOUND IT! $_",2);
+            $matches += 1;
+        } else {
+            if($allmatch != 0) {
+                CondorTest::debug("Search this log <$logname> for <$regexp> no none matching lines allowed\n",2);
+                CondorTest::debug("Failing found $_\n",2);
+                return(0);
+            }
+        }
+    }
+    if($matches > 0) {
+        return(1);
+    }
+    return(0);
+}
+
 ##############################################################################
 #
 # PersonalPolicySearchLog
@@ -1731,12 +2085,10 @@ sub findOutput
 
 # Call down to Condor Perl Module for now
 
-sub debug
-{
+sub debug {
     my $string = shift;
-	my $level = shift;
-	my $newstring = "CT:$string";
-	Condor::debug($newstring,$level);
+    my $level = shift;
+    Condor::debug("<CondorTest> $string", $level);
 }
 
 
@@ -1918,7 +2270,7 @@ sub KillPersonal
 		$logdir = $1 . "/log";
 	} else {
 		debug("KillPersonal passed this config<<$personal_config>>\n",2);
-		die "Can not extract log directory\n";
+		e ie "Can not extract log directory\n";
 	}
 	debug("Doing core ERROR check in  KillPersonal\n",2);
 	$failed_coreERROR = CoreCheck($handle, $logdir, $teststrt, $teststop);
@@ -2276,7 +2628,8 @@ sub AddRunningTest {
     my $test = shift;
     my $runningfile = FindControlFile();
     debug( "Adding <$test> to running tests\n",$debuglevel);
-    runcmd("touch $runningfile/$test");
+    open(OUT, '>', '$runningfile/$test');
+    close(OUT);
 }
 
 sub RemoveRunningTest {
@@ -2311,6 +2664,19 @@ sub WriteFileOrDie
 	close OUT;
 }
 
-
+# we want to produce a temporary file to use as a fresh start
+# through StartCondorWithParams. 
+sub CreateLocalConfig
+{
+    my $text = shift;
+    my $name = shift;
+    $name = "$name$$";
+    open(FI,">$name") or die "Failed to create local config starter file<$name>:$!\n";
+    print "Created <$name>\n";
+    print FI "$text";
+    runcmd("cat $name");
+    close(FI);
+    return($name);
+}
 
 1;
