@@ -158,6 +158,7 @@ sub runcmd {
 		die_on_failed_expectation
 		emit_output
 		expect_result
+		emit_string
 		use_system
 		sh_wrap
 		);
@@ -189,7 +190,12 @@ sub runcmd {
 	my @errlines;
 
 	if(${$options}{emit_output} == TRUE) {
-		PrintHeader();
+		if(exists ${$options}{emit_string}) {
+			PrintAddComment(${$options}{emit_string});
+			PrintAltHeader();
+		} else {
+			PrintHeader();
+		}
 		PrintStart($date,$args);
 	}
 
@@ -431,6 +437,18 @@ sub PrintStdErr {
 	}
 }
 
+
+sub PrintAddComment
+{
+	my $message = shift;
+	print "\n+-------------------------------------------------------------------------------\n";
+	print "+ $message\n";
+}
+
+sub PrintAltHeader {
+	print "+-------------------------------------------------------------------------------\n";
+}
+
 sub PrintHeader {
 	print "\n+-------------------------------------------------------------------------------\n";
 }
