@@ -149,7 +149,6 @@ int
 pseudo_shell( char *command, int /*len*/ )
 {
 	int rval;
-	int terrno;
 
 	dprintf( D_SYSCALLS, "\tcommand = \"%s\"\n", command );
 
@@ -1699,7 +1698,11 @@ pseudo_chdir( const char *path )
 int
 pseudo_register_fs_domain( const char *fs_domain )
 {
-	strcpy( Executing_Filesystem_Domain, fs_domain );
+	if( strlen ( fs_domain ) < MAX_STRING ) {
+		strcpy( Executing_Filesystem_Domain, fs_domain );
+	} else {
+		EXCEPT( "Attempt to overflow Executing_Filesystem_Domain with %s", fs_domain );
+	}
 	dprintf( D_SYSCALLS, "\tFS_Domain = \"%s\"\n", fs_domain );
 	return 0;
 }
@@ -1710,7 +1713,11 @@ pseudo_register_fs_domain( const char *fs_domain )
 int
 pseudo_register_uid_domain( const char *uid_domain )
 {
-	strcpy( Executing_UID_Domain, uid_domain );
+	if( strlen( uid_domain ) < MAX_STRING ) {
+		strcpy( Executing_UID_Domain, uid_domain );
+	} else {
+		EXCEPT( "Attempt to overflow Executing_UID_Domain with %s", uid_domain );
+	}
 	dprintf( D_SYSCALLS, "\tUID_Domain = \"%s\"\n", uid_domain );
 	return 0;
 }
@@ -2170,7 +2177,7 @@ pseudo_register_syscall_version( const char *job_version )
 	buf += shadow_version;
 	buf += "\n\nYou must do one of the following:\n\n";
 
-	line.sprintf(
+	line.formatstr(
 		"1) Remove your job (\"condor_rm %d.%d\"), re-link it with a\n",
 		 Proc->id.cluster, Proc->id.proc );
 	buf += line;
@@ -2183,7 +2190,7 @@ pseudo_register_syscall_version( const char *job_version )
 		"of the condor_shadow program on your submit machine.\n"
 		"In this case, once the compatible shadow is in place, you\n";
 
-	line.sprintf( "can release your job with \"condor_release %d.%d\".\n",
+	line.formatstr( "can release your job with \"condor_release %d.%d\".\n",
 			 Proc->id.cluster, Proc->id.proc );
 	buf += line;
 
