@@ -200,7 +200,7 @@ int JobExitStatus = 0;                 /* the job's exit status */
 int MaxDiscardedRunTime = 3600;
 
 extern "C" int ExceptCleanup(int,int,const char*);
-extern int Termlog;
+static int Termlog = 0;
 
 StdUnivSock	*sock_RSC1 = NULL, *RSC_ShadowInit(int rscsock, int errsock);
 StdUnivSock	*RSC_MyShadowInit(int rscsock, int errsock);
@@ -277,7 +277,7 @@ main(int argc, char *argv[] )
 
 	if( argc > 1 ) {
 		if( strcmp("-t",argv[1]) == MATCH ) {
-			Termlog++;
+			Termlog = 1;
 			argv++;
 			argc--;
 		}
@@ -298,7 +298,10 @@ main(int argc, char *argv[] )
 	*/ 
 	set_condor_priv();
 
-	dprintf_config( get_mySubSystem()->getName(), get_param_functions() );
+	if(Termlog)
+		dprintf_set_tool_debug(get_mySubSystem()->getName(), 0);
+	else
+		dprintf_config( get_mySubSystem()->getName() );
 	DebugId = whoami;
 
 	// create a database connection object
@@ -926,8 +929,8 @@ void
 update_job_status( struct rusage *localp, struct rusage *remotep )
 {
 	int		status = -1;
-	float utime = 0.0;
-	float stime = 0.0;
+	double utime = 0.0;
+	double stime = 0.0;
 	int tot_sus=0, cum_sus=0, last_sus=0;
 	char buf[1024*50];
 
