@@ -28,6 +28,7 @@
 #include "overflow.h"
 #include <math.h>
 
+#include "slot_builder.h"
 
 ResMgr::ResMgr()
 {
@@ -235,7 +236,7 @@ ResMgr::init_config_classad( void )
 	configInsert( config_classad, "HIBERNATE", false );
 	if( !configInsert( config_classad, ATTR_UNHIBERNATE, false ) ) {
 		MyString default_expr;
-		default_expr.sprintf("MY.%s =!= UNDEFINED",ATTR_MACHINE_LAST_MATCH_TIME);
+		default_expr.formatstr("MY.%s =!= UNDEFINED",ATTR_MACHINE_LAST_MATCH_TIME);
 		config_classad->AssignExpr( ATTR_UNHIBERNATE, default_expr.Value() );
 	}
 #endif /* HAVE_HIBERNATION */
@@ -259,7 +260,7 @@ ResMgr::init_config_classad( void )
 			config_classad->Assign( ATTR_CPU_BUSY, false );
 		}
 	}
-
+	
 	// Publish all DaemonCore-specific attributes, which also handles
 	// STARTD_ATTRS for us.
 	daemonCore->publish(config_classad);
@@ -433,6 +434,8 @@ ResMgr::init_resources( void )
 
     stats.Init();
 
+    m_attr->init_machine_resources();
+
 		// These things can only be set once, at startup, so they
 		// don't need to be in build_cpu_attrs() at all.
 	if (param_boolean("ALLOW_VM_CRUFT", false)) {
@@ -454,10 +457,10 @@ ResMgr::init_resources( void )
 		// string lists for each type definition.  This only happens
 		// once!  If you change the type definitions, you must restart
 		// the startd, or else too much weirdness is possible.
-	initTypes( 1 );
+	initTypes( max_types, type_strings, 1 );
 
 		// First, see how many slots of each type are specified.
-	num_res = countTypes( &type_nums, true );
+	num_res = countTypes( max_types, num_cpus(), &type_nums, true );
 
 	if( ! num_res ) {
 			// We're not configured to advertise any nodes.
@@ -469,7 +472,7 @@ ResMgr::init_resources( void )
 		// See if the config file allows for a valid set of
 		// CpuAttributes objects.  Since this is the startup-code
 		// we'll let it EXCEPT() if there is an error.
-	new_cpu_attrs = buildCpuAttrs( num_res, type_nums, true );
+	new_cpu_attrs = buildCpuAttrs( m_attr, max_types, type_strings, num_res, type_nums, true );
 	if( ! new_cpu_attrs ) {
 		EXCEPT( "buildCpuAttrs() failed and should have already EXCEPT'ed" );
 	}
@@ -500,6 +503,19 @@ ResMgr::init_resources( void )
 #endif
 }
 
+
+bool
+ResMgr::typeNumCmp( int* a, int* b )
+{
+	int i;
+	for( i=0; i<max_types; i++ ) {
+		if( a[i] != b[i] ) {
+			return false;
+		}
+	}
+	return true;
+}
+
 bool
 ResMgr::reconfig_resources( void )
 {
@@ -527,10 +543,10 @@ ResMgr::reconfig_resources( void )
 
 		// See if any new types were defined.  Don't except if there's
 		// any errors, just dprintf().
-	initTypes( 0 );
+	initTypes( max_types, type_strings, 0 );
 
 		// First, see how many slots of each type are specified.
-	num = countTypes( &new_type_nums, false );
+	num = countTypes( max_types, num_cpus(), &new_type_nums, false );
 
 	if( typeNumCmp(new_type_nums, type_nums) ) {
 			// We want the same number of each slot type that we've got
@@ -542,7 +558,7 @@ ResMgr::reconfig_resources( void )
 
 		// See if the config file allows for a valid set of
 		// CpuAttributes objects.
-	new_cpu_attrs = buildCpuAttrs( num, new_type_nums, false );
+	new_cpu_attrs = buildCpuAttrs( m_attr, max_types, type_strings, num, new_type_nums, false );
 	if( ! new_cpu_attrs ) {
 			// There was an error, abort.  We still return true to
 			// indicate that we're done doing our thing...
@@ -642,501 +658,6 @@ ResMgr::reconfig_resources( void )
 	return processAllocList();
 }
 
-
-CpuAttributes**
-ResMgr::buildCpuAttrs( int total, int* type_num_array, bool except )
-{
-	int i, j, num;
-	CpuAttributes* cap;
-	CpuAttributes** cap_array;
-
-		// Available system resources.
-	AvailAttributes avail( m_attr );
-
-	cap_array = new CpuAttributes* [total];
-	if( ! cap_array ) {
-		EXCEPT( "Out of memory!" );
-	}
-
-	num = 0;
-	for( i=0; i<max_types; i++ ) {
-		if( type_num_array[i] ) {
-			for( j=0; j<type_num_array[i]; j++ ) {
-				cap = buildSlot( num+1, type_strings[i], i, except );
-				if( avail.decrement(cap) && num < total ) {
-					cap_array[num] = cap;
-					num++;
-				} else {
-						// We ran out of system resources.
-					dprintf( D_ALWAYS,
-							 "ERROR: Can't allocate %s slot of type %d\n",
-							 num_string(j+1), i );
-					dprintf( D_ALWAYS | D_NOHEADER, "\tRequesting: " );
-					cap->show_totals( D_ALWAYS );
-					dprintf( D_ALWAYS | D_NOHEADER, "\tAvailable:  " );
-					avail.show_totals( D_ALWAYS, cap );
-					delete cap;	// This isn't in our array yet.
-					if( except ) {
-						EXCEPT( "Ran out of system resources" );
-					} else {
-							// Gracefully cleanup and abort
-						for( i=0; i<num; i++ ) {
-							delete cap_array[i];
-						}
-						delete [] cap_array;
-						return NULL;
-					}
-				}
-			}
-		}
-	}
-
-		// now replace "auto" shares with final value
-	for( i=0; i<num; i++ ) {
-		cap = cap_array[i];
-		if( !avail.computeAutoShares(cap) ) {
-
-				// We ran out of system resources.
-			dprintf( D_ALWAYS,
-					 "ERROR: Can't allocate slot id %d during auto "
-					 "allocation of resources\n", i+1 );
-			dprintf( D_ALWAYS | D_NOHEADER, "\tRequesting: " );
-			cap->show_totals( D_ALWAYS );
-			dprintf( D_ALWAYS | D_NOHEADER, "\tAvailable:  " );
-			avail.show_totals( D_ALWAYS, cap );
-			delete cap;	// This isn't in our array yet.
-			if( except ) {
-				EXCEPT( "Ran out of system resources in auto allocation" );
-			} else {
-					// Gracefully cleanup and abort
-				for( i=0; i<num; i++ ) {
-					delete cap_array[i];
-				}
-				delete [] cap_array;
-				return NULL;
-			}
-		}
-	}
-
-	return cap_array;
-}
-
-static void
-_checkInvalidParam( const char* name, bool except ) {
-	char* tmp;
-	if ((tmp = param(name))) {
-		if (except) {
-			EXCEPT( "Can't define %s in the config file", name );
-		} else {
-			dprintf( D_ALWAYS,
-					 "Can't define %s in the config file, ignoring\n",
-					 name );
-		}
-		free(tmp);
-	}
-}
-
-void
-ResMgr::initTypes( bool except )
-{
-	int i;
-	char* tmp;
-	MyString buf;
-
-	_checkInvalidParam("SLOT_TYPE_0", except);
-		// CRUFT
-	_checkInvalidParam("VIRTUAL_MACHINE_TYPE_0", except);
-
-	if (! type_strings[0]) {
-			// Type 0 is the special type for evenly divided slots.
-		type_strings[0] = new StringList();
-		type_strings[0]->initializeFromString("auto");
-	}
-
-	for( i=1; i < max_types; i++ ) {
-		if( type_strings[i] ) {
-			continue;
-		}
-		buf.sprintf("SLOT_TYPE_%d", i);
-		tmp = param(buf.Value());
-		if (!tmp) {
-			if (param_boolean("ALLOW_VM_CRUFT", false)) {
-				buf.sprintf("VIRTUAL_MACHINE_TYPE_%d", i);
-				if (!(tmp = param(buf.Value()))) {
-					continue;
-				}
-			} else {
-				continue;
-			}
-		}
-		type_strings[i] = new StringList();
-		type_strings[i]->initializeFromString( tmp );
-		free( tmp );
-	}
-}
-
-
-int
-ResMgr::countTypes( int** array_ptr, bool except )
-{
-	int i, num=0, num_set=0;
-    MyString param_name;
-    MyString cruft_name;
-	int* my_type_nums = new int[max_types];
-
-	if( ! array_ptr ) {
-		EXCEPT( "ResMgr:countTypes() called with NULL array_ptr!" );
-	}
-
-		// Type 0 is special, user's shouldn't define it.
-	_checkInvalidParam("NUM_SLOTS_TYPE_0", except);
-		// CRUFT
-	_checkInvalidParam("NUM_VIRTUAL_MACHINES_TYPE_0", except);
-
-	for( i=1; i<max_types; i++ ) {
-		param_name.sprintf("NUM_SLOTS_TYPE_%d", i);
-		if (param_boolean("ALLOW_VM_CRUFT", false)) {
-			cruft_name.sprintf("NUM_VIRTUAL_MACHINES_TYPE_%d", i);
-			my_type_nums[i] = param_integer(param_name.Value(),
-											 param_integer(cruft_name.Value(),
-														   0));
-		} else {
-			my_type_nums[i] = param_integer(param_name.Value(), 0);
-		}
-		if (my_type_nums[i]) {
-			num_set = 1;
-			num += my_type_nums[i];
-		}
-	}
-
-	if( num_set ) {
-			// We found type-specific stuff, use that.
-		my_type_nums[0] = 0;
-	} else {
-			// We haven't found any special types yet.  Therefore,
-			// we're evenly dividing things, so we only have to figure
-			// out how many nodes to advertise.
-		if (param_boolean("ALLOW_VM_CRUFT", false)) {
-			my_type_nums[0] = param_integer("NUM_SLOTS",
-										  param_integer("NUM_VIRTUAL_MACHINES",
-														num_cpus()));
-		} else {
-			my_type_nums[0] = param_integer("NUM_SLOTS", num_cpus());
-		}
-		num = my_type_nums[0];
-	}
-	*array_ptr = my_type_nums;
-	return num;
-}
-
-
-bool
-ResMgr::typeNumCmp( int* a, int* b )
-{
-	int i;
-	for( i=0; i<max_types; i++ ) {
-		if( a[i] != b[i] ) {
-			return false;
-		}
-	}
-	return true;
-}
-
-
-CpuAttributes*
-ResMgr::buildSlot( int slot_id, StringList* list, int type, bool except )
-{
-	char *attr, *val;
-	int cpus=0, ram=0;
-	float disk=0, swap=0, share;
-	float default_share = AUTO_SHARE;
-
-	MyString execute_dir, partition_id;
-	GetConfigExecuteDir( slot_id, &execute_dir, &partition_id );
-
-	if ( list == NULL) {
-	  // give everything the default share and return
-
-	  cpus = compute_cpus( default_share );
-	  ram = compute_phys_mem( default_share );
-	  swap = default_share;
-	  disk = default_share;
-
-	  return new CpuAttributes( m_attr, type, cpus, ram, swap, disk, execute_dir, partition_id );
-	}
-		// For this parsing code, deal with the following example
-		// string list:
-		// "c=1, r=25%, d=1/4, s=25%"
-		// There may be a bare number (no equals sign) that specifies
-		// the default share for any items not explicitly defined.  Example:
-		// "c=1, 25%"
-
-	list->rewind();
-	while( (attr = list->next()) ) {
-		if( ! (val = strchr(attr, '=')) ) {
-				// There's no = in this description, it must be one
-				// percentage or fraction for all attributes.
-				// For example "1/4" or "25%".  So, we can just parse
-				// it as a percentage and use that for everything.
-			default_share = parse_value( attr, type, except );
-			if( default_share <= 0 && !IS_AUTO_SHARE(default_share) ) {
-				dprintf( D_ALWAYS, "ERROR: Bad description of slot type %d: ",
-						 type );
-				dprintf( D_ALWAYS | D_NOHEADER,  "\"%s\" is invalid.\n", attr );
-				dprintf( D_ALWAYS | D_NOHEADER,
-						 "\tYou must specify a percentage (like \"25%%\"), " );
-				dprintf( D_ALWAYS | D_NOHEADER, "a fraction (like \"1/4\"),\n" );
-				dprintf( D_ALWAYS | D_NOHEADER,
-						 "\tor list all attributes (like \"c=1, r=25%%, s=25%%, d=25%%\").\n" );
-				dprintf( D_ALWAYS | D_NOHEADER,
-						 "\tSee the manual for details.\n" );
-				if( except ) {
-					DC_Exit( 4 );
-				} else {
-					return NULL;
-				}
-			}
-			continue;
-		}
-
-			// If we're still here, this is part of a string that
-			// lists out seperate attributes and the share for each one.
-
-			// Get the value for this attribute.  It'll either be a
-			// percentage, or it'll be a distinct value (in which
-			// case, parse_value() will return negative.
-		if( ! val[1] ) {
-			dprintf( D_ALWAYS,
-					 "Can't parse attribute \"%s\" in description of slot type %d\n",
-					 attr, type );
-			if( except ) {
-				DC_Exit( 4 );
-			} else {
-				return NULL;
-			}
-		}
-		share = parse_value( &val[1], type, except );
-		if( share < 0.001 ) {
-				// Invalid share.
-		}
-
-			// Figure out what attribute we're dealing with.
-		switch( tolower(attr[0]) ) {
-		case 'c':
-			cpus = compute_cpus( share );
-			break;
-		case 'r':
-		case 'm':
-			ram = compute_phys_mem( share );
-			break;
-		case 's':
-		case 'v':
-			if( share > 0 || IS_AUTO_SHARE(share) ) {
-				swap = share;
-			} else {
-				dprintf( D_ALWAYS,
-						 "You must specify a percent or fraction for swap in slot type %d\n",
-						 type );
-				if( except ) {
-					DC_Exit( 4 );
-				} else {
-					return NULL;
-				}
-			}
-			break;
-		case 'd':
-			if( share > 0 || IS_AUTO_SHARE(share) ) {
-				disk = share;
-			} else {
-				dprintf( D_ALWAYS,
-						 "You must specify a percent or fraction for disk in slot type %d\n",
-						type );
-				if( except ) {
-					DC_Exit( 4 );
-				} else {
-					return NULL;
-				}
-			}
-			break;
-		default:
-			dprintf( D_ALWAYS, "Unknown attribute \"%s\" in slot type %d\n",
-					 attr, type );
-			if( except ) {
-				DC_Exit( 4 );
-			} else {
-				return NULL;
-			}
-			break;
-		}
-	}
-
-		// We're all done parsing the string.  Any attribute not
-		// listed will get the default share.
-	if( ! cpus ) {
-		cpus = compute_cpus( default_share );
-	}
-	if( ! ram ) {
-		ram = compute_phys_mem( default_share );
-	}
-	if( swap <= 0.0001 ) {
-		swap = default_share;
-	}
-	if( disk <= 0.0001 ) {
-		disk = default_share;
-	}
-
-		// Now create the object.
-	return new CpuAttributes( m_attr, type, cpus, ram, swap, disk, execute_dir, partition_id );
-}
-
-void
-ResMgr::GetConfigExecuteDir( int slot_id, MyString *execute_dir, MyString *partition_id )
-{
-	MyString execute_param;
-	char *execute_value = NULL;
-	execute_param.sprintf("SLOT%d_EXECUTE",slot_id);
-	execute_value = param( execute_param.Value() );
-	if( !execute_value ) {
-		execute_value = param( "EXECUTE" );
-	}
-	if( !execute_value ) {
-		EXCEPT("EXECUTE (or %s) is not defined in the configuration.",
-			   execute_param.Value());
-	}
-
-#if defined(WIN32)
-	int i;
-		// switch delimiter char in exec path on WIN32
-	for (i=0; execute_value[i]; i++) {
-		if (execute_value[i] == '/') {
-			execute_value[i] = '\\';
-		}
-	}
-#endif
-
-	*execute_dir = execute_value;
-	free( execute_value );
-
-		// Get a unique identifier for the partition containing the execute dir
-	char *partition_value = NULL;
-	bool partition_rc = sysapi_partition_id( execute_dir->Value(), &partition_value );
-	if( !partition_rc ) {
-		struct stat statbuf;
-		if( stat(execute_dir->Value(), &statbuf)!=0 ) {
-			int stat_errno = errno;
-			EXCEPT("Error accessing execute directory %s specified in the configuration setting %s: (errno=%d) %s",
-				   execute_dir->Value(), execute_param.Value(), stat_errno, strerror(stat_errno) );
-		}
-		EXCEPT("Failed to get partition id for %s=%s",
-			   execute_param.Value(), execute_dir->Value());
-	}
-	ASSERT( partition_value );
-	*partition_id = partition_value;
-	free(partition_value);
-}
-
-/*
-   Parse a string in one of a few formats, and return a float that
-   represents the value.  Either it's "auto", or it's a fraction, like
-   "1/4", or it's a percent, like "25%" (in both cases we'd return
-   .25), or it's a regular value, like "64", in which case, we return
-   the negative value, so that our caller knows it's an absolute
-   value, not a fraction.
-*/
-float
-ResMgr::parse_value( const char* str, int type, bool except )
-{
-	char *tmp, *foo = strdup( str );
-	float val;
-	if( strcasecmp(foo,"auto") == 0 || strcasecmp(foo,"automatic") == 0 ) {
-		free( foo );
-		return AUTO_SHARE;
-	}
-	else if( (tmp = strchr(foo, '%')) ) {
-			// It's a percent
-		*tmp = '\0';
-		val = (float)atoi(foo) / 100;
-		free( foo );
-		return val;
-	} else if( (tmp = strchr(foo, '/')) ) {
-			// It's a fraction
-		*tmp = '\0';
-		if( ! tmp[1] ) {
-			dprintf( D_ALWAYS, "Can't parse attribute \"%s\" in description of slot type %d\n",
-					 foo, type );
-			if( except ) {
-				DC_Exit( 4 );
-			} else {
-				free( foo );
-				return 0;
-			}
-		}
-		val = (float)atoi(foo) / ((float)atoi(&tmp[1]));
-		free( foo );
-		return val;
-	} else {
-			// This must just be an absolute value.  Return it as a negative
-			// float, so the caller knows it's not a fraction.
-		val = -(float)atoi(foo);
-		free( foo );
-		return val;
-	}
-}
-
-
-/*
-  Generally speaking, we want to round down for fractional amounts of
-  a CPU.  However, we never want to advertise less than 1.  Plus, if
-  the share in question is negative, it means it's an absolute value, not a
-  fraction.
-*/
-int
-ResMgr::compute_cpus( float share )
-{
-	int cpus;
-	if( IS_AUTO_SHARE(share) ) {
-			// Currently, "auto" for cpus just means 1 cpu per slot.
-		return 1;
-	}
-	if( share > 0 ) {
-		cpus = (int)floor( share * num_cpus() );
-	} else {
-		cpus = (int)floor( -share );
-	}
-	if( ! cpus ) {
-		cpus = 1;
-	}
-	return cpus;
-}
-
-
-/*
-  Generally speaking, we want to round down for fractional amounts of
-  physical memory.  However, we never want to advertise less than 1.
-  Plus, if the share in question is negative, it means it's an absolute
-  value, not a fraction.
-*/
-int
-ResMgr::compute_phys_mem( float share )
-{
-	int phys_mem;
-	if( IS_AUTO_SHARE(share) ) {
-			// This will be replaced later with an even share of whatever
-			// memory is left over.
-		return AUTO_MEM;
-	}
-	if( share > 0 ) {
-		phys_mem = (int)floor( share * m_attr->phys_mem() );
-	} else {
-		phys_mem = (int)floor( -share );
-	}
-	if( ! phys_mem ) {
-		phys_mem = 1;
-	}
-	return phys_mem;
-}
-
-
 void
 ResMgr::walk( VoidResourceMember memberfunc )
 {
@@ -1213,12 +734,12 @@ ResMgr::adlist_register( StartdNamedClassAd *ad )
 }
 
 int
-ResMgr::adlist_replace( const char *name, ClassAd *newAd, bool report_diff )
+ResMgr::adlist_replace( const char *name, const char *prefix, ClassAd *newAd, bool report_diff )
 {
 	if( report_diff ) {
 		StringList ignore_list;
-		MyString ignore = name;
-		ignore += "_LastUpdate";
+		MyString ignore = prefix;
+		ignore += "LastUpdate";
 		ignore_list.append( ignore.Value() );
 		return extra_ads.Replace( name, newAd, true, &ignore_list );
 	}
@@ -1732,7 +1253,7 @@ ResMgr::assign_load( void )
 	}
 	if( is_smp() ) {
 			// Print out the totals we already know.
-		if( DebugFlags & D_LOAD ) {
+		if( IsDebugLevel( D_LOAD ) ) {
 			dprintf( D_FULLDEBUG,
 					 "%s %.3f\t%s %.3f\t%s %.3f\n",
 					 "SystemLoad:", m_attr->load(),
@@ -1912,6 +1433,8 @@ ResMgr::addResource( Resource *rip )
 		EXCEPT("Error: attempt to add a NULL resource\n");
 	}
 
+	calculateAffinityMask(rip);
+
 	new_resources = new Resource*[nresources + 1];
 	if( !new_resources ) {
 		EXCEPT("Failed to allocate memory for new resource\n");
@@ -1932,6 +1455,7 @@ ResMgr::addResource( Resource *rip )
 
 	resources = new_resources;
 	nresources++;
+
 }
 
 
@@ -1997,6 +1521,51 @@ ResMgr::removeResource( Resource* rip )
 }
 
 
+void
+ResMgr::calculateAffinityMask( Resource *rip) {
+	int firstCore = 0;
+	int numCores  = m_attr->num_real_cpus();
+	numCores -= firstCore;
+
+	int *coreOccupancy = new int[numCores];
+	for (int i = 0; i < numCores; i++) {
+		coreOccupancy[i] = 0;
+	}
+
+	// Invert the slots' affinity mask to figure out
+	// which cpu core is already used the least.
+
+	for( int i = 0; i < nresources; i++ ) {
+		std::list<int> *cores = resources[i]->get_affinity_set();
+		for (std::list<int>::iterator it = cores->begin();
+			 it != cores->end(); it++) {
+
+			int used_core_num = *it;
+			if (used_core_num < numCores) {
+				coreOccupancy[used_core_num]++;
+			}
+		}
+	}
+
+	int coresToAssign = rip->r_attr->num_cpus();
+	while (coresToAssign--) {
+		int leastUsedCore = 0;
+		int leastUsedCoreUsage = coreOccupancy[0];
+
+		for (int i = 0; i < numCores; i++) {
+			if (coreOccupancy[i] < leastUsedCoreUsage) {
+				leastUsedCore = i;
+				leastUsedCoreUsage = coreOccupancy[i];
+			}
+		}
+
+		rip->get_affinity_set()->push_back(leastUsedCore);
+		coreOccupancy[leastUsedCore]++;
+	}
+
+	delete [] coreOccupancy;
+}
+
 void
 ResMgr::deleteResource( Resource* rip )
 {
@@ -2350,12 +1919,12 @@ ResMgr::check_use( void )
 int
 ownerStateCmp( const void* a, const void* b )
 {
-	Resource *rip1, *rip2;
+	const Resource *rip1, *rip2;
 	int val1, val2, diff;
 	float fval1, fval2;
 	State s;
-	rip1 = *((Resource**)a);
-	rip2 = *((Resource**)b);
+	rip1 = *((Resource* const *)a);
+	rip2 = *((Resource* const *)b);
 		// Since the State enum is already in the "right" order for
 		// this kind of sort, we don't need to do anything fancy, we
 		// just cast the state enum to an int and we're done.
@@ -2384,12 +1953,12 @@ ownerStateCmp( const void* a, const void* b )
 int
 claimedRankCmp( const void* a, const void* b )
 {
-	Resource *rip1, *rip2;
+	const Resource *rip1, *rip2;
 	int val1, val2, diff;
 	float fval1, fval2;
 	State s;
-	rip1 = *((Resource**)a);
-	rip2 = *((Resource**)b);
+	rip1 = *((Resource* const *)a);
+	rip2 = *((Resource* const *)b);
 
 	s = rip1->state();
 	val1 = (int)s;
@@ -2424,13 +1993,13 @@ claimedRankCmp( const void* a, const void* b )
 int
 newCODClaimCmp( const void* a, const void* b )
 {
-	Resource *rip1, *rip2;
+	const Resource *rip1, *rip2;
 	int val1, val2, diff;
 	int numCOD1, numCOD2;
 	float fval1, fval2;
 	State s;
-	rip1 = *((Resource**)a);
-	rip2 = *((Resource**)b);
+	rip1 = *((Resource* const *)a);
+	rip2 = *((Resource* const *)b);
 
 	numCOD1 = rip1->r_cod_mgr->numClaims();
 	numCOD2 = rip2->r_cod_mgr->numClaims();
@@ -2499,17 +2068,17 @@ ResMgr::startDraining(int how_fast,bool resume_on_completion,ExprTree *check_exp
 			classad::EvalState eval_state;
 			eval_state.SetScopes( resources[i]->r_classad );
 			if( !check_expr->Evaluate( eval_state, v ) ) {
-				sprintf(error_msg,"Failed to evaluate draining check expression against %s.", resources[i]->r_name );
+				formatstr(error_msg,"Failed to evaluate draining check expression against %s.", resources[i]->r_name );
 				error_code = DRAINING_CHECK_EXPR_FAILED;
 				return false;
 			}
 			if( !v.IsBooleanValue(check_ok) ) {
-				sprintf(error_msg,"Draining check expression does not evaluate to a bool on %s.", resources[i]->r_name );
+				formatstr(error_msg,"Draining check expression does not evaluate to a bool on %s.", resources[i]->r_name );
 				error_code = DRAINING_CHECK_EXPR_FAILED;
 				return false;
 			}
 			if( !check_ok ) {
-				sprintf(error_msg,"Draining check expression is false on %s.", resources[i]->r_name );
+				formatstr(error_msg,"Draining check expression is false on %s.", resources[i]->r_name );
 				error_code = DRAINING_CHECK_EXPR_FAILED;
 				return false;
 			}
@@ -2519,7 +2088,7 @@ ResMgr::startDraining(int how_fast,bool resume_on_completion,ExprTree *check_exp
 	draining = true;
 	last_drain_start_time = time(NULL);
 	draining_id += 1;
-	sprintf(new_request_id,"%d",draining_id);
+	formatstr(new_request_id,"%d",draining_id);
 	this->resume_on_completion_of_draining = resume_on_completion;
 
 	if( how_fast <= DRAIN_GRACEFUL ) {
@@ -2568,7 +2137,7 @@ ResMgr::cancelDraining(std::string request_id,std::string &error_msg,int &error_
 	}
 
 	if( !request_id.empty() && atoi(request_id.c_str()) != this->draining_id ) {
-		sprintf(error_msg,"No matching draining request id %s.",request_id.c_str());
+		formatstr(error_msg,"No matching draining request id %s.",request_id.c_str());
 		error_code = DRAINING_NO_MATCHING_REQUEST_ID;
 		return false;
 	}
@@ -2660,7 +2229,7 @@ ResMgr::getDrainingRequestId( Resource * /*rip*/, std::string &request_id )
 	if( !draining ) {
 		return false;
 	}
-	sprintf(request_id,"%d",draining_id);
+	formatstr(request_id,"%d",draining_id);
 	return true;
 }
 
