@@ -40,7 +40,7 @@
 #include "get_full_hostname.h"
 #include "condor_random_num.h"
 #include "../condor_procapi/procapi.h"
-#include "misc_utils.h"
+//#include "misc_utils.h"
 #include "get_daemon_name.h"
 #include "enum_utils.h"
 #include "condor_version.h"
