@@ -57,6 +57,12 @@ extern void main_shutdown_fast();
 const char* JOB_AD_FILENAME = ".job.ad";
 const char* MACHINE_AD_FILENAME = ".machine.ad";
 
+#ifdef WIN32
+// Note inversion of argument order...
+#define realpath(path,resolved_path) _fullpath((resolved_path),(path),_MAX_PATH)
+#endif
+
+
 /* CStarter class implementation */
 
 CStarter::CStarter()
@@ -136,7 +142,7 @@ CStarter::Init( JobInfoCommunicator* my_jic, const char* original_cwd,
 			// EXECUTE is, or our CWD if that's not defined...
 		WorkingDir = Execute;
 	} else {
-		WorkingDir.sprintf( "%s%cdir_%ld", Execute, DIR_DELIM_CHAR, 
+		WorkingDir.formatstr( "%s%cdir_%ld", Execute, DIR_DELIM_CHAR, 
 				 (long)daemonCore->getpid() );
 	}
 
@@ -216,14 +222,27 @@ CStarter::Init( JobInfoCommunicator* my_jic, const char* original_cwd,
 						  "START_SSHD",
 						  (CommandHandlercpp)&CStarter::startSSHD,
 						  "CStarter::startSSHD", this, READ );
+	daemonCore->
+		Register_Command( STARTER_PEEK,
+						"STARTER_PEEK",
+						(CommandHandlercpp)&CStarter::peek,
+						"CStarter::peek", this, READ );
+
 		// initialize our JobInfoCommunicator
 	if( ! jic->init() ) {
 		dprintf( D_ALWAYS, 
 				 "Failed to initialize JobInfoCommunicator, aborting\n" );
 		return false;
 	}
+
 	// jic already assumed to be nonzero above
+	//
+	// also, the jic->init call above has set up the user priv state via
+	// initUserPriv, so we can safely use it now, and we should so that we can
+	// actually do this in the user-owned execute dir.
+	priv_state rl_p = set_user_priv();
 	sysapi_set_resource_limits(jic->getStackSize());
+	set_priv (rl_p);
 
 		// Now, ask our JobInfoCommunicator to setup the environment
 		// where our job is going to execute.  This might include
@@ -566,7 +585,7 @@ CStarter::createJobOwnerSecSession( int /*cmd*/, Stream* s )
 	MyString error_msg;
 	ClassAd input;
 	s->decode();
-	if( !input.initFromStream(*s) || !s->end_of_message() ) {
+	if( !getClassAd(s, input) || !s->end_of_message() ) {
 		dprintf(D_ALWAYS,"Failed to read request in createJobOwnerSecSession()\n");
 		return FALSE;
 	}
@@ -653,7 +672,7 @@ CStarter::createJobOwnerSecSession( int /*cmd*/, Stream* s )
 				fqu.Value());
 	}
 
-	if( !response.put(*s) || !s->end_of_message() ) {
+	if( !putClassAd(s, response) || !s->end_of_message() ) {
 		dprintf(D_ALWAYS,
 				"createJobOwnerSecSession failed to send response\n");
 	}
@@ -665,16 +684,16 @@ CStarter::createJobOwnerSecSession( int /*cmd*/, Stream* s )
 }
 
 int
-CStarter::vSSHDFailed(Stream *s,bool retry,char const *fmt,va_list args)
+CStarter::vMessageFailed(Stream *s,bool retry, const std::string &prefix,char const *fmt,va_list args)
 {
 	MyString error_msg;
-	error_msg.vsprintf( fmt, args );
+	error_msg.vformatstr( fmt, args );
 
 		// old classads cannot handle a string ending in a double quote
 		// followed by a newline, so strip off any trailing newline
 	error_msg.trim();
 
-	dprintf(D_ALWAYS,"START_SSHD failed: %s\n",error_msg.Value());
+	dprintf(D_ALWAYS,"%s failed: %s\n", prefix.c_str(), error_msg.Value());
 
 	ClassAd response;
 	response.Assign(ATTR_RESULT,false);
@@ -684,8 +703,8 @@ CStarter::vSSHDFailed(Stream *s,bool retry,char const *fmt,va_list args)
 	}
 
 	s->encode();
-	if( !response.put(*s) || !s->end_of_message() ) {
-		dprintf(D_ALWAYS,"Failed to send response to START_SSHD.\n");
+	if( !putClassAd(s, response) || !s->end_of_message() ) {
+		dprintf(D_ALWAYS,"Failed to send response to %s.\n", prefix.c_str());
 	}
 
 	return FALSE;
@@ -696,7 +715,7 @@ CStarter::SSHDRetry(Stream *s,char const *fmt,...)
 {
 	va_list args;
 	va_start( args, fmt );
-	vSSHDFailed(s,true,fmt,args);
+	vMessageFailed(s, true, "START_SSHD", fmt, args);
 	va_end( args );
 
 	return FALSE;
@@ -706,12 +725,33 @@ CStarter::SSHDFailed(Stream *s,char const *fmt,...)
 {
 	va_list args;
 	va_start( args, fmt );
-	vSSHDFailed(s,false,fmt,args);
+	vMessageFailed(s, false, "START_SSHD", fmt, args);
 	va_end( args );
 
 	return FALSE;
 }
 
+int
+CStarter::PeekRetry(Stream *s,char const *fmt,...)
+{
+        va_list args;
+        va_start( args, fmt );
+        vMessageFailed(s, true, "PEEK", fmt, args);
+        va_end( args );
+
+        return FALSE;
+}
+int
+CStarter::PeekFailed(Stream *s,char const *fmt,...)
+{
+        va_list args;
+        va_start( args, fmt );
+        vMessageFailed(s, false, "PEEK", fmt, args);
+        va_end( args );
+
+        return FALSE;
+}
+
 // The following functions are only needed if SSH_TO_JOB is supported.
 
 #if HAVE_SSH_TO_JOB
@@ -744,7 +784,7 @@ static bool extract_delimited_data_as_base64(
 	int end = find_str_in_buffer(input_buffer,input_len,end_marker);
 	if( start < 0 ) {
 		if( error_msg ) {
-			error_msg->sprintf("Failed to find '%s' in input: %.*s",
+			error_msg->formatstr("Failed to find '%s' in input: %.*s",
 							   begin_marker,input_len,input_buffer);
 		}
 		return false;
@@ -752,7 +792,7 @@ static bool extract_delimited_data_as_base64(
 	start += strlen(begin_marker);
 	if( end < 0 || end < start ) {
 		if( error_msg ) {
-			error_msg->sprintf("Failed to find '%s' in input: %.*s",
+			error_msg->formatstr("Failed to find '%s' in input: %.*s",
 							   end_marker,input_len,input_buffer);
 		}
 		return false;
@@ -776,7 +816,7 @@ static bool extract_delimited_data(
 	int end = find_str_in_buffer(input_buffer,input_len,end_marker);
 	if( start < 0 ) {
 		if( error_msg ) {
-			error_msg->sprintf("Failed to find '%s' in input: %.*s",
+			error_msg->formatstr("Failed to find '%s' in input: %.*s",
 							   begin_marker,input_len,input_buffer);
 		}
 		return false;
@@ -784,17 +824,390 @@ static bool extract_delimited_data(
 	start += strlen(begin_marker);
 	if( end < 0 || end < start ) {
 		if( error_msg ) {
-			error_msg->sprintf("Failed to find '%s' in input: %.*s",
+			error_msg->formatstr("Failed to find '%s' in input: %.*s",
 							   end_marker,input_len,input_buffer);
 		}
 		return false;
 	}
-	output_buffer.sprintf("%.*s",end-start,input_buffer+start);
+	output_buffer.formatstr("%.*s",end-start,input_buffer+start);
 	return true;
 }
 
 #endif
 
+int
+CStarter::peek(int /*cmd*/, Stream *sock)
+{
+	// This command should only be allowed by the job owner.
+	MyString error_msg;
+	ReliSock *s = (ReliSock*)sock;
+	char const *fqu = s->getFullyQualifiedUser();
+	MyString job_owner;
+	getJobOwnerFQUOrDummy(job_owner);
+	if( !fqu || job_owner != fqu )
+	{
+		dprintf(D_ALWAYS, "Unauthorized attempt to peek at job logfiles by '%s'\n", fqu ? fqu : "");
+		return false;
+	}
+
+	compat_classad::ClassAd input;
+	s->decode();
+	if( !getClassAd(s, input) || !s->end_of_message() ) {
+		dprintf(D_ALWAYS, "Failed to read request for peeking at logs.\n");
+		return false;
+	}
+
+	compat_classad::ClassAd *jobad = NULL;
+	compat_classad::ClassAd *machinead = NULL;
+        if( jic )
+	{
+                jobad = jic->jobClassAd();
+		machinead = jic->machClassAd();
+        }
+
+	bool enabled = param_boolean("ENABLE_PEEK_JOB",true,true,machinead,jobad);
+	if( !enabled ) {
+		return PeekFailed(s, "Rejecting request, because ENABLE_PEEK_JOB=false");
+	}
+
+	if( !jic || !jobad ) {
+		return PeekRetry(s, "Rejecting request, because job not yet initialized.");
+	}
+
+	if( !m_job_environment_is_ready ) {
+		// This can happen if file transfer is still in progress.
+		return PeekRetry(s, "Rejecting request, because the job execution environment is not yet ready.");
+	}
+	if( m_all_jobs_done ) {
+		return PeekFailed(s, "Rejecting request, because the job is finished.");
+	}
+ 
+	if( !jic->userPrivInitialized() ) {
+		return PeekRetry(s, "Rejecting request, because job execution account not yet established.");
+	}
+
+	ssize_t max_xfer = -1;
+	input.EvaluateAttrInt(ATTR_MAX_TRANSFER_BYTES, max_xfer);
+
+	const char *jic_iwd = GetWorkingDir();
+	if (!jic_iwd) return PeekFailed(s, "Unknown job remote IWD.");
+	std::string iwd = jic_iwd;
+	std::vector<char> real_iwd; real_iwd.reserve(MAXPATHLEN+1);
+	if (!realpath(iwd.c_str(), &real_iwd[0]))
+	{
+		return PeekFailed(s, "Unable to resolve IWD %s to real path. (errno=%d, %s)", iwd.c_str(), errno, strerror(errno));
+	}
+	iwd = &real_iwd[0];
+
+	// Ok, sanity checks pass.  Let's iterate through the files requested and make sure they are in the
+	// stagein / stageout list.
+	//
+	std::vector<ExprTree*> file_expr_list;
+	std::vector<ExprTree*> off_expr_list;
+	std::vector<std::string> file_list;
+	std::vector<off_t> offsets_list;
+
+	// Allow the user to request stdout/err explicitly instead of by name;
+	// this is done because HTCondor will sometimes rewrite the location of the stdout/err
+	// and we don't want to have to replicate that logic in the client.
+	bool transfer_stdout = false;
+	if (input.EvaluateAttrBool(ATTR_JOB_OUTPUT, transfer_stdout) && transfer_stdout)
+	{
+		std::string out;
+		if (jobad->EvaluateAttrString(ATTR_JOB_OUTPUT, out))
+		{
+			classad::Value value; value.SetIntegerValue(0);
+			file_expr_list.push_back(classad::Literal::MakeLiteral(value));
+			file_list.push_back(out);
+			off_t stdout_off = -1;
+			input.EvaluateAttrInt("OutOffset", stdout_off);
+			offsets_list.push_back(stdout_off);
+			value.SetIntegerValue(stdout_off);
+			off_expr_list.push_back(classad::Literal::MakeLiteral(value));
+		}
+	}
+	bool transfer_stderr = false;
+	if (input.EvaluateAttrBool(ATTR_JOB_ERROR, transfer_stderr) && transfer_stderr)
+	{
+		std::string err;
+		if (jobad->EvaluateAttrString(ATTR_JOB_ERROR, err))
+		{
+			classad::Value value; value.SetIntegerValue(1);
+			file_expr_list.push_back(classad::Literal::MakeLiteral(value));
+			file_list.push_back(err);
+			off_t stderr_off = -1;
+			input.EvaluateAttrInt("ErrOffset", stderr_off);
+			offsets_list.push_back(stderr_off);
+			value.SetIntegerValue(stderr_off);
+			off_expr_list.push_back(classad::Literal::MakeLiteral(value));
+		}
+	}
+
+	classad::Value transfer_list_value;
+	std::vector<std::string> transfer_list;
+	classad_shared_ptr<classad::ExprList> transfer_list_ptr;
+	if (input.EvaluateAttr("TransferFiles", transfer_list_value) && transfer_list_value.IsSListValue(transfer_list_ptr))
+	{
+		transfer_list.reserve(transfer_list_ptr->size());
+		for (classad::ExprList::const_iterator it = transfer_list_ptr->begin();
+			it != transfer_list_ptr->end();
+			it++)
+		{
+			std::string transfer_entry;
+			classad::Value transfer_value;
+			if (!(*it)->Evaluate(transfer_value) || !transfer_value.IsStringValue(transfer_entry))
+			{
+				return PeekFailed(s, "Could not evaluate transfer list.");
+			}
+			transfer_list.push_back(transfer_entry);
+		}
+	}
+
+	std::vector<off_t> transfer_offsets; transfer_offsets.reserve(transfer_list.size());
+	for (size_t idx = 0; idx < transfer_list.size(); idx++) transfer_offsets.push_back(-1);
+
+	if (input.EvaluateAttr("TransferOffsets", transfer_list_value) && transfer_list_value.IsSListValue(transfer_list_ptr))
+	{
+		size_t idx = 0;
+		for (classad::ExprList::const_iterator it = transfer_list_ptr->begin();
+			it != transfer_list_ptr->end() && idx < transfer_list.size();
+			it++, idx++)
+		{
+			classad::Value transfer_value;
+			off_t transfer_entry;
+			if ((*it)->Evaluate(transfer_value) && transfer_value.IsIntegerValue(transfer_entry))
+			{
+				transfer_offsets[idx] = transfer_entry;
+			}
+		}
+	}
+
+	bool missing = false;
+	std::vector<off_t>::const_iterator iter2 = transfer_offsets.begin();
+	for (std::vector<std::string>::const_iterator iter = transfer_list.begin();
+		!missing && (iter != transfer_list.end()) && (iter2 != transfer_offsets.end());
+		iter++, iter2++)
+	{
+		missing = true;
+		std::string filename;
+		if (jobad->EvaluateAttrString(ATTR_JOB_ERROR, filename) && (filename == *iter))
+		{
+			missing = false;
+			if (!transfer_stderr)
+			{
+				classad::Value value; value.SetStringValue(filename);
+				file_expr_list.push_back(classad::Literal::MakeLiteral(value));
+				file_list.push_back(filename);
+				value.SetIntegerValue(*iter2);
+				off_expr_list.push_back(classad::Literal::MakeLiteral(value));
+				offsets_list.push_back(*iter2);
+			}
+			continue;
+		}
+		if (jobad->EvaluateAttrString(ATTR_JOB_OUTPUT, filename) && (filename == *iter))
+		{
+			missing = false;
+			if (!transfer_stdout)
+			{
+				classad::Value value; value.SetStringValue(filename);
+				file_expr_list.push_back(classad::Literal::MakeLiteral(value));
+				file_list.push_back(filename);
+				value.SetIntegerValue(*iter2);
+				off_expr_list.push_back(classad::Literal::MakeLiteral(value));
+				offsets_list.push_back(*iter2);
+			}
+			continue;
+		}
+		std::string job_transfer_list;
+		bool found = false;
+		if (jobad->EvaluateAttrString(ATTR_TRANSFER_OUTPUT_FILES, job_transfer_list))
+		{
+			StringList job_sl(job_transfer_list.c_str());
+			job_sl.rewind();
+			const char *job_iter;
+			while ((job_iter = job_sl.next()))
+			{
+				if (strcmp(iter->c_str(), job_iter) == 0)
+				{
+					filename = job_iter;
+					found = true;
+					break;
+				}
+			}
+		}
+		if (found)
+		{
+			missing = false;
+			classad::Value value; value.SetStringValue(filename);
+			file_expr_list.push_back(classad::Literal::MakeLiteral(value));
+			value.SetIntegerValue(*iter2);
+			off_expr_list.push_back(classad::Literal::MakeLiteral(value));
+			file_list.push_back(*iter);
+			offsets_list.push_back(*iter2);
+		}
+		else
+		{
+			return PeekFailed(s, "File %s requested but not authorized to be sent.\n", iter->c_str());
+		}
+	}
+
+	// Calculate the offsets and sizes we think we are able to do
+	std::vector<size_t> file_sizes_list; file_sizes_list.reserve(file_list.size());
+	{
+	ssize_t remaining = max_xfer;
+	TemporaryPrivSentry sentry(PRIV_USER);
+	std::vector<off_t>::iterator it2 = offsets_list.begin();
+	std::vector<char> real_filename; real_filename.reserve(MAXPATHLEN+1);
+	unsigned idx = 0;
+	for (std::vector<std::string>::iterator it = file_list.begin();
+		it != file_list.end() && it2 != offsets_list.end();
+		it++, it2++, idx++)
+	{
+		size_t size = 0;
+		off_t offset = *it2;
+
+		if (it->size() && ((*it)[0] != DIR_DELIM_CHAR))
+		{
+			*it = iwd + DIR_DELIM_CHAR + *it;
+		}
+		if (!realpath(it->c_str(), &(real_filename[0])))
+		{
+			return PeekRetry(s, "Unable to resolve file %s to real path. (errno=%d, %s)", it->c_str(), errno, strerror(errno));
+		}
+		*it = &(real_filename[0]);
+		if (it->substr(0, iwd.size()) != iwd)
+		{
+			return PeekFailed(s, "Invalid symlink or filename (%s) outside sandbox (%s)", it->c_str(), iwd.c_str());
+		}
+		dprintf(D_FULLDEBUG, "Peeking at %s inside sandbox %s.\n", it->c_str(), iwd.c_str());
+
+		int fd = safe_open_wrapper(it->c_str(), O_RDONLY);
+		struct stat stat_buf;
+		if (fd < 0)
+		{
+			dprintf(D_ALWAYS, "Cannot open file %s for stat / peeking at logs (errno=%d, %s).\n", it->c_str(), errno, strerror(errno));
+		}
+		else if (fstat(fd, &stat_buf) < 0)
+		{
+			dprintf(D_ALWAYS, "Cannot stat file %s for peeking at logs.\n", it->c_str());
+			close(fd);
+		}
+		else
+		{
+			size = stat_buf.st_size;
+			close(fd);
+		}
+		if (offset > 0 && size < static_cast<size_t>(offset))
+		{
+			offset = size;
+			*it2 = offset;
+			classad::Value value; value.SetIntegerValue(*it2);
+			off_expr_list[idx] = classad::Literal::MakeLiteral(value);
+		}
+		if (remaining >= 0)
+		{
+			if (offset < 0)
+			{
+				remaining -= size;
+				if (remaining < 0)
+				{
+					size += remaining;
+					*it2 = -remaining;
+					remaining = 0;
+				}
+				else
+				{
+					*it2 = 0;
+				}
+				classad::Value value; value.SetIntegerValue(*it2);
+				off_expr_list[idx] = classad::Literal::MakeLiteral(value);
+			}
+			else
+			{
+				if (remaining > static_cast<ssize_t>(size) - offset)
+				{
+					size -= offset;
+					remaining -= size;
+				}
+				else
+				{
+					size = remaining;
+					remaining = 0;
+				}
+			}
+		}
+		else
+		{
+			size = -1;
+			if (*it2 < 0)
+			{
+				*it2 = 0;
+				classad::Value value; value.SetIntegerValue(*it2);
+				off_expr_list[idx] = classad::Literal::MakeLiteral(value);
+			}
+		}
+		file_sizes_list.push_back(size);
+	}
+	}
+
+	compat_classad::ClassAd reply;
+	reply.InsertAttr(ATTR_RESULT, true);
+	classad::ExprTree *list = classad::ExprList::MakeExprList(file_expr_list);
+	if (!reply.Insert("TransferFiles", list))
+	{
+		return PeekFailed(s, "Failed to add transfer files list.\n");
+	}
+	list = classad::ExprList::MakeExprList(off_expr_list);
+	if (!reply.Insert("TransferOffsets", list))
+	{
+		return PeekFailed(s, "Failed to add transfer offsets list.\n");
+	}
+	dPrintAd(D_FULLDEBUG, reply);
+
+	s->encode();
+	// From here on out, *always* send the same number of files as specified by
+	// the response ad, even if it means putting an empty file.
+	if (!putClassAd(s, reply) || !s->end_of_message()) {
+		dprintf(D_ALWAYS, "Failed to send read request response for peeking at logs.\n");
+		return false;
+	}
+
+	size_t file_count = 0;
+	{
+	TemporaryPrivSentry sentry(PRIV_USER);
+	std::vector<off_t>::const_iterator it2 = offsets_list.begin();
+	std::vector<size_t>::const_iterator it3 = file_sizes_list.begin();
+	for (std::vector<std::string>::const_iterator it = file_list.begin();
+		it != file_list.end() && it2 != offsets_list.end() && it3 != file_sizes_list.end();
+		it++, it2++, it3++)
+	{
+		filesize_t size = -1;
+
+		int fd = safe_open_wrapper(it->c_str(), O_RDONLY);
+		if (fd < 0)
+		{
+			dprintf(D_ALWAYS, "Cannot open file %s for peeking at logs (errno=%d, %s).\n", it->c_str(), errno, strerror(errno));
+			s->put_empty_file(&size);
+			continue;
+		}
+		
+		s->put_file(&size, fd, *it2, *it3);
+		close(fd);
+		if (size < 0 && size != PUT_FILE_MAX_BYTES_EXCEEDED)
+		{
+			dprintf(D_ALWAYS, "Failed to send file %s for peeking at logs (%ld).\n", it->c_str(), (long)size);
+			continue;
+		}
+		file_count++;
+	}
+	}
+	if (!s->code(file_count) || !s->end_of_message())
+	{
+		dprintf(D_ALWAYS, "Failed to send file count %ld for peeking at logs.\n", file_count);
+	}
+	return file_count == file_list.size();
+}
+
 int
 CStarter::startSSHD( int /*cmd*/, Stream* s )
 {
@@ -812,7 +1225,7 @@ CStarter::startSSHD( int /*cmd*/, Stream* s )
 
 	ClassAd input;
 	s->decode();
-	if( !input.initFromStream(*s) || !s->end_of_message() ) {
+	if( !getClassAd(s, input) || !s->end_of_message() ) {
 		dprintf(D_ALWAYS,"Failed to read request in START_SSHD.\n");
 		return FALSE;
 	}
@@ -867,9 +1280,9 @@ CStarter::startSSHD( int /*cmd*/, Stream* s )
 	}
 	MyString ssh_to_job_sshd_setup;
 	MyString ssh_to_job_shell_setup;
-	ssh_to_job_sshd_setup.sprintf(
+	ssh_to_job_sshd_setup.formatstr(
 		"%s%ccondor_ssh_to_job_sshd_setup",libexec.Value(),DIR_DELIM_CHAR);
-	ssh_to_job_shell_setup.sprintf(
+	ssh_to_job_shell_setup.formatstr(
 		"%s%ccondor_ssh_to_job_shell_setup",libexec.Value(),DIR_DELIM_CHAR);
 
 	if( access(ssh_to_job_sshd_setup.Value(),X_OK)!=0 ) {
@@ -884,7 +1297,7 @@ CStarter::startSSHD( int /*cmd*/, Stream* s )
 	MyString sshd_config_template;
 	if( !param(sshd_config_template,"SSH_TO_JOB_SSHD_CONFIG_TEMPLATE") ) {
 		if( param(sshd_config_template,"LIB") ) {
-			sshd_config_template.sprintf_cat("%ccondor_ssh_to_job_sshd_config_template",DIR_DELIM_CHAR);
+			sshd_config_template.formatstr_cat("%ccondor_ssh_to_job_sshd_config_template",DIR_DELIM_CHAR);
 		}
 		else {
 			return SSHDFailed(s,"SSH_TO_JOB_SSHD_CONFIG_TEMPLATE and LIB are not defined.  At least one of them is required.");
@@ -1040,7 +1453,7 @@ CStarter::startSSHD( int /*cmd*/, Stream* s )
 
 		// look for magic success string
 	if( find_str_in_buffer(setup_output,setup_output_len,"condor_ssh_to_job_sshd_setup SUCCESS") < 0 ) {
-		error_msg.sprintf("condor_ssh_to_job_sshd_setup failed: %s",
+		error_msg.formatstr("condor_ssh_to_job_sshd_setup failed: %s",
 						  setup_output);
 		free( setup_output );
 		return SSHDFailed(s,"%s",error_msg.Value());
@@ -1107,7 +1520,7 @@ CStarter::startSSHD( int /*cmd*/, Stream* s )
 	dprintf(D_FULLDEBUG,"StartSSHD: session_dir='%s'\n",session_dir.Value());
 
 	MyString sshd_config_file;
-	sshd_config_file.sprintf("%s%csshd_config",session_dir.Value(),DIR_DELIM_CHAR);
+	sshd_config_file.formatstr("%s%csshd_config",session_dir.Value(),DIR_DELIM_CHAR);
 
 
 
@@ -1188,7 +1601,7 @@ CStarter::startSSHD( int /*cmd*/, Stream* s )
 	response.Assign(ATTR_SSH_PRIVATE_CLIENT_KEY,private_client_key.Value());
 
 	s->encode();
-	if( !response.put(*s) || !s->end_of_message() ) {
+	if( !putClassAd(s, response) || !s->end_of_message() ) {
 		dprintf(D_ALWAYS,"Failed to send response to START_SSHD.\n");
 		return FALSE;
 	}
@@ -1196,7 +1609,7 @@ CStarter::startSSHD( int /*cmd*/, Stream* s )
 
 
 	MyString sshd_log_fname;
-	sshd_log_fname.sprintf(
+	sshd_log_fname.formatstr(
 		"%s%c%s",session_dir.Value(),DIR_DELIM_CHAR,"sshd.log");
 
 
@@ -1215,7 +1628,7 @@ CStarter::startSSHD( int /*cmd*/, Stream* s )
 		dprintf(D_ALWAYS,"Failed to create SSHDProc.\n");
 		return FALSE;
 	}
-	if( !proc->StartJob(std,std_fname) ) {
+	if( !proc->SshStartJob(std,std_fname) ) {
 		dprintf(D_ALWAYS,"Failed to start sshd.\n");
 		return FALSE;
 	}
@@ -1378,10 +1791,22 @@ CStarter::createTempExecuteDir( void )
 
 	CondorPrivSepHelper* cpsh = condorPrivSepHelper();
 	if (cpsh != NULL) {
+		// the privsep switchboard now ALWAYS creates directories with
+		// permissions 0700.
 		cpsh->initialize_sandbox(WorkingDir.Value());
 		WriteAdFiles();
 	} else {
-		if( mkdir(WorkingDir.Value(), 0777) < 0 ) {
+		// we can only get here if we are not using *CONDOR* PrivSep.  but we
+		// might be using glexec.  glexec relies on being able to read the
+		// contents of the execute directory as a non-condor user, so in that
+		// case, use 0755.  for all other cases, use the more-restrictive 0700.
+		int dir_perms = 0700;
+#if defined(LINUX)
+		if(glexecPrivSepHelper()) {
+			dir_perms = 0755;
+		}
+#endif
+		if( mkdir(WorkingDir.Value(), dir_perms) < 0 ) {
 			dprintf( D_FAILURE|D_ALWAYS,
 			         "couldn't create dir %s: %s\n",
 			         WorkingDir.Value(),
@@ -1481,7 +1906,12 @@ CStarter::createTempExecuteDir( void )
 
 #endif /* WIN32 */
 
-	if( chdir(WorkingDir.Value()) < 0 ) {
+	// switch to user priv -- it's the owner of the directory we just made
+	priv_state ch_p = set_user_priv();
+	int chdir_result = chdir(WorkingDir.Value());
+	set_priv( ch_p );
+
+	if( chdir_result < 0 ) {
 		dprintf( D_FAILURE|D_ALWAYS, "couldn't move to %s: %s\n", WorkingDir.Value(),
 				 strerror(errno) ); 
 		set_priv( priv );
@@ -1614,12 +2044,12 @@ CStarter::jobWaitUntilExecuteTime( void )
 		 	// got a positive integer. Otherwise we'll have to kick out
 		 	//
 		if ( ! jobAd->EvalInteger( ATTR_DEFERRAL_TIME, NULL, deferralTime ) ) {
-			error.sprintf( "Invalid deferred execution time for Job %d.%d.",
+			error.formatstr( "Invalid deferred execution time for Job %d.%d.",
 							this->jic->jobCluster(),
 							this->jic->jobProc() );
 			abort = true;
 		} else if ( deferralTime <= 0 ) {
-			error.sprintf( "Invalid execution time '%d' for Job %d.%d.",
+			error.formatstr( "Invalid execution time '%d' for Job %d.%d.",
 							deferralTime,
 							this->jic->jobCluster(),
 							this->jic->jobProc() );
@@ -1673,7 +2103,7 @@ CStarter::jobWaitUntilExecuteTime( void )
 				//
 			if ( deltaT < 0 ) {
 				if ( abs( deltaT ) > deferralWindow ) {
-					error.sprintf( "Job %d.%d missed its execution time.",
+					error.formatstr( "Job %d.%d missed its execution time.",
 								this->jic->jobCluster(),
 								this->jic->jobProc() );
 					abort = true;
@@ -2026,11 +2456,11 @@ CStarter::WriteRecoveryFile( ClassAd *recovery_ad )
 	}
 
 	if ( m_recoveryFile.Length() == 0 ) {
-		m_recoveryFile.sprintf( "%s%cdir_%ld.recover", Execute,
+		m_recoveryFile.formatstr( "%s%cdir_%ld.recover", Execute,
 								DIR_DELIM_CHAR, (long)daemonCore->getpid() );
 	}
 
-	tmp_file.sprintf( "%s.tmp", m_recoveryFile.Value() );
+	tmp_file.formatstr( "%s.tmp", m_recoveryFile.Value() );
 
 	tmp_fp = safe_fcreate_replace_if_exists( tmp_file.Value(), "w" );
 	if ( tmp_fp == NULL ) {
@@ -2038,7 +2468,7 @@ CStarter::WriteRecoveryFile( ClassAd *recovery_ad )
 		return;
 	}
 
-	if ( recovery_ad->fPrint( tmp_fp ) == FALSE ) {
+	if ( fPrintAd( tmp_fp, *recovery_ad ) == FALSE ) {
 		dprintf( D_ALWAYS, "Failed to write recovery file\n" );
 		fclose( tmp_fp );
 		return;
@@ -2515,7 +2945,7 @@ CStarter::publishJobInfoAd(List<UserProc>* proc_list, ClassAd* ad)
 	if (m_deferred_job_update)
 	{
 		MyString buf;
-		buf.sprintf( "%s=\"Exited\"", ATTR_JOB_STATE );
+		buf.formatstr( "%s=\"Exited\"", ATTR_JOB_STATE );
 	}
 	
 	UserProc *job;
@@ -2560,7 +2990,7 @@ CStarter::GetJobEnv( ClassAd *jobad, Env *job_env, MyString *env_errors )
 	ASSERT( job_env );
 	if( !job_env->MergeFromV1RawOrV2Quoted(env_str,env_errors) ) {
 		if( env_errors ) {
-			env_errors->sprintf_cat(
+			env_errors->formatstr_cat(
 				" The full value for STARTER_JOB_ENVIRONMENT: %s\n",env_str);
 		}
 		free(env_str);
@@ -2570,7 +3000,7 @@ CStarter::GetJobEnv( ClassAd *jobad, Env *job_env, MyString *env_errors )
 
 	if(!job_env->MergeFrom(jobad,env_errors)) {
 		if( env_errors ) {
-			env_errors->sprintf_cat(
+			env_errors->formatstr_cat(
 				" (This error was from the environment string in the job "
 				"ClassAd.)");
 		}
@@ -2606,7 +3036,7 @@ CStarter::PublishToEnv( Env* proc_env )
 		if( ! job_pids.IsEmpty() ) {
 			job_pids += " ";
 		}
-		job_pids.sprintf_cat("%d",uproc->GetJobPid());		
+		job_pids.formatstr_cat("%d",uproc->GetJobPid());		
 	}
 		// put the pid of the job in the environment, used by sshd and hooks
 	proc_env->SetEnv("_CONDOR_JOB_PIDS",job_pids);
@@ -2702,7 +3132,7 @@ CStarter::getMySlotNumber( void )
 
 		char* resource_prefix = param("STARTD_RESOURCE_PREFIX");
 		if( resource_prefix ) {
-			prefix.sprintf(".%s",resource_prefix);
+			prefix.formatstr(".%s",resource_prefix);
 			free( resource_prefix );
 		}
 		else {
@@ -2740,7 +3170,7 @@ CStarter::getMySlotName(void) {
 
 		char* resource_prefix = param("STARTD_RESOURCE_PREFIX");
 		if( resource_prefix ) {
-			prefix.sprintf(".%s",resource_prefix);
+			prefix.formatstr(".%s",resource_prefix);
 			free( resource_prefix );
 		}
 		else {
@@ -2846,7 +3276,7 @@ CStarter::removeTempExecuteDir( void )
 #if !defined(WIN32)
 	if (condorPrivSepHelper() != NULL) {
 		MyString path_name;
-		path_name.sprintf("%s/%s", Execute, dir_name.Value());
+		path_name.formatstr("%s/%s", Execute, dir_name.Value());
 		if (!privsep_remove_dir(path_name.Value())) {
 			dprintf(D_ALWAYS,
 			        "privsep_remove_dir failed for %s\n",
@@ -2928,7 +3358,7 @@ CStarter::WriteAdFiles()
 	ad = this->jic->jobClassAd();
 	if (ad != NULL)
 	{
-		filename.sprintf("%s%c%s", dir, DIR_DELIM_CHAR, JOB_AD_FILENAME);
+		filename.formatstr("%s%c%s", dir, DIR_DELIM_CHAR, JOB_AD_FILENAME);
 		fp = safe_fopen_wrapper_follow(filename.Value(), "w");
 		if (!fp)
 		{
@@ -2939,9 +3369,7 @@ CStarter::WriteAdFiles()
 		}
 		else
 		{
-			ad->SetPrivateAttributesInvisible(true);
-			ad->fPrint(fp);
-			ad->SetPrivateAttributesInvisible(false);
+			fPrintAd(fp, *ad, true);
 			fclose(fp);
 		}
 	}
@@ -2955,7 +3383,7 @@ CStarter::WriteAdFiles()
 	ad = this->jic->machClassAd();
 	if (ad != NULL)
 	{
-		filename.sprintf("%s%c%s", dir, DIR_DELIM_CHAR, MACHINE_AD_FILENAME);
+		filename.formatstr("%s%c%s", dir, DIR_DELIM_CHAR, MACHINE_AD_FILENAME);
 		fp = safe_fopen_wrapper_follow(filename.Value(), "w");
 		if (!fp)
 		{
@@ -2966,9 +3394,7 @@ CStarter::WriteAdFiles()
 		}
 		else
 		{
-			ad->SetPrivateAttributesInvisible(true);
-			ad->fPrint(fp);
-			ad->SetPrivateAttributesInvisible(false);
+			fPrintAd(fp, *ad, true);
 			fclose(fp);
 		}
 	}
