@@ -62,7 +62,7 @@ public:
 			proccessed and handed to use directly
 			@param filename Full path to userlog to write
 		*/
-	bool init( const char* filename, bool is_xml, 
+	bool init( const std::vector<const char*>& filename, bool is_xml, 
 			   int cluster, int proc, int subproc );
 
 		/** Initialize ourselves with the info in the given job ad.
