@@ -199,7 +199,7 @@ MPIMasterProc::alterEnv()
 
 	free( condor_rsh );
 
-	if(DebugFlags & D_FULLDEBUG) {
+	if(IsFulldebug(D_FULLDEBUG)) {
 		MyString env_str;
 		envobject.getDelimitedStringForDisplay(&env_str);
 		dprintf ( D_FULLDEBUG, "New env: %s\n", env_str.Value() );
