@@ -66,6 +66,10 @@ public:
 		*/
 	virtual bool PublishUpdateAd( ClassAd* ad );
 
+	virtual bool SupportsPIDNamespace() { return true;}
+
+	virtual std::string CgroupSuffix() { return "";}
+
 	bool finishShutdownFast();
 
 private:
@@ -74,6 +78,15 @@ private:
 #if !defined(WIN32)
 	int m_escalation_tid;
 #endif
+
+	// Configure OOM killer for this job
+	int m_memory_limit; // Memory limit, in MB.
+	int m_oom_fd; // The file descriptor which recieves events
+	int m_oom_efd; // The event FD to watch
+	int setupOOMScore(int new_score);
+	int outOfMemoryEvent(int fd);
+	int setupOOMEvent(const std::string & cgroup_string);
+
 };
 
 #endif
