@@ -273,7 +273,7 @@ VMProc::StartJob()
 	// Get job name
 	MyString vm_job_name;
 	if( JobAd->LookupString( ATTR_JOB_CMD, vm_job_name) != 1 ) {
-		err_msg.sprintf("%s cannot be found in job classAd.", ATTR_JOB_CMD);
+		err_msg.formatstr("%s cannot be found in job classAd.", ATTR_JOB_CMD);
 		dprintf(D_ALWAYS, "%s\n", err_msg.Value());
 		Starter->jic->notifyStarterError( err_msg.Value(), true, 
 				CONDOR_HOLD_CODE_FailedToCreateProcess, 0);
@@ -284,7 +284,7 @@ VMProc::StartJob()
 	// vm_type should be from ClassAd
 	MyString vm_type_name;
 	if( JobAd->LookupString( ATTR_JOB_VM_TYPE, vm_type_name) != 1 ) {
-		err_msg.sprintf("%s cannot be found in job classAd.", ATTR_JOB_VM_TYPE);
+		err_msg.formatstr("%s cannot be found in job classAd.", ATTR_JOB_VM_TYPE);
 		dprintf(D_ALWAYS, "%s\n", err_msg.Value());
 		Starter->jic->notifyStarterError( err_msg.Value(), true, 
 				CONDOR_HOLD_CODE_FailedToCreateProcess, 0);
@@ -310,6 +310,24 @@ VMProc::StartJob()
 	}
 	*/
 
+	// For Xen and KVM jobs, the vm-gahp issues libvirt commands as root
+	// (since some common configurations only allow root to create VMs).
+	// Libvirt will refuse to create the VM if the acting user doesn't
+	// have explicit permission to access the VM's disk image files,
+	// even though root can ignore those permission settings.
+	// Therefore, we need to relax the permissions on the execute
+	// directory.
+	if ( strcasecmp( m_vm_type.Value(), CONDOR_VM_UNIVERSE_KVM ) == MATCH ||
+		 strcasecmp( m_vm_type.Value(), CONDOR_VM_UNIVERSE_XEN ) == MATCH ) {
+		priv_state oldpriv = set_user_priv();
+		if ( chmod( Starter->GetWorkingDir(), 0755 ) == -1 ) {
+			set_priv( oldpriv );
+			dprintf( D_ALWAYS, "Failed to chmod execute directory for Xen/KVM job: %s\n", strerror( errno ) );
+			return false;
+		}
+		set_priv( oldpriv );
+	}
+
 	ClassAd recovery_ad = *JobAd;
 	MyString vm_name;
 	if ( strcasecmp( m_vm_type.Value(), CONDOR_VM_UNIVERSE_KVM ) == MATCH ||
@@ -1199,10 +1217,9 @@ VMProc::PublishUpdateAd( ClassAd* ad )
 		ad->AssignExpr(ATTR_MEMORY_USAGE, memory_usage.c_str());
 	}
 
-	MyString buf;
 	if( (strcasecmp(m_vm_type.Value(), CONDOR_VM_UNIVERSE_XEN) == MATCH) || (strcasecmp(m_vm_type.Value(), CONDOR_VM_UNIVERSE_KVM) == MATCH) ) {
-		float sys_time = m_vm_cputime;
-		float user_time = 0.0;
+		double sys_time = m_vm_cputime;
+		double user_time = 0.0;
 
 		// Publish it into the ad.
 		ad->Assign(ATTR_JOB_REMOTE_SYS_CPU, sys_time );
@@ -1227,13 +1244,11 @@ VMProc::PublishUpdateAd( ClassAd* ad )
 		}
 
 		// Publish it into the ad.
-		ad->Assign(ATTR_JOB_REMOTE_SYS_CPU, float(sys_time));
-		ad->Assign(ATTR_JOB_REMOTE_USER_CPU, float(user_time));
+		ad->Assign(ATTR_JOB_REMOTE_SYS_CPU, (double)sys_time);
+		ad->Assign(ATTR_JOB_REMOTE_USER_CPU, (double)user_time);
 
-		buf.sprintf("%s=%lu", ATTR_IMAGE_SIZE, max_image );
-		ad->InsertOrUpdate( buf.Value());
-		buf.sprintf("%s=%lu", ATTR_RESIDENT_SET_SIZE, rss );
-		ad->InsertOrUpdate( buf.Value());
+		ad->Assign(ATTR_IMAGE_SIZE, max_image);
+		ad->Assign(ATTR_RESIDENT_SET_SIZE, rss);
 		if( pss_available ) {
 			ad->Assign(ATTR_PROPORTIONAL_SET_SIZE,pss);
 		}
