@@ -52,6 +52,7 @@ int sysapi_ncpus(void);
 /* get raw ncpus without making any calls to config system */
 void sysapi_ncpus_raw_no_param(int *num_cpus,int *num_hyperthread_cpus);
 
+#if 0 // removed from condor_utils to reduce shadow memory use
 /* calculate the number of mips the machine is. Even though this is a user
 	thing and on platforms like NT it is done by hand, it goes in the sysapi.
 	The reasoning is that you are still asking for a fundamental parameter
@@ -63,6 +64,7 @@ int sysapi_mips(void);
 /* as above, but for kflops */
 int sysapi_kflops_raw(void);
 int sysapi_kflops(void);
+#endif
 
 /* return the idle time on the machine in the arguments */
 void sysapi_idle_time_raw(time_t *m_idle, time_t *m_console_idle);
@@ -94,6 +96,7 @@ const char* sysapi_opsys_long_name(void);
 const char* sysapi_opsys_short_name(void);
         int sysapi_opsys_major_version(void);
 const char* sysapi_opsys_legacy(void);
+void sysapi_opsys_dump(int category);
 
 // temporary attributes for raw utsname info
 const char* sysapi_utsname_sysname(void);
@@ -152,6 +155,10 @@ const char* sysapi_kernel_memory_model( void );
 const char* sysapi_ckptpltfrm_raw( void );
 const char* sysapi_ckptpltfrm( void );
 
+/* determine the instruction set extensions on x86 machines */
+const char* sysapi_processor_flags_raw( void );
+const char* sysapi_processor_flags( void );
+
 /* determine the syscall gate address on machines where that makes sense */
 const char * sysapi_vsyscall_gate_addr_raw( void );
 const char * sysapi_vsyscall_gate_addr( void );
@@ -174,27 +181,30 @@ END_C_DECLS
 
 class NetworkDeviceInfo {
 public:
-	NetworkDeviceInfo(char const *the_name,char const *the_ip):
+	NetworkDeviceInfo(char const *the_name,char const *the_ip, bool the_up):
 		m_name(the_name),
-		m_ip(the_ip)
+		m_ip(the_ip),
+		m_up(the_up)
 	{
 	}
 
 	NetworkDeviceInfo(NetworkDeviceInfo const &other):
 		m_name(other.m_name),
-		m_ip(other.m_ip)
+		m_ip(other.m_ip),
+		m_up(other.m_up)
 	{
 	}
 
 	char const *name() { return m_name.c_str(); }
 	char const *IP() { return m_ip.c_str(); }
+	bool is_up() const { return m_up; }
 
 private:
 	std::string m_name;
 	std::string m_ip;
+	bool m_up;
 };
 
-bool sysapi_get_network_device_info_raw(std::vector<NetworkDeviceInfo> &devices);
 bool sysapi_get_network_device_info(std::vector<NetworkDeviceInfo> &devices);
 
 void sysapi_clear_network_device_info_cache();
