@@ -188,10 +188,12 @@ void _condor_prestart( int syscall_mode );
 /* I hate C++, especially when it is badly mixed with C */
 extern void init_syscall_connection_noret( int );
 void init_image_with_file_name( const char *ckpt_name );
+void init_image_relocatable( int );
+void init_image_with_iwd( char * );
 void restart(void);
 
 /* These are the various remote system calls we need to worry about */
-extern int REMOTE_CONDOR_register_syscall_version(char *version);
+extern int REMOTE_CONDOR_register_syscall_version(const char *version);
 extern int REMOTE_CONDOR_chdir(const char *path);
 extern int REMOTE_CONDOR_image_size(int kbytes);
 extern int REMOTE_CONDOR_get_ckpt_mode(enum condor_signal_t sig);
@@ -204,6 +206,7 @@ extern int _condor_dprintf_works;
 extern int	_condor_DebugFD;
 static int do_remote_syscalls = 1;
 
+
 /* This is what we use to determine if an executable has been linked with
 	Condor. Do not rename this function because things like analyze_exec 
 	look for it. */
@@ -363,7 +366,7 @@ MAIN( int argc, char *argv[], char **envp )
 			  -Derek Wright 9/29/99
 			*/
 		if( (strncmp(arg, "D_", 2) == MATCH) ) {
-			_condor_set_debug_flags( arg );
+			_condor_set_debug_flags( arg, 0 );
 			continue;
 		}
 
@@ -408,7 +411,7 @@ MAIN( int argc, char *argv[], char **envp )
 			   -Derek Wright 9/30/99
 			 */
 		if( (strncmp(arg, "D_", 2) == MATCH) ) {
-			_condor_set_debug_flags( arg );
+			_condor_set_debug_flags( arg, 0 );
 			continue;
 		}
 
@@ -462,7 +465,7 @@ MAIN( int argc, char *argv[], char **envp )
 
 		_linked_with_condor_message();
 			// Also, register the version with the shadow
-		REMOTE_CONDOR_register_syscall_version( (char *) CondorVersion() );
+		REMOTE_CONDOR_register_syscall_version( CondorVersion() );
 
 		SetSyscalls( SYS_REMOTE | SYS_MAPPED );
 
@@ -879,10 +882,10 @@ get_ckpt_name()
 void
 _condor_setup_dprintf()
 {
-	if( ! DebugFlags ) {
+	if( ! IsDebugLevel(D_ALWAYS) ) {
 			// If it hasn't already been set, give a default, so we
 			// still get dprintf() if we're running in Condor.
-		DebugFlags = D_ALWAYS | D_NOHEADER;
+		_condor_set_debug_flags( NULL, D_ALWAYS | D_NOHEADER);
 	}
 
 		// Now, initialize what FD we print to.  If we got to this
