@@ -37,7 +37,7 @@ REMAP_TWO( ftruncate, __ftruncate, int , int , off_t )
 REMAP_THREE( poll, __libc_poll, int , struct pollfd *, unsigned long , int )
 REMAP_THREE( poll, __poll, int , struct pollfd *, unsigned long , int )
 
-#if defined(GLIBC23) || defined(GLIBC24) || defined(GLIBC25) || defined(GLIBC27)|| defined(GLIBC211) || defined(GLIBC212)
+#if defined(GLIBC23) || defined(GLIBC24) || defined(GLIBC25) || defined(GLIBC27)|| defined(GLIBC211) || defined(GLIBC212) || defined(GLIBC213)
 
 	REMAP_TWO( ftruncate64, __ftruncate64, int , int , off64_t )
 #endif
@@ -50,7 +50,7 @@ REMAP_THREE_VARARGS( ioctl, __ioctl, int , unsigned long , unsigned long , int)
 
 REMAP_THREE( lseek, __lseek, off_t, int, off_t, int )
 
-#if defined(GLIBC21) || defined(GLIBC22) || defined(GLIBC23) || defined(GLIBC24) || defined(GLIBC25) || defined(GLIBC27) || defined(GLIBC211) || defined(GLIBC212)
+#if defined(GLIBC21) || defined(GLIBC22) || defined(GLIBC23) || defined(GLIBC24) || defined(GLIBC25) || defined(GLIBC27) || defined(GLIBC211) || defined(GLIBC212) || defined(GLIBC213)
 REMAP_THREE( llseek, __llseek, off64_t, int, off64_t, int )
 REMAP_THREE( llseek, _llseek, off64_t, int, off64_t, int )
 REMAP_THREE( llseek, __lseek64, off64_t, int, off64_t, int )
@@ -68,7 +68,7 @@ REMAP_THREE( llseek, lseek64, long long int, int, long long int, int )
 
 REMAP_THREE_VARARGS( open, __open, int, const char *, int, int )
 
-#if defined(GLIBC23) || defined(GLIBC24) || defined(GLIBC25) || defined(GLIBC27) || defined(GLIBC211) || defined(GLIBC212)
+#if defined(GLIBC23) || defined(GLIBC24) || defined(GLIBC25) || defined(GLIBC27) || defined(GLIBC211) || defined(GLIBC212) || defined(GLIBC213)
 REMAP_THREE_VARARGS( open64, __open64, int, const char *, int, int )
 REMAP_THREE_VARARGS( open64, __libc_open64, int, const char *, int, int )
 #endif
@@ -76,13 +76,13 @@ REMAP_THREE_VARARGS( open64, __libc_open64, int, const char *, int, int )
 REMAP_THREE( read, __read, ssize_t, int, __ptr_t, size_t )
 
 REMAP_THREE( write, __write, ssize_t, int, const __ptr_t, size_t )
-#if defined(GLIBC22) || defined(GLIBC23) || defined(GLIBC24) || defined(GLIBC25) || defined(GLIBC27) || defined(GLIBC211) || defined(GLIBC212)
+#if defined(GLIBC22) || defined(GLIBC23) || defined(GLIBC24) || defined(GLIBC25) || defined(GLIBC27) || defined(GLIBC211) || defined(GLIBC212) || defined(GLIBC213)
 REMAP_THREE( write, __libc_write, ssize_t, int, const __ptr_t, size_t )
 #endif
 
 /* make a bunch of __libc remaps for the things that have fd's or paths.
 	These were new entry points in glibc22 */
-#if defined(GLIBC22) || defined(GLIBC23) || defined(GLIBC24) || defined(GLIBC25) || defined(GLIBC27) || defined(GLIBC211) || defined(GLIBC212)
+#if defined(GLIBC22) || defined(GLIBC23) || defined(GLIBC24) || defined(GLIBC25) || defined(GLIBC27) || defined(GLIBC211) || defined(GLIBC212) || defined(GLIBC213)
 REMAP_ONE( close, __libc_close, int , int )
 REMAP_TWO( creat, __libc_creat, int, const char*, mode_t )
 REMAP_ONE( dup, __libc_dup, int, int )
@@ -90,7 +90,7 @@ REMAP_TWO( dup2, __libc_dup2, int, int, int )
 REMAP_ONE( fchdir, __libc_fchdir, int, int )
 REMAP_THREE_VARARGS( fcntl, __libc_fcntl, int , int , int , int)
 
-#if defined(GLIBC25) || defined(GLIBC27) || defined(GLIBC211) || defined(GLIBC212)
+#if defined(GLIBC25) || defined(GLIBC27) || defined(GLIBC211) || defined(GLIBC212) || defined(GLIBC213)
 /* According to the implementation of these function in glibc 2.5 they appear
 to be simple wrappers around their specific system call */
 REMAP_THREE_VARARGS( fcntl, __fcntl_nocancel, int , int , int , int )
@@ -112,7 +112,7 @@ REMAP_THREE_VARARGS( open, __libc_open, int, const char *, int, int )
 REMAP_THREE( read, __libc_read, ssize_t, int, __ptr_t, size_t )
 #endif
 
-#if defined(GLIBC23) || defined(GLIBC24) || defined(GLIBC25) || defined(GLIBC27) || defined(GLIBC211) || defined(GLIBC212)
+#if defined(GLIBC23) || defined(GLIBC24) || defined(GLIBC25) || defined(GLIBC27) || defined(GLIBC211) || defined(GLIBC212) || defined(GLIBC213)
 REMAP_THREE( lseek, __libc_lseek, off_t, int, off_t, int )
 REMAP_FIVE( select, __libc_select, int , int , fd_set *, fd_set *, fd_set *, struct timeval *)
 #endif
@@ -163,7 +163,7 @@ REMAP_TWO( getrlimit, __getrlimit, int , int , struct rlimit *)
 REMAP_TWO( gettimeofday, __gettimeofday, int , struct timeval *, struct timezone *)
 
 /* getuid is now a weak alias to __getuid */
-#if !defined(GLIBC22) && !defined(GLIBC23) && !defined(GLIBC24) && !defined(GLIBC25) && !defined(GLIBC27) && !defined(GLIBC211) && !defined(GLIBC212)
+#if !defined(GLIBC22) && !defined(GLIBC23) && !defined(GLIBC24) && !defined(GLIBC25) && !defined(GLIBC27) && !defined(GLIBC211) && !defined(GLIBC212) && !defined(GLIBC213)
 REMAP_ZERO( getuid, __getuid, uid_t )
 #endif
 
@@ -184,7 +184,7 @@ REMAP_THREE( msync, __libc_msync, int , void *, size_t , int )
 REMAP_TWO( munmap, __munmap, int, void *, size_t )
 REMAP_ONE( pipe, __pipe, int , int *)
 
-#if defined(GLIBC25) || defined(GLIBC27) || defined(GLIBC211) || defined(GLIBC212)
+#if defined(GLIBC25) || defined(GLIBC27) || defined(GLIBC211) || defined(GLIBC212) || defined(GLIBC213)
 REMAP_THREE( readlink, __readlink, ssize_t , const char *, char *, size_t )
 #else
 REMAP_THREE( readlink, __readlink, int , const char *, char *, size_t )
@@ -221,7 +221,7 @@ REMAP_ONE( umount, __umount, int , const char *)
 #endif
 
 /* uname is now a weak alias to __uname */
-#if !defined(GLIBC22) && !defined(GLIBC23) && !defined(GLIBC24) && !defined(GLIBC25) && !defined(GLIBC27) && !defined(GLIBC211) && !defined(GLIBC212)
+#if !defined(GLIBC22) && !defined(GLIBC23) && !defined(GLIBC24) && !defined(GLIBC25) && !defined(GLIBC27) && !defined(GLIBC211) && !defined(GLIBC212) && !defined(GLIBC213)
 REMAP_ONE( uname, __uname, int , struct utsname *)
 #endif
 
@@ -246,14 +246,14 @@ REMAP_FOUR( profil, __profil, int , char *, int , int , int );
 
 #if defined(GLIBC)
 
-#if !defined(GLIBC22) && !defined(GLIBC23) && !defined(GLIBC24) && !defined(GLIBC25) && !defined(GLIBC27) && !defined(GLIBC211) && !defined(GLIBC212)
+#if !defined(GLIBC22) && !defined(GLIBC23) && !defined(GLIBC24) && !defined(GLIBC25) && !defined(GLIBC27) && !defined(GLIBC211) && !defined(GLIBC212) && !defined(GLIBC213)
 /* clone has a much different prototype not supported by the stub generator
 	under glibc22 and later machines */
 REMAP_TWO( clone, __clone, pid_t , void *, unsigned long )
 #endif
 
 
-#if !defined(GLIBC27) && !defined(GLIBC211) && !defined(GLIBC212)
+#if !defined(GLIBC27) && !defined(GLIBC211) && !defined(GLIBC212) && !defined(GLIBC213)
 REMAP_TWO( fstat, __fstat, int , int , struct stat *)
 #endif
 
@@ -775,7 +775,7 @@ int __wprintf_chk(int /* flag */, const wchar_t *format, ...)
 
 #endif /* GLIBC23 */
 
-#if defined(GLIBC25) || defined(GLIBC27) || defined(GLIBC211) || defined(GLIBC212)
+#if defined(GLIBC25) || defined(GLIBC27) || defined(GLIBC211) || defined(GLIBC212) || defined(GLIBC213)
 /* This forces malloc.o from libc.a to not be brought in, and allows stduniv
 	code to use the malloc-user.c API instead */
 REMAP_TWO( memalign, __libc_memalign, void*, size_t, size_t )
