@@ -89,7 +89,7 @@ open_named_pipe( const char *name, int access_mode, int target_fd )
 {
 	int		fd;
 
-	if( (fd=safe_open_wrapper(name,access_mode)) < 0 ) {
+	if( (fd=safe_open_wrapper_follow(name,access_mode)) < 0 ) {
 		assert( fd >= 0 );
 	}
 
