@@ -19,6 +19,7 @@
 ##**************************************************************
 
 use CondorTest;
+use CondorUtils;
 
 $cmd      = 'job_filexfer_input-onegone_van.cmd';
 $template = 'job_filexfer_input-onegone_van.template';
@@ -105,7 +106,7 @@ open(CMD,"<$template") || die "Can not open command file: $!\n";
 open(NEWCMD,">$cmd") || die "Can not open command file: $!\n";
 while(<CMD>)
 {
-	CondorTest::fullchomp($_);
+	CondorUtils::fullchomp($_);
 	CondorTest::debug("$_\n",1);
 	$line = $_;
 	if( $line =~ /^\s*input\s*=\s*job_\d+_dir\/([a-zA-Z_]+)\d*\.txt\s*$/)
