@@ -19,6 +19,8 @@
 ##**************************************************************
 
 use CondorTest;
+use CondorUtils;
+
 use Expect;
 use Socket;
 use Sys::Hostname;
@@ -36,8 +38,7 @@ my $startpostmasterport = 5432;
 my $postmasterinc = int(rand(100) + 13);
 $currenthost = CondorTest::getFqdnHost();
 $domain = `dnsdomainname`;
-CondorTest::fullchomp($currenthost);
-CondorTest::fullchomp($domain);
+CondorUtils::fullchomp($domain);
 
 # for quill this must be the fully qualified name which
 # is what quill uses to builds its search string for the
