@@ -60,12 +60,9 @@ int main( int argc, char *argv[] )
 	bool use_tcp = false;
 	bool with_ack = false;
 	bool allow_multiple = false;
-	param_functions *p_funcs = NULL;
-
 
 	myDistro->Init( argc, argv );
 	config();
-	p_funcs = get_param_functions();
 
 	for( i=1; i<argc; i++ ) {
 		if(!strcmp(argv[i],"-help")) {
@@ -91,9 +88,7 @@ int main( int argc, char *argv[] )
 			exit(0);
 		} else if(!strcmp(argv[i],"-debug")) {
 				// dprintf to console
-			Termlog = 1;
-			p_funcs = get_param_functions();
-			dprintf_config ("TOOL", p_funcs);
+			dprintf_set_tool_debug("TOOL", 0);
 		} else if(argv[i][0]!='-' || !strcmp(argv[i],"-")) {
 			if(command==-1) {
 				command = getCollectorCommandNum(argv[i]);
@@ -207,7 +202,7 @@ int main( int argc, char *argv[] )
 				// If there's no "MyAddress", generate one..
 			if( !ad->Lookup( ATTR_MY_ADDRESS ) ) {
 				MyString tmp;
-				tmp.sprintf( "<%s:0>", my_ip_string() );
+				tmp.formatstr( "<%s:0>", my_ip_string() );
 				ad->Assign( ATTR_MY_ADDRESS, tmp.Value() );
 			}
 
@@ -228,7 +223,7 @@ int main( int argc, char *argv[] )
 
 			int result = 0;
 			if ( sock ) {
-				result += ad->put( *sock );
+				result += putClassAd( sock, *ad );
 				result += sock->end_of_message();
 			}
 			if ( result != 2 ) {
