@@ -672,7 +672,7 @@ static void readHistoryFromFile(char *JobHistoryFileName, char* constraint, Expr
         }
         if (!constraint || EvalBool(ad, constraintExpr)) {
             if (longformat) { 
-                ad->fPrint(stdout); printf("\n"); 
+                fPrintAd(stdout, *ad); printf("\n"); 
             } else {
                 if (customFormat) {
                     mask.display(stdout, ad);
