@@ -31,6 +31,7 @@
 #include "daemon.h"
 #include "dc_schedd.h"
 #include "MyString.h"
+#include "util_lib_proto.h"   // for blankline()
 
 void
 usage(char name[])
@@ -59,7 +60,7 @@ main(int argc, char *argv[])
 	bool UseConstraint = false;
 	MyString schedd_name;
 	MyString pool_name;
-	param_functions *p_funcs;
+	ExprTree* value_expr;
 
 	myDistro->Init( argc, argv );
 	config();
@@ -75,9 +76,7 @@ main(int argc, char *argv[])
 	// if -debug is present, it must be first. sigh.
 	if (argv[nextarg][0] == '-' && argv[nextarg][1] == 'd') {
 		// output dprintf messages to stderror at TOOL_DEBUG level
-		Termlog = 1;
-		p_funcs = get_param_functions();
-		dprintf_config ("TOOL", p_funcs);
+		dprintf_set_tool_debug("TOOL", 0);
 		nextarg++;
 	}
 
@@ -165,12 +164,12 @@ main(int argc, char *argv[])
 			}
 			UseConstraint = false;
 		} else {
-			constraint.sprintf("(%s == %d)", ATTR_CLUSTER_ID, cluster);
+			constraint.formatstr("(%s == %d)", ATTR_CLUSTER_ID, cluster);
 			UseConstraint = true;
 		}
 		nextarg++;
 	} else {
-		constraint.sprintf("(%s == \"%s\")", ATTR_OWNER, argv[nextarg]);
+		constraint.formatstr("(%s == \"%s\")", ATTR_OWNER, argv[nextarg]);
 		nextarg++;
 		UseConstraint = true;
 	}
@@ -186,14 +185,44 @@ main(int argc, char *argv[])
 		if (ProtectedAttribute(argv[nextarg])) {
 			fprintf(stderr, "Update of attribute \"%s\" is not allowed.\n",
 					argv[nextarg]);
+			fprintf(stderr,
+				"Transaction failed.  No attributes were set.\n");
 			exit(1);
 		}
+
+		// Check validity of attribute-name
+		if ( blankline(argv[nextarg]) ||
+			 !IsValidAttrName(argv[nextarg]) )
+		{
+			fprintf(stderr,
+				"Update aborted, illegal attribute-name specified for attribute \"%s\".\n",
+				argv[nextarg]);
+			fprintf(stderr,
+				"Transaction failed.  No attributes were set.\n");
+			exit(1);
+		}
+
+		// Check validity of attribute-value
+		value_expr = NULL;
+		if ( blankline(argv[nextarg+1]) ||
+			 !IsValidAttrValue(argv[nextarg+1]) ||
+			 ParseClassAdRvalExpr(argv[nextarg+1], value_expr) )
+		{
+			fprintf(stderr,
+				"Update aborted, illegal attribute-value specified for attribute \"%s\".\n",
+				argv[nextarg]);
+			fprintf(stderr,
+				"Transaction failed.  No attributes were set.\n");
+			exit(1);
+		}
+		if (value_expr) delete value_expr;
+
 		if (UseConstraint) {
 			// Try to communicate with the newer protocol first
 			if (SetAttributeByConstraint(constraint.Value(),
 							argv[nextarg],
 							argv[nextarg+1],
-							SETDIRTY|SHOULDLOG) < 0) {
+							SETDIRTY) < 0) {
 				if (SetAttributeByConstraint(constraint.Value(),
 							argv[nextarg],
 							argv[nextarg+1]) < 0) {
@@ -201,15 +230,19 @@ main(int argc, char *argv[])
 					fprintf(stderr,
 						"Failed to set attribute \"%s\" by constraint: %s\n",
 						argv[nextarg], constraint.Value());
+					fprintf(stderr,
+						"Transaction failed.  No attributes were set.\n");
 					exit(1);
 				}
 			}
 		} else {
 			if (SetAttribute(cluster, proc, argv[nextarg],
-							 argv[nextarg+1], SETDIRTY|SHOULDLOG) < 0) {
+							 argv[nextarg+1], SETDIRTY) < 0) {
 				fprintf(stderr,
 						"Failed to set attribute \"%s\" for job %d.%d.\n",
 						argv[nextarg], cluster, proc);
+				fprintf(stderr,
+						"Transaction failed.  No attributes were set.\n");
 				exit(1);
 			}
 		}
