@@ -29,11 +29,17 @@
 #include "emit.h"
 #include "unit_test_utils.h"
 
+
+// IPV6_REMOVED - IPv6 changes obsoleted these interfaces.  However, 
+// these tests probably should be updated to the new interfaces,
+// so they are being kept here for nwo.
+#ifdef IPV6_REMOVED
 static bool test_normal_case(void);
 static bool test_slash_notation(void);
 static bool test_classless_mask(void);
 static bool test_wildcard(void);
 static bool test_plain_ip(void);
+#endif
 
 bool FTEST_is_valid_network(void) {
 		// beginning junk for getPortFromAddr(() {
@@ -44,16 +50,19 @@ bool FTEST_is_valid_network(void) {
 	
 		// driver to run the tests and all required setup
 	FunctionDriver driver;
+#ifdef IPV6_REMOVED
 	driver.register_function(test_normal_case);
 	driver.register_function(test_slash_notation);
 	driver.register_function(test_classless_mask);
 	driver.register_function(test_wildcard);
 	driver.register_function(test_plain_ip);
+#endif
 	
 		// run the tests
 	return driver.do_all_functions();
 }
 
+#ifdef IPV6_REMOVED
 static bool test_normal_case() {
 	emit_test("Is normal input parsed correctly?");
 	char* inputstring = strdup("192.168.3.4/255.255.255.0");
@@ -193,3 +202,4 @@ static bool test_plain_ip() {
 	}
 	PASS;
 }
+#endif
