@@ -64,6 +64,7 @@ bool OTEST_UserPolicy(void);
 bool OTEST_Directory(void);
 bool OTEST_TmpDir(void);
 bool OTEST_StatInfo(void);
+bool OTEST_condor_sockaddr();
 
 	// function map that maps testing function names to testing functions
 const static struct {
@@ -100,6 +101,7 @@ const static struct {
 	map(OTEST_Directory),
 	map(OTEST_TmpDir),
 	map(OTEST_StatInfo),
+	map(OTEST_condor_sockaddr),
 };
 int function_map_num_elems = sizeof(function_map) / sizeof(function_map[0]);
 
