@@ -361,6 +361,19 @@ MyString::operator+=( long l )
 }
 
 
+MyString&
+MyString::operator+=( long long l )
+{
+        const int bufLen = 64;
+        char tmp[bufLen];
+        ::snprintf( tmp, bufLen, "%lld", l );
+        int s_len = strlen( tmp );
+        ASSERT(s_len < bufLen);
+        append_str( tmp, s_len );
+        return *this;
+}
+
+
 MyString& 
 MyString::operator+=( double d )
 {
@@ -548,7 +561,7 @@ MyString::replaceString(
 }
 
 bool
-MyString::vsprintf_cat(const char *format,va_list args) 
+MyString::vformatstr_cat(const char *format,va_list args) 
 {
 	char *buffer = NULL;
 	int s_len;
@@ -584,40 +597,39 @@ MyString::vsprintf_cat(const char *format,va_list args)
 }
 
 bool 
-MyString::sprintf_cat(const char *format,...)
+MyString::formatstr_cat(const char *format,...)
 {
 	bool    succeeded;
 	va_list args;
 
 	va_start(args, format);
-	succeeded = vsprintf_cat(format,args);
+	succeeded = vformatstr_cat(format,args);
 	va_end(args);
 
 	return succeeded;
 }
 
 bool
-MyString::vsprintf(const char *format,va_list args)
+MyString::vformatstr(const char *format,va_list args)
 {
 	Len = 0;
 	if(Data) Data[0] = '\0';
-	return vsprintf_cat(format,args);
+	return vformatstr_cat(format,args);
 }
 
-bool 
-MyString::sprintf(const char *format,...)
+bool
+MyString::formatstr(const char *format,...)
 {
 	bool    succeeded;
 	va_list args;
 
 	va_start(args, format);
-	succeeded = vsprintf(format,args);
+	succeeded = vformatstr(format,args);
 	va_end(args);
 
 	return succeeded;
 }
 
-
 void
 MyString::lower_case(void)
 {
