@@ -32,7 +32,7 @@
 #include "condor_config.h"
 #include "filename_tools.h"
 #include "proc.h"
-
+#include "condor_version.h"
 
 // Remove/replace characters from the string so it can be used as an attribute name
 // it changes the string that is passed to it.  first leading an trailing spaces
@@ -264,8 +264,8 @@ ClassAd *CreateJobAd( const char *owner, int universe, const char *cmd )
 {
 	ClassAd *job_ad = new ClassAd();
 
-	job_ad->SetMyTypeName(JOB_ADTYPE);
-	job_ad->SetTargetTypeName(STARTD_ADTYPE);
+	SetMyTypeName(*job_ad, JOB_ADTYPE);
+	SetTargetTypeName(*job_ad, STARTD_ADTYPE);
 
 	if ( owner ) {
 		job_ad->Assign( ATTR_OWNER, owner );
@@ -278,11 +278,11 @@ ClassAd *CreateJobAd( const char *owner, int universe, const char *cmd )
 	job_ad->Assign( ATTR_Q_DATE, (int)time(NULL) );
 	job_ad->Assign( ATTR_COMPLETION_DATE, 0 );
 
-	job_ad->Assign( ATTR_JOB_REMOTE_WALL_CLOCK, (float)0.0 );
-	job_ad->Assign( ATTR_JOB_LOCAL_USER_CPU, (float)0.0 );
-	job_ad->Assign( ATTR_JOB_LOCAL_SYS_CPU, (float)0.0 );
-	job_ad->Assign( ATTR_JOB_REMOTE_USER_CPU, (float)0.0 );
-	job_ad->Assign( ATTR_JOB_REMOTE_SYS_CPU, (float)0.0 );
+	job_ad->Assign( ATTR_JOB_REMOTE_WALL_CLOCK, 0.0 );
+	job_ad->Assign( ATTR_JOB_LOCAL_USER_CPU, 0.0 );
+	job_ad->Assign( ATTR_JOB_LOCAL_SYS_CPU, 0.0 );
+	job_ad->Assign( ATTR_JOB_REMOTE_USER_CPU, 0.0 );
+	job_ad->Assign( ATTR_JOB_REMOTE_SYS_CPU, 0.0 );
 
 		// This is a magic cookie, see how condor_submit sets it
 	job_ad->Assign( ATTR_CORE_SIZE, -1 );
@@ -323,7 +323,7 @@ ClassAd *CreateJobAd( const char *owner, int universe, const char *cmd )
 
 	
 
-	job_ad->Assign( ATTR_IMAGE_SIZE, 0 );
+	job_ad->Assign( ATTR_IMAGE_SIZE, 100 );
 
 	job_ad->Assign( ATTR_JOB_IWD, "/tmp" );
 	job_ad->Assign( ATTR_JOB_INPUT, NULL_FILE );
@@ -365,6 +365,21 @@ ClassAd *CreateJobAd( const char *owner, int universe, const char *cmd )
 
 	job_ad->Assign( ATTR_JOB_LEAVE_IN_QUEUE, false );
 
+	job_ad->AssignExpr( ATTR_REQUEST_MEMORY, "ifthenelse(MemoryUsage isnt undefined,MemoryUsage,( ImageSize + 1023 ) / 1024)" );
+	job_ad->AssignExpr( ATTR_REQUEST_DISK, "DiskUsage" );
+	job_ad->Assign( ATTR_DISK_USAGE, 1 );
+	job_ad->Assign( ATTR_REQUEST_CPUS, 1 );
+
+	// Without these, the starter will not automatically remap the stdout/err (look at sha 422735d9)
+	// and possibly won't put them in the correct directory.
+	job_ad->Assign( ATTR_STREAM_OUTPUT, false );
+	job_ad->Assign( ATTR_STREAM_ERROR, false );
+
+	job_ad->Assign( ATTR_VERSION, CondorVersion() );
+	job_ad->Assign( ATTR_PLATFORM, CondorPlatform() );
+
+	job_ad->Assign( ATTR_Q_DATE, time(NULL) );
+
 	return job_ad;
 }
 
