@@ -89,11 +89,17 @@ ClassAdLog::ClassAdLog(const char *filename,int max_historical_logs_arg) : table
 	unsigned long count = 0;
 	bool is_clean = true; // was cleanly closed (until we find out otherwise)
 	bool requires_successful_cleaning = false;
-	long next_log_entry_pos = 0;
-	while ((log_rec = ReadLogEntry(log_fp, InstantiateLogEntry)) != 0) {
+	long long next_log_entry_pos = 0;
+    long long curr_log_entry_pos = 0;
+	while ((log_rec = ReadLogEntry(log_fp, 1+count, InstantiateLogEntry)) != 0) {
+        curr_log_entry_pos = next_log_entry_pos;
 		next_log_entry_pos = ftell(log_fp);
 		count++;
 		switch (log_rec->get_op_type()) {
+        case CondorLogOp_Error:
+            // this is defensive, ought to be caught in InstantiateLogEntry()
+            EXCEPT("ERROR: transaction record %lu was bad (byte offset %lld)\n", count, curr_log_entry_pos);
+            break;
 		case CondorLogOp_BeginTransaction:
 			// this file contains transactions, so it must not
 			// have been cleanly shut down
@@ -134,7 +140,7 @@ ClassAdLog::ClassAdLog(const char *filename,int max_historical_logs_arg) : table
 			}
 		}
 	}
-	long final_log_entry_pos = ftell(log_fp);
+	long long final_log_entry_pos = ftell(log_fp);
 	if( next_log_entry_pos != final_log_entry_pos ) {
 		// The log file has a broken line at the end so we _must_
 		// _not_ write anything more into this log.
@@ -232,7 +238,7 @@ ClassAdLog::SaveHistoricalLogs()
 	if(!max_historical_logs) return true;
 
 	MyString new_histfile;
-	if(!new_histfile.sprintf("%s.%lu",logFilename(),historical_sequence_number))
+	if(!new_histfile.formatstr("%s.%lu",logFilename(),historical_sequence_number))
 	{
 		dprintf(D_ALWAYS,"Aborting save of historical log: out of memory.\n");
 		return false;
@@ -246,7 +252,7 @@ ClassAdLog::SaveHistoricalLogs()
 	}
 
 	MyString old_histfile;
-	if(!old_histfile.sprintf("%s.%lu",logFilename(),historical_sequence_number - max_historical_logs))
+	if(!old_histfile.formatstr("%s.%lu",logFilename(),historical_sequence_number - max_historical_logs))
 	{
 		dprintf(D_ALWAYS,"Aborting cleanup of historical logs: out of memory.\n");
 		return true; // this is not a fatal error
@@ -291,7 +297,7 @@ ClassAdLog::TruncLog()
 		return false;
 	}
 
-	tmp_log_filename.sprintf( "%s.tmp", logFilename());
+	tmp_log_filename.formatstr( "%s.tmp", logFilename());
 	new_log_fd = safe_open_wrapper_follow(tmp_log_filename.Value(), O_RDWR | O_CREAT | O_LARGEFILE, 0600);
 	if (new_log_fd < 0) {
 		dprintf(D_ALWAYS, "failed to rotate log: safe_open_wrapper(%s) returns %d\n",
@@ -522,14 +528,21 @@ ClassAdLog::ExamineTransaction(const char *key, const char *name, char *&val, Cl
 			if (!name) {
 				if ( !ad ) {
 					ad = new ClassAd;
+					ad->EnableDirtyTracking();
 					ASSERT(ad);
 				}
 				if (val) {
 					free(val);
 					val = NULL;
 				}
-				val = strdup(((LogSetAttribute *)log)->get_value());
-				ad->AssignExpr(lname,val);
+                ExprTree* expr = ((LogSetAttribute *)log)->get_expr();
+                if (expr) {
+		    expr = expr->Copy();
+                    ad->Insert(lname, expr, false);
+                } else {
+                    val = strdup(((LogSetAttribute *)log)->get_value());
+                    ad->AssignExpr(lname, val);
+                }
 				attrsAdded++;
 			}
 			break;
@@ -612,7 +625,7 @@ ClassAdLog::LogState(FILE *fp)
 	while(table.iterate(ad) == 1) {
 		table.getCurrentKey(hashval);
 		hashval.sprint(key);
-		log = new LogNewClassAd(key.Value(), ad->GetMyTypeName(), ad->GetTargetTypeName());
+		log = new LogNewClassAd(key.Value(), GetMyTypeName(*ad), GetTargetTypeName(*ad));
 		if (log->Write(fp) < 0) {
 			EXCEPT("write to %s failed, errno = %d", logFilename(), errno);
 		}
@@ -722,8 +735,9 @@ LogNewClassAd::Play(void *data_structure)
 	int result;
 	ClassAdHashTable *table = (ClassAdHashTable *)data_structure;
 	ClassAd	*ad = new ClassAd();
-	ad->SetMyTypeName(mytype);
-	ad->SetTargetTypeName(targettype);
+	SetMyTypeName(*ad, mytype);
+	SetTargetTypeName(*ad, targettype);
+	ad->EnableDirtyTracking();
 	result = table->insert(HashKey(key), ad);
 
 #if defined(HAVE_DLOPEN)
@@ -838,9 +852,14 @@ LogSetAttribute::LogSetAttribute(const char *k, const char *n, const char *val,
 	op_type = CondorLogOp_SetAttribute;
 	key = strdup(k);
 	name = strdup(n);
-	if (val && strlen(val)) {
+	value_expr = NULL;
+	if (val && strlen(val) && !blankline(val) &&
+		!ParseClassAdRvalExpr(val, value_expr))
+	{
 		value = strdup(val);
 	} else {
+		if (value_expr) delete value_expr;
+		value_expr = NULL;
 		value = strdup("UNDEFINED");
 	}
 	is_dirty = dirty;
@@ -852,6 +871,7 @@ LogSetAttribute::~LogSetAttribute()
 	free(key);
 	free(name);
 	free(value);
+    if (value_expr != NULL) delete value_expr;
 }
 
 
@@ -863,7 +883,18 @@ LogSetAttribute::Play(void *data_structure)
 	ClassAd *ad = 0;
 	if (table->lookup(HashKey(key), ad) < 0)
 		return -1;
-	rval = ad->AssignExpr(name, value);
+    if (value_expr) {
+		// Such a shame, do we really need to make a
+		// copy of value_expr here?  Seems like we could just
+		// assign it and then set value_expr to NULL and avoid
+		// copying a parse tree, since after we Play it I doubt
+		// this class does anything more with value_expr beyond
+		// deallocating it.  - Todd 11/13 <tannenba@cs.wisc.edu>
+        ExprTree * pTree = value_expr->Copy();
+        rval = ad->Insert(name, pTree, false);
+    } else {
+        rval = ad->AssignExpr(name, value);
+    }
 	ad->SetDirtyFlag(name, is_dirty);
 
 #if defined(HAVE_DLOPEN)
@@ -936,6 +967,18 @@ LogSetAttribute::ReadBody(FILE* fp)
 	if (rval < 0) {
 		return rval;
 	}
+
+    if (value_expr) delete value_expr;
+    value_expr = NULL;
+    if (ParseClassAdRvalExpr(value, value_expr)) {
+        if (value_expr) delete value_expr;
+        value_expr = NULL;
+        if (param_boolean("CLASSAD_LOG_STRICT_PARSING", true)) {
+            return -1;
+        } else {
+            dprintf(D_ALWAYS, "WARNING: strict classad parsing failed for expression: \"%s\"\n", value);
+        }
+    }
 	return rval + rval1;
 }
 
@@ -1035,11 +1078,14 @@ LogDeleteAttribute::ReadBody(FILE* fp)
 }
 
 LogRecord	*
-InstantiateLogEntry(FILE *fp, int type)
+InstantiateLogEntry(FILE *fp, unsigned long recnum, int type)
 {
 	LogRecord	*log_rec;
 
 	switch(type) {
+        case CondorLogOp_Error:
+            log_rec = new LogRecordError();
+            break;
 	    case CondorLogOp_NewClassAd:
 		    log_rec = new LogNewClassAd("", "", "");
 			break;
@@ -1062,41 +1108,54 @@ InstantiateLogEntry(FILE *fp, int type)
 			log_rec = new LogHistoricalSequenceNumber(0,0);
 			break;
 	    default:
-		    return 0;
+		    return NULL;
 			break;
 	}
 
-	long pos = ftell(fp);
-
-		// check if we got a bogus record indicating a bad log file
-	if( log_rec->ReadBody(fp) < 0 ) {
-
-			// check if this bogus record is in the midst of a transaction
-			// (try to find a CloseTransaction log record)
+	long long pos = ftell(fp);
 
+	// Check if we got a bogus record indicating a bad log file.  There are two basic
+    // failure modes.  The first mode is some kind of parse failure that occurs at the
+    // end of the log, not followed by an end-of-transaction operation.  In this case, the
+    // incomplete transaction at the end is skipped and ignored with a warning.  The second
+    // mode is a failure that occurs inside a complete transaction (one with an end-of-
+    // transaction op).  A complete transaction with corruption is unrecoverable, and 
+    // causes a fatal exception.
+	if (log_rec->ReadBody(fp) < 0  ||  log_rec->get_op_type() == CondorLogOp_Error) {
+        dprintf(D_ALWAYS, "WARNING: Encountered corrupt log record %lu (byte offset %lld)\n", recnum, pos);
 
 		char	line[ATTRLIST_MAX_EXPRESSION + 64];
 		int		op;
 
 		delete log_rec;
 		if( !fp ) {
-			EXCEPT("Error: failed fdopen() when recovering corrpupt log file");
+			EXCEPT("Error: failed fdopen() while recovering from corrupt log record %lu", recnum);
 		}
 
+        // check if this bogus record is in the midst of a transaction
+	    // (try to find a CloseTransaction log record)
+        const unsigned long maxfollow = 3;
+        dprintf(D_ALWAYS, "Lines following corrupt log record %lu (up to %lu):\n", recnum, maxfollow);
+        unsigned long nlines = 0;
 		while( fgets( line, ATTRLIST_MAX_EXPRESSION+64, fp ) ) {
-			if( sscanf( line, "%d ", &op ) != 1 ) {
-					// no op field in line; more bad log records...
+            nlines += 1;
+            if (nlines <= maxfollow) {
+                dprintf(D_ALWAYS, "    %s", line);
+                int ll = strlen(line);
+                if (ll <= 0  ||  line[ll-1] != '\n') dprintf(D_ALWAYS, "\n");
+            }
+			if (sscanf( line, "%d ", &op ) != 1  ||  !valid_record_optype(op)) {
+				// no op field in line; more bad log records...
 				continue;
 			}
 			if( op == CondorLogOp_EndTransaction ) {
 					// aargh!  bad record in transaction.  abort!
-				EXCEPT("Error: bad record with op=%d (at byte offset %ld) in corrupt logfile",type,pos);
+				EXCEPT("Error: corrupt log record %lu (byte offset %lld) occurred inside closed transaction, recovery failed", recnum, pos);
 			}
 		}
 
 		if( !feof( fp ) ) {
-			EXCEPT("Error: failed recovering from corrupt file, errno=%d",
-				errno );
+			EXCEPT("Error: failed recovering from corrupt log record %lu, errno=%d", recnum, errno);
 		}
 
 			// there wasn't an error in reading the file, and the bad log 
