@@ -20,67 +20,64 @@
 
 #include "condor_common.h"
 #include "command_strings.h"
+#include <map>
 
 const struct Translation DCTranslation[] = {
 	{ "QMGMT_READ_CMD", QMGMT_READ_CMD }, 
 	{ "QMGMT_WRITE_CMD", QMGMT_WRITE_CMD }, 
-	{ "CONTINUE_FRGM_JOB", CONTINUE_FRGN_JOB },
 	{ "CONTINUE_CLAIM", CONTINUE_CLAIM },
-	{ "SUSPEND_FRGN_JOB", SUSPEND_FRGN_JOB },
 	{ "SUSPEND_CLAIM", SUSPEND_CLAIM },
-	{ "CKPT_FRGN_JOB", CKPT_FRGN_JOB },
 	{ "DEACTIVATE_CLAIM", DEACTIVATE_CLAIM },
-	{ "KILL_FRGN_JOB", KILL_FRGN_JOB },
 	{ "DEACTIVATE_CLAIM_FORCIBLY", DEACTIVATE_CLAIM_FORCIBLY },
-	{ "LOCAL_STATUS", LOCAL_STATUS },
-	{ "LOCAL_STATISTICS", LOCAL_STATISTICS },
+//	{ "LOCAL_STATUS", LOCAL_STATUS },				/* Not used */
+//	{ "LOCAL_STATISTICS", LOCAL_STATISTICS },		/* Not used */
 	{ "PERMISSION", PERMISSION },
-	{ "SET_DEBUG_FLAGS", SET_DEBUG_FLAGS },
-	{ "PREEMPT_LOCAL_JOBS", PREEMPT_LOCAL_JOBS },
-	{ "RM_LOCAL_JOB", RM_LOCAL_JOB },
-	{ "START_FRGN_JOB", START_FRGN_JOB },
-	{ "AVAILABILITY", AVAILABILITY },
-	{ "NUM_FRGN_JOBS", NUM_FRGN_JOBS },
-	{ "STARTD_INFO", STARTD_INFO },
-	{ "SCHEDD_INFO", SCHEDD_INFO },
+//	{ "SET_DEBUG_FLAGS", SET_DEBUG_FLAGS },			/* Not used */
+//	{ "PREEMPT_LOCAL_JOBS", PREEMPT_LOCAL_JOBS },	/* Not used */
+//	{ "RM_LOCAL_JOB", RM_LOCAL_JOB },				/* Not used */
+//	{ "START_FRGN_JOB", START_FRGN_JOB },			/* Not used */
+//	{ "AVAILABILITY", AVAILABILITY },				/* Not used */
+//	{ "NUM_FRGN_JOBS", NUM_FRGN_JOBS },				/* Not used */
+//	{ "STARTD_INFO", STARTD_INFO },					/* Not used */
+//	{ "SCHEDD_INFO", SCHEDD_INFO },					/* Not used */
 	{ "NEGOTIATE", NEGOTIATE },
 	{ "NEGOTIATE_WITH_SIGATTRS", NEGOTIATE_WITH_SIGATTRS},
 	{ "SEND_JOB_INFO", SEND_JOB_INFO },
 	{ "NO_MORE_JOBS", NO_MORE_JOBS },
 	{ "JOB_INFO", JOB_INFO },
-	{ "GIVE_STATUS", GIVE_STATUS },
+//	{ "GIVE_STATUS", GIVE_STATUS },					/* Not used */
 	{ "RESCHEDULE", RESCHEDULE },
-	{ "PING", PING },
-	{ "NEGOTIATOR_INFO", NEGOTIATOR_INFO },
-	{ "GIVE_STATUS_LINES", GIVE_STATUS_LINES },
+//	{ "PING", PING },								/* Not used */
+//	{ "NEGOTIATOR_INFO", NEGOTIATOR_INFO },			/* Not used */
+//	{ "GIVE_STATUS_LINES", GIVE_STATUS_LINES },		/* Not used */
 	{ "END_NEGOTIATE", END_NEGOTIATE },
 	{ "REJECTED", REJECTED },
 	{ "X_EVENT_NOTIFICATION", X_EVENT_NOTIFICATION },
-	{ "RECONFIG", RECONFIG },
-	{ "GET_HISTORY", GET_HISTORY },
-	{ "UNLINK_HISTORY_FILE", UNLINK_HISTORY_FILE },
-	{ "UNLINK_HISTORY_FILE_DONE", UNLINK_HISTORY_FILE_DONE },
-	{ "DO_NOT_UNLINK_HISTORY_FILE", DO_NOT_UNLINK_HISTORY_FILE },
-	{ "SEND_ALL_JOBS", SEND_ALL_JOBS },
-	{ "SEND_ALL_JOBS_PRIO", SEND_ALL_JOBS_PRIO },
+//	{ "RECONFIG", RECONFIG },						/* Not used */
+//	{ "GET_HISTORY", GET_HISTORY },					/* Not used */
+//	{ "UNLINK_HISTORY_FILE", UNLINK_HISTORY_FILE },		/* Not used */
+//	{ "UNLINK_HISTORY_FILE_DONE", UNLINK_HISTORY_FILE_DONE },	/* Not used */
+//	{ "DO_NOT_UNLINK_HISTORY_FILE", DO_NOT_UNLINK_HISTORY_FILE },	/* Not used */
+//	{ "SEND_ALL_JOBS", SEND_ALL_JOBS },				/* Not used */
+//	{ "SEND_ALL_JOBS_PRIO", SEND_ALL_JOBS_PRIO },	/* Not used */
 	{ "REQ_NEW_PROC", REQ_NEW_PROC },
 	{ "PCKPT_FRGN_JOB", PCKPT_FRGN_JOB },
-	{ "SEND_RUNNING_JOBS", SEND_RUNNING_JOBS },
-	{ "CHECK_CAPABILITY", CHECK_CAPABILITY },
-	{ "GIVE_PRIORITY", GIVE_PRIORITY },
+//	{ "SEND_RUNNING_JOBS", SEND_RUNNING_JOBS },		/* Not used */
+//	{ "CHECK_CAPABILITY", CHECK_CAPABILITY },		/* Not used */
+//	{ "GIVE_PRIORITY", GIVE_PRIORITY },				/* Not used */
 	{ "MATCH_INFO", MATCH_INFO },
 	{ "ALIVE", ALIVE },
 	{ "REQUEST_CLAIM", REQUEST_CLAIM },
 	{ "RELEASE_CLAIM", RELEASE_CLAIM },
 	{ "ACTIVATE_CLAIM", ACTIVATE_CLAIM },
-	{ "PRIORITY_INFO", PRIORITY_INFO },
+//	{ "PRIORITY_INFO", PRIORITY_INFO },				/* Not used */
 	{ "PCKPT_ALL_JOBS", PCKPT_ALL_JOBS },
 	{ "VACATE_ALL_CLAIMS", VACATE_ALL_CLAIMS },
 	{ "GIVE_STATE", GIVE_STATE },
 	{ "SET_PRIORITY", SET_PRIORITY },
-	{ "GIVE_CLASSAD", GIVE_CLASSAD },
+//	{ "GIVE_CLASSAD", GIVE_CLASSAD },				/* Not used */
 	{ "GET_PRIORITY", GET_PRIORITY },
-	{ "GIVE_REQUEST_AD", GIVE_REQUEST_AD },
+//	{ "GIVE_REQUEST_AD", GIVE_REQUEST_AD },			/* Not used */
 	{ "RESTART", RESTART },
 	{ "RESTART_PEACEFUL", RESTART_PEACEFUL },
 	{ "DAEMONS_OFF", DAEMONS_OFF },
@@ -105,7 +102,7 @@ const struct Translation DCTranslation[] = {
 	{ "GIVE_TOTALS_CLASSAD", GIVE_TOTALS_CLASSAD },
 	{ "DUMP_STATE", DUMP_STATE },
 	{ "PERMISSION_AND_AD", PERMISSION_AND_AD },
-	{ "REQUEST_NETWORK", REQUEST_NETWORK },
+//	{ "REQUEST_NETWORK", REQUEST_NETWORK },			/* Not used */
 	{ "VACATE_ALL_FAST", VACATE_ALL_FAST },
 	{ "VACATE_CLAIM_FAST", VACATE_CLAIM_FAST },
 	{ "REJECTED_WITH_REASON", REJECTED_WITH_REASON },
@@ -143,11 +140,11 @@ const struct Translation DCTranslation[] = {
 	{ "DC_INVALIDATE_KEY", DC_INVALIDATE_KEY },
 	{ "FILETRANS_UPLOAD", FILETRANS_UPLOAD },
 	{ "FILETRANS_DOWNLOAD", FILETRANS_DOWNLOAD },
-	{ "PW_SETPASS", PW_SETPASS },
-	{ "PW_GETPASS", PW_GETPASS },
-	{ "PW_CLEARPASS", PW_CLEARPASS },
+//	{ "PW_SETPASS", PW_SETPASS },					/* Not used */
+//	{ "PW_GETPASS", PW_GETPASS },					/* Not used */
+//	{ "PW_CLEARPASS", PW_CLEARPASS },				/* Not used */
 	{ "SHADOW_UPDATEINFO", SHADOW_UPDATEINFO },
-	{ "TAKE_MATCH", TAKE_MATCH },
+//	{ "TAKE_MATCH", TAKE_MATCH },					/* Not used */
 	{ "MPI_START_COMRADE", MPI_START_COMRADE },
 	{ "CHILD_OFF", CHILD_OFF },
 	{ "CHILD_OFF_FAST", CHILD_OFF_FAST },
@@ -158,6 +155,7 @@ const struct Translation DCTranslation[] = {
 	{ "DELEGATE_GSI_CRED_STARTD", DELEGATE_GSI_CRED_STARTD },
 	{ "REQUEST_SANDBOX_LOCATION", REQUEST_SANDBOX_LOCATION },
 	/* maybe not right place, but so what for now */
+	{ "TRANSFERD_REGISTER", TRANSFERD_REGISTER },
 	{ "TRANSFERD_CONTROL_CHANNEL", TRANSFERD_CONTROL_CHANNEL},
 	{ "TRANSFERD_WRITE_FILES", TRANSFERD_WRITE_FILES},
 	{ "TRANSFERD_READ_FILES", TRANSFERD_READ_FILES},
@@ -174,6 +172,7 @@ const struct Translation DCTranslation[] = {
 	{ "GET_JOB_CONNECT_INFO", GET_JOB_CONNECT_INFO },
 	{ "CREATE_JOB_OWNER_SEC_SESSION", CREATE_JOB_OWNER_SEC_SESSION },
 	{ "START_SSHD", START_SSHD },
+	{ "STARTER_PEEK", STARTER_PEEK },
 	{ "SHARED_PORT_CONNECT", SHARED_PORT_CONNECT },
 	{ "SHARED_PORT_PASS_SOCK", SHARED_PORT_PASS_SOCK },
 	{ "RECYCLE_SHADOW", RECYCLE_SHADOW },
@@ -181,7 +180,58 @@ const struct Translation DCTranslation[] = {
         { "UPDATE_JOBAD", UPDATE_JOBAD },
 	{ "DRAIN_JOBS", DRAIN_JOBS },
 	{ "CANCEL_DRAIN_JOBS", CANCEL_DRAIN_JOBS },
-	{ NULL, 0 }
+	{ "DC_AUTHENTICATE", DC_AUTHENTICATE },
+	{ "DC_SEC_QUERY", DC_SEC_QUERY },
+	{ "DC_NOP", DC_NOP },
+	{ "DC_NOP_READ", DC_NOP_READ },
+	{ "DC_NOP_WRITE", DC_NOP_WRITE },
+	{ "DC_NOP_NEGOTIATOR", DC_NOP_NEGOTIATOR },
+	{ "DC_NOP_ADMINISTRATOR", DC_NOP_ADMINISTRATOR },
+	{ "DC_NOP_OWNER", DC_NOP_OWNER },
+	{ "DC_NOP_CONFIG", DC_NOP_CONFIG },
+	{ "DC_NOP_DAEMON", DC_NOP_DAEMON },
+	{ "DC_NOP_ADVERTISE_STARTD", DC_NOP_ADVERTISE_STARTD },
+	{ "DC_NOP_ADVERTISE_SCHEDD", DC_NOP_ADVERTISE_SCHEDD },
+	{ "DC_NOP_ADVERTISE_MASTER", DC_NOP_ADVERTISE_MASTER },
+	{ "DC_OFF_FORCE", DC_OFF_FORCE },
+	{ "DC_PURGE_LOG", DC_PURGE_LOG },
+	{ "DC_SET_FORCE_SHUTDOWN", DC_SET_FORCE_SHUTDOWN },
+	{ "DC_SET_PEACEFUL_SHUTDOWN", DC_SET_PEACEFUL_SHUTDOWN },
+	{ "DC_TIME_OFFSET", DC_TIME_OFFSET },
+	{ "SPOOL_JOB_FILES", SPOOL_JOB_FILES },
+	{ "SPOOL_JOB_FILES_WITH_PERMS", SPOOL_JOB_FILES_WITH_PERMS },
+	{ "TRANSFER_DATA", TRANSFER_DATA },
+	{ "TRANSFER_DATA_WITH_PERMS", TRANSFER_DATA_WITH_PERMS },
+	{ "UPDATE_GSI_CRED", UPDATE_GSI_CRED },
+	{ "GET_MYPROXY_PASSWORD", GET_MYPROXY_PASSWORD },
+	{ "GIVE_MATCHES", GIVE_MATCHES },
+	{ "SET_ACCUMUSAGE", SET_ACCUMUSAGE },
+	{ "SET_BEGINTIME", SET_BEGINTIME },
+	{ "SET_LASTTIME", SET_LASTTIME },
+	{ "SET_SHUTDOWN_PROGRAM", SET_SHUTDOWN_PROGRAM },
+	{ "GET_PRIORITY_ROLLUP", GET_PRIORITY_ROLLUP },
+	{ "STORE_CRED", STORE_CRED },
+	{ "STORE_POOL_CRED", STORE_POOL_CRED },
+	{ "CREDD_STORE_CRED", CREDD_STORE_CRED },
+	{ "CREDD_GET_CRED", CREDD_GET_CRED },
+	{ "CREDD_REMOVE_CRED" , CREDD_REMOVE_CRED },
+	{ "CREDD_QUERY_CRED", CREDD_QUERY_CRED },
+	{ "CREDD_GET_PASSWD", CREDD_GET_PASSWD },
+	{ "CREDD_NOP", CREDD_NOP },
+	{ "HAD_ALIVE_CMD", HAD_ALIVE_CMD },
+	{ "HAD_SEND_ID_CMD", HAD_SEND_ID_CMD },
+//	{ "HAD_REPL_UPDATE_VERSION", HAD_REPL_UPDATE_VERSION },	/* Not used */
+	{ "HAD_BEFORE_PASSIVE_STATE", HAD_BEFORE_PASSIVE_STATE },
+	{ "HAD_AFTER_ELECTION_STATE", HAD_AFTER_ELECTION_STATE },
+	{ "HAD_AFTER_LEADER_STATE", HAD_AFTER_LEADER_STATE },
+	{ "HAD_IN_LEADER_STATE", HAD_IN_LEADER_STATE },
+	{ "REPLICATION_TRANSFER_FILE", REPLICATION_TRANSFER_FILE },
+	{ "REPLICATION_LEADER_VERSION", REPLICATION_LEADER_VERSION },
+	{ "REPLICATION_NEWLY_JOINED_VERSION", REPLICATION_NEWLY_JOINED_VERSION },
+	{ "REPLICATION_GIVING_UP_VERSION", REPLICATION_GIVING_UP_VERSION },
+	{ "REPLICATION_SOLICIT_VERSION", REPLICATION_SOLICIT_VERSION },
+	{ "REPLICATION_SOLICIT_VERSION_REPLY", REPLICATION_SOLICIT_VERSION_REPLY },
+	{ "", 0 }
 };
 
 const struct Translation CollectorTranslation[] = {
@@ -189,7 +239,7 @@ const struct Translation CollectorTranslation[] = {
     { "UPDATE_STARTD_AD_WITH_ACK", UPDATE_STARTD_AD_WITH_ACK },
 	{ "UPDATE_SCHEDD_AD", UPDATE_SCHEDD_AD },
 	{ "UPDATE_MASTER_AD", UPDATE_MASTER_AD },
-	{ "UPDATE_GATEWAY_AD", UPDATE_GATEWAY_AD },
+//	{ "UPDATE_GATEWAY_AD", UPDATE_GATEWAY_AD },		/* Not used */
 	{ "UPDATE_CKPT_SRVR_AD", UPDATE_CKPT_SRVR_AD },
 	{ "UPDATE_NEGOTIATOR_AD", UPDATE_NEGOTIATOR_AD },
 	{ "UPDATE_HAD_AD", UPDATE_HAD_AD },
@@ -199,19 +249,20 @@ const struct Translation CollectorTranslation[] = {
 	{ "QUERY_STARTD_ADS", QUERY_STARTD_ADS },
 	{ "QUERY_SCHEDD_ADS", QUERY_SCHEDD_ADS },
 	{ "QUERY_MASTER_ADS", QUERY_MASTER_ADS },
-	{ "QUERY_GATEWAY_ADS", QUERY_GATEWAY_ADS },
+//	{ "QUERY_GATEWAY_ADS", QUERY_GATEWAY_ADS },		/* Not used */
 	{ "QUERY_CKPT_SRVR_ADS", QUERY_CKPT_SRVR_ADS },
 	{ "QUERY_STARTD_PVT_ADS", QUERY_STARTD_PVT_ADS },
 	{ "QUERY_NEGOTIATOR_ADS", QUERY_NEGOTIATOR_ADS },
 	{ "QUERY_HAD_ADS", QUERY_HAD_ADS },
 	{ "QUERY_XFER_SERVICE_ADS", QUERY_XFER_SERVICE_ADS },
 	{ "QUERY_LEASE_MANAGER_ADS", QUERY_LEASE_MANAGER_ADS },
+	{ "QUERY_GENERIC_ADS", QUERY_GENERIC_ADS },
 	{ "UPDATE_SUBMITTOR_AD", UPDATE_SUBMITTOR_AD },
 	{ "QUERY_SUBMITTOR_ADS", QUERY_SUBMITTOR_ADS },
 	{ "INVALIDATE_STARTD_ADS", INVALIDATE_STARTD_ADS },
 	{ "INVALIDATE_SCHEDD_ADS", INVALIDATE_SCHEDD_ADS },
 	{ "INVALIDATE_MASTER_ADS", INVALIDATE_MASTER_ADS },
-	{ "INVALIDATE_GATEWAY_ADS", INVALIDATE_GATEWAY_ADS },
+//	{ "INVALIDATE_GATEWAY_ADS", INVALIDATE_GATEWAY_ADS },	/* Not used */
 	{ "INVALIDATE_CKPT_SRVR_ADS", INVALIDATE_CKPT_SRVR_ADS },
 	{ "INVALIDATE_SUBMITTOR_ADS", INVALIDATE_SUBMITTOR_ADS },
 	{ "INVALIDATE_NEGOTIATOR_ADS", INVALIDATE_NEGOTIATOR_ADS },
@@ -244,7 +295,10 @@ const struct Translation CollectorTranslation[] = {
 	{ "QUERY_GRID_ADS", QUERY_GRID_ADS },
 	{ "INVALIDATE_GRID_ADS", INVALIDATE_GRID_ADS },
 	{ "MERGE_STARTD_AD", MERGE_STARTD_AD },
-	{ NULL, 0 }
+	{ "UPDATE_QUILL_AD", UPDATE_QUILL_AD },
+	{ "QUERY_QUILL_ADS", QUERY_QUILL_ADS },
+	{ "INVALIDATE_QUILL_ADS", INVALIDATE_QUILL_ADS },
+	{ "", 0 }
 };
 
 
@@ -259,14 +313,14 @@ const struct Translation CAResultTranslation[] = {
 	{ "LocateFailed", CA_LOCATE_FAILED },
 	{ "ConnectFailed", CA_CONNECT_FAILED },
 	{ "CommunicationError", CA_COMMUNICATION_ERROR },
-	{ NULL, 0 }
+	{ "", 0 }
 };
 
 const struct Translation DrainingScheduleTranslation[] = {
 	{ "graceful", DRAIN_GRACEFUL },
 	{ "quick", DRAIN_QUICK },
 	{ "fast", DRAIN_FAST },
-	{ NULL, 0 }
+	{ "", 0 }
 };
 
 
@@ -280,6 +334,42 @@ getCommandString( int num )
 	return result;
 }
 
+// return the command name for known commmand or "command NNN" for unknown commands
+// returned pointer is valid forever and the caller does not free it
+const char*
+getCommandStringSafe( int num )
+{
+	char const *result = getCommandString(num);
+	if (result)
+		return result;
+
+	return getUnknownCommandString(num);
+}
+
+// return a string pointer representation of an unknown command id
+// that can be cached by the caller for the lifetime of the process.
+const char*
+getUnknownCommandString(int num)
+{
+	static std::map<int, const char*> * pcmds = NULL;
+	if ( ! pcmds) {
+		pcmds = new std::map<int, const char*>();
+		if ( ! pcmds) return "malloc-fail!";
+	}
+
+	std::map<int, const char*>::iterator it = pcmds->find(num);
+	if (it != pcmds->end()) {
+		return it->second;
+	}
+
+	static const char fmt[] = "command %u";
+	char * pstr = (char*)malloc(sizeof(fmt)+8); // max int string is 10 bytes (-2 for %d)
+	if ( ! pstr) return "malloc-fail!";
+	sprintf(pstr, fmt, num);
+	(*pcmds)[num] = pstr;
+	return pstr;
+}
+
 
 int
 getCommandNum( const char* command )
