@@ -90,12 +90,17 @@ const char *ExprTreeToString( classad::ExprTree *expr )
 	return buffer.c_str();
 }
 
+#define IS_DOUBLE_TRUE(val) (bool)(int)((val)*100000)
+
 bool EvalBool(compat_classad::ClassAd *ad, const char *constraint)
 {
 	static classad::ExprTree *tree = NULL;
 	static char * saved_constraint = NULL;
-	compat_classad::EvalResult result;
+	classad::Value result;
 	bool constraint_changed = true;
+	double doubleVal;
+	long long intVal;
+	bool boolVal;
 
 	if ( saved_constraint ) {
 		if ( strcmp(saved_constraint,constraint) == 0 ) {
@@ -126,30 +131,41 @@ bool EvalBool(compat_classad::ClassAd *ad, const char *constraint)
 
 	// Evaluate constraint with ad in the target scope so that constraints
 	// have the same semantics as the collector queries.  --RR
-	if ( !EvalExprTree( tree, ad, NULL, &result ) ) {
+	if ( !EvalExprTree( tree, ad, NULL, result ) ) {
 		dprintf( D_ALWAYS, "can't evaluate constraint: %s\n", constraint );
 		return false;
 	}
-	if ( result.type == compat_classad::LX_INTEGER ) {
-		return (bool)result.i;
+	if( result.IsBooleanValue( boolVal ) ) {
+		return boolVal;
+	} else if( result.IsIntegerValue( intVal ) ) {
+		return intVal != 0;
+	} else if( result.IsRealValue( doubleVal ) ) {
+		return IS_DOUBLE_TRUE(doubleVal);
 	}
-	dprintf( D_ALWAYS, "constraint (%s) does not evaluate to bool\n",
+	dprintf( D_FULLDEBUG, "constraint (%s) does not evaluate to bool\n",
 		constraint );
 	return false;
 }
 
 bool EvalBool(compat_classad::ClassAd *ad, classad::ExprTree *tree)
 {
-	compat_classad::EvalResult result;
+	classad::Value result;
+	double doubleVal;
+	long long intVal;
+	bool boolVal;
 
 	// Evaluate constraint with ad in the target scope so that constraints
 	// have the same semantics as the collector queries.  --RR
-	if ( !EvalExprTree( tree, ad, NULL, &result ) ) {        
+	if ( !EvalExprTree( tree, ad, NULL, result ) ) {        
 		return false;
 	}
 
-	if ( result.type == compat_classad::LX_INTEGER ) {
-		return (bool)result.i;
+	if( result.IsBooleanValue( boolVal ) ) {
+		return boolVal;
+	} else if( result.IsIntegerValue( intVal ) ) {
+		return intVal != 0;
+	} else if( result.IsRealValue( doubleVal ) ) {
+		return IS_DOUBLE_TRUE(doubleVal);
 	}
 
 	return false;
@@ -228,75 +244,6 @@ int EvalExprTree( classad::ExprTree *expr, compat_classad::ClassAd *source,
 	return rc;
 }
 
-int EvalExprTree( classad::ExprTree *expr, compat_classad::ClassAd *source,
-				  compat_classad::ClassAd *target, compat_classad::EvalResult *result )
-{
-	int rc = TRUE;
-	if ( !expr || !source || !result ) {
-		return FALSE;
-	}
-
-	classad::Value val;
-	const classad::ClassAd *old_scope = expr->GetParentScope();
-	classad::MatchClassAd *mad = NULL;
-
-	expr->SetParentScope( source );
-	if ( target && target != source ) {
-		mad = compat_classad::getTheMatchAd( source, target );
-	} else {
-		compat_classad::getTheMyRef( source );
-	}
-	result->clear(); // avoid leaking memory
-	if ( source->EvaluateExpr( expr, val ) ) {
-		switch ( val.GetType() ) {
-		case classad::Value::ERROR_VALUE:
-			result->type = compat_classad::LX_ERROR;
-			break;
-		case classad::Value::UNDEFINED_VALUE:
-			result->type = compat_classad::LX_UNDEFINED;
-			break;
-		case classad::Value::BOOLEAN_VALUE: {
-			result->type = compat_classad::LX_INTEGER;
-			bool v;
-			val.IsBooleanValue( v );
-			result->i = v ? 1 : 0;
-			break;
-		}
-		case classad::Value::INTEGER_VALUE:
-			result->type = compat_classad::LX_INTEGER;
-			val.IsIntegerValue( result->i );
-			break;
-		case classad::Value::REAL_VALUE: {
-			result->type = compat_classad::LX_FLOAT;
-			double d;
-			val.IsRealValue( d );
-			result->f = d;
-			break;
-		}
-		case classad::Value::STRING_VALUE: {
-			result->type = compat_classad::LX_STRING;
-			std::string s;
-			val.IsStringValue( s );
-			result->s = strnewp( s.c_str() );
-			break;
-		}
-		default:
-			rc = FALSE;
-		}
-	} else {
-		rc = FALSE;
-	}
-
-	if ( mad ) {
-		compat_classad::releaseTheMatchAd();
-	} else {
-		compat_classad::releaseTheMyRef( source );
-	}
-	expr->SetParentScope( old_scope );
-
-	return rc;
-}
-
 bool IsAMatch( compat_classad::ClassAd *ad1, compat_classad::ClassAd *ad2 )
 {
 	classad::MatchClassAd *mad = compat_classad::getTheMatchAd( ad1, ad2 );
@@ -312,8 +259,8 @@ bool IsAHalfMatch( compat_classad::ClassAd *my, compat_classad::ClassAd *target
 		// The collector relies on this function to check the target type.
 		// Eventually, we should move that check either into the collector
 		// or into the requirements expression.
-	char const *my_target_type = my->GetTargetTypeName();
-	char const *target_type = target->GetMyTypeName();
+	char const *my_target_type = GetTargetTypeName(*my);
+	char const *target_type = GetMyTypeName(*target);
 	if( !my_target_type ) {
 		my_target_type = "";
 	}
@@ -338,3 +285,33 @@ void AttrList_setPublishServerTime( bool publish )
 {
 	AttrList_setPublishServerTimeMangled( publish );
 }
+
+/**************************************************************************
+ *
+ * Function: AddClassAdXMLFileHeader
+ * Purpose:  Print the stuff that should appear at the beginning of an
+ *           XML file that contains a series of ClassAds.
+ *
+ **************************************************************************/
+void AddClassAdXMLFileHeader(std::string &buffer)
+{
+	buffer += "<?xml version=\"1.0\"?>\n";
+	buffer += "<!DOCTYPE classads SYSTEM \"classads.dtd\">\n";
+	buffer += "<classads>\n";
+	return;
+
+}
+
+/**************************************************************************
+ *
+ * Function: AddClassAdXMLFileFooter
+ * Purpose:  Print the stuff that should appear at the end of an XML file
+ *           that contains a series of ClassAds.
+ *
+ **************************************************************************/
+void AddClassAdXMLFileFooter(std::string &buffer)
+{
+	buffer += "</classads>\n";
+	return;
+
+}
