@@ -291,6 +291,18 @@ Sinful::getSharedPortID() const
 	return getParam(ATTR_SOCK);
 }
 
+void
+Sinful::setAlias(char const *alias)
+{
+	setParam(ATTR_ALIAS,alias);
+}
+
+char const *
+Sinful::getAlias() const
+{
+	return getParam(ATTR_ALIAS);
+}
+
 void
 Sinful::setPrivateAddr(char const *addr)
 {
