@@ -21,6 +21,7 @@
 #include <string.h>
 #include "condor_config.h"
 #include "dbms_utils.h"
+#include "misc_utils.h"
 #include "condor_attributes.h"
 #include "MyString.h"
 
@@ -30,6 +31,88 @@
 #include "condor_tt/condor_ttdb.h"
 #include "condor_tt/jobqueuecollection.h"
 
+MyString condor_ttdb_fillEscapeCharacters(const char * str, dbtype dt) {
+	int i;
+	
+	int len = strlen(str);
+
+	MyString newstr;
+        
+	for (i = 0; i < len; i++) {
+		switch(str[i]) {
+        case '\\':
+			if (dt == T_PGSQL) {
+					/* postgres need to escape backslash */
+				newstr += '\\';
+				newstr += '\\';
+			}
+            break;
+        case '\'':
+				/* postgres can escape a single quote with 
+				   another single quote */
+            newstr += '\'';
+            newstr += '\'';
+            break;
+        default:
+            newstr += str[i];
+            break;
+		}
+	}
+    return newstr;
+}
+
+MyString condor_ttdb_buildts(time_t *tv, dbtype dt)
+{
+	char tsv[100];
+	struct tm *tm;	
+	MyString rv;
+
+	tm = localtime(tv);		
+
+	snprintf(tsv, 100, "%d/%d/%d %02d:%02d:%02d %s", 
+			 tm->tm_mon+1,
+			 tm->tm_mday,
+			 tm->tm_year+1900,
+			 tm->tm_hour,
+			 tm->tm_min,
+			 tm->tm_sec,
+			 my_timezone(tm->tm_isdst));	
+
+	switch(dt) {
+	case T_PGSQL:
+		rv.sprintf("'%s'", tsv);		
+		break;
+	default:
+		break;
+	}
+
+	return rv;
+}
+
+MyString condor_ttdb_buildtsval(time_t *tv, dbtype dt)
+{
+	char tsv[100];
+	struct tm *tm;	
+	MyString rv;
+
+	tm = localtime(tv);		
+
+	snprintf(tsv, 100, "%d/%d/%d %02d:%02d:%02d %s", 
+			 tm->tm_mon+1,
+			 tm->tm_mday,
+			 tm->tm_year+1900,
+			 tm->tm_hour,
+			 tm->tm_min,
+			 tm->tm_sec,
+			 my_timezone(tm->tm_isdst));	
+
+	switch(dt) {
+	default:
+		break;
+	}
+
+	return rv;
+}
 extern "C" {
 
 //! Gets the writer password required by the quill++
@@ -57,7 +140,7 @@ static char * getWritePassword(const char *write_passwd_fname,
 
 	len = strlen(prefix);
 
-	fp = safe_fopen_wrapper(write_passwd_fname, "r");
+	fp = safe_fopen_wrapper_follow(write_passwd_fname, "r");
 
 	if(fp == NULL) {
 		EXCEPT("Unable to open password file %s\n", write_passwd_fname);
