@@ -2052,18 +2052,14 @@ CreateUserDirectory ( HANDLE user_token, PCSTR directory ) {
 
     SECURITY_DESCRIPTOR         security_descriptor;
     SECURITY_ATTRIBUTES         security_attributes;
-    PACL                        acl             = NULL;
+    PACL                        pacl            = NULL;
     SID_IDENTIFIER_AUTHORITY    nt_authority    = SECURITY_NT_AUTHORITY;
     PSID                        user_sid        = NULL,
                                 system_sid      = NULL,
                                 admin_sid       = NULL;
-    PSID                        sids[]          = { user_sid, 
-                                                    system_sid, 
-                                                    admin_sid };
     DWORD                       last_error      = ERROR_SUCCESS,
                                 size            = 0,
-                                i               = 0,
-                                count           = 0;
+                                i               = 0;
     ACE_HEADER                  *ace_header     = NULL;
     BOOL                        got_sid         = FALSE,
                                 initialized     = FALSE,
@@ -2143,10 +2139,11 @@ CreateUserDirectory ( HANDLE user_token, PCSTR directory ) {
              + ( 6 * ( sizeof ( ACCESS_ALLOWED_ACE ) 
                        - sizeof ( DWORD ) ) );
 
-        acl = (PACL) new BYTE[size];
+        pacl = (PACL) malloc(size);
+        ASSERT( pacl );
 
         initialized = InitializeAcl (
-            acl, 
+            pacl, 
             size, 
             ACL_REVISION );
 
@@ -2165,11 +2162,12 @@ CreateUserDirectory ( HANDLE user_token, PCSTR directory ) {
         }
 
         /* add non-inheritable ACEs */
-        count = condor_countof ( sids );
+        PSID sids[] = { user_sid, system_sid, admin_sid };
+        UINT count = condor_countof ( sids );
         for ( i = 0; i < count; ++i ) {
             
             added = AddAccessAllowedAce (
-                acl,
+                pacl,
                 ACL_REVISION,
                 FILE_ALL_ACCESS,
                 sids[i] );
@@ -2195,7 +2193,7 @@ CreateUserDirectory ( HANDLE user_token, PCSTR directory ) {
         for ( i = 0; i < count; ++i ) {
             
             added = AddAccessAllowedAceEx (
-                acl,
+                pacl,
                 ACL_REVISION,
                 OBJECT_INHERIT_ACE 
                 | CONTAINER_INHERIT_ACE 
@@ -2242,7 +2240,7 @@ CreateUserDirectory ( HANDLE user_token, PCSTR directory ) {
         sd_set = SetSecurityDescriptorDacl (
             &security_descriptor, 
             TRUE, 
-            acl, 
+            pacl, 
             FALSE );
 
         if ( !sd_set ) {
@@ -2299,8 +2297,8 @@ CreateUserDirectory ( HANDLE user_token, PCSTR directory ) {
             FreeSid ( admin_sid );
         }
 
-        if ( acl ) {
-             GlobalFree ( acl );
+        if ( pacl ) {
+            free (pacl);
         }
 
     }
