@@ -65,10 +65,18 @@ typedef HashTable <int, FileTransfer *> TransThreadHashTable;
 typedef HashTable <MyString, CatalogEntry *> FileCatalogHashTable;
 typedef HashTable <MyString, MyString> PluginHashTable;
 
-typedef int		(Service::*FileTransferHandler)(FileTransfer *);
+typedef int		(Service::*FileTransferHandlerCpp)(FileTransfer *);
+typedef int		(*FileTransferHandler)(FileTransfer *);
+
+enum FileTransferStatus {
+	XFER_STATUS_UNKNOWN,
+	XFER_STATUS_QUEUED,
+	XFER_STATUS_ACTIVE,
+	XFER_STATUS_DONE
+};
 
 
-class FileTransfer {
+class FileTransfer: public Service {
 
   public:
 
@@ -123,6 +131,15 @@ class FileTransfer {
 		       commands */
 	void setSecuritySession(char const *session_id);
 
+		/** Set limits on how much data will be sent/received per job
+			(i.e. per call to DoUpload() or DoDownload()).  The job is
+			put on hold if the limit is exceeded.  The files are sent
+			in the usual order, and the file that hit the limit will
+			be missing its tail.
+		 */
+	void setMaxUploadBytes(filesize_t MaxUploadBytes);
+	void setMaxDownloadBytes(filesize_t MaxUploadBytes);
+
 	/** @return 1 on success, 0 on failure */
 	int DownloadFiles(bool blocking=true);
 
@@ -135,18 +152,24 @@ class FileTransfer {
 			last transfer.  It is safe for the handler to deallocate the
 			FileTransfer object.
 		*/
-	void RegisterCallback(FileTransferHandler handler, Service* handlerclass)
+	void RegisterCallback(FileTransferHandler handler, bool want_status_updates=false)
 		{ 
 			ClientCallback = handler; 
+			ClientCallbackWantsStatusUpdates = want_status_updates;
+		}
+	void RegisterCallback(FileTransferHandlerCpp handler, Service* handlerclass, bool want_status_updates=false)
+		{ 
+			ClientCallbackCpp = handler; 
 			ClientCallbackClass = handlerclass;
+			ClientCallbackWantsStatusUpdates = want_status_updates;
 		}
 
 	enum TransferType { NoType, DownloadFilesType, UploadFilesType };
 
 	struct FileTransferInfo {
 		FileTransferInfo() : bytes(0), duration(0), type(NoType),
-		    success(true), in_progress(false), try_again(true), hold_code(0),
-		    hold_subcode(0) {}
+		    success(true), in_progress(false), xfer_status(XFER_STATUS_UNKNOWN),
+			try_again(true), hold_code(0), hold_subcode(0) {}
 
 		void addSpooledFile(char const *name_in_spool);
 
@@ -155,6 +178,7 @@ class FileTransfer {
 		TransferType type;
 		bool success;
 		bool in_progress;
+		FileTransferStatus xfer_status;
 		bool try_again;
 		int hold_code;
 		int hold_subcode;
@@ -174,6 +198,17 @@ class FileTransfer {
 
 	static int Reaper(Service *, int pid, int exit_status);
 
+	static bool SetServerShouldBlock( bool block );
+
+		// Stop accepting new transfer requests for this instance of
+		// the file transfer object.
+		// Also abort this object's active transfer, if any.
+	void stopServer();
+
+		// If this file transfer object has a transfer running in
+		// the background, kill it.
+	void abortActiveTransfer();
+
 	int Suspend();
 
 	int Continue();
@@ -273,12 +308,19 @@ class FileTransfer {
 		return true;
 	}
 
+	ClassAd *GetJobAd();
+
+	bool transferIsInProgress() { return ActiveTransferTid != -1; }
+
   protected:
 
 	int Download(ReliSock *s, bool blocking);
 	int Upload(ReliSock *s, bool blocking);
 	static int DownloadThread(void *arg, Stream *s);
 	static int UploadThread(void *arg, Stream *s);
+	int TransferPipeHandler(int p);
+	bool ReadTransferPipeMsg();
+	void UpdateXferStatus(FileTransferStatus status);
 
 		/** Actually download the files.
 			@return -1 on failure, bytes transferred otherwise
@@ -332,8 +374,11 @@ class FileTransfer {
 	int ActiveTransferTid;
 	time_t TransferStart;
 	int TransferPipe[2];
+	bool registered_xfer_pipe;
 	FileTransferHandler ClientCallback;
+	FileTransferHandlerCpp ClientCallbackCpp;
 	Service* ClientCallbackClass;
+	bool ClientCallbackWantsStatusUpdates;
 	FileTransferInfo Info;
 	PluginHashTable* plugin_table;
 	bool I_support_filetransfer_plugins;
@@ -347,6 +392,7 @@ class FileTransfer {
 	static int CommandsRegistered;
 	static int SequenceNum;
 	static int ReaperId;
+	static bool ServerShouldBlock;
 	int clientSockTimeout;
 	bool did_init;
 	bool simple_init;
@@ -356,6 +402,8 @@ class FileTransfer {
 	TransferQueueContactInfo m_xfer_queue_contact_info;
 	MyString m_jobid; // what job we are working on, for informational purposes
 	char *m_sec_session_id;
+	filesize_t MaxUploadBytes;
+	filesize_t MaxDownloadBytes;
 
 	// stores the path to the proxy after one is received
 	MyString LocalProxyName;
@@ -381,10 +429,10 @@ class FileTransfer {
 
 	// Receive message indicating that the peer is ready to receive the file
 	// and save failure information with SaveTransferInfo().
-	bool ReceiveTransferGoAhead(Stream *s,char const *fname,bool downloading,bool &go_ahead_always);
+	bool ReceiveTransferGoAhead(Stream *s,char const *fname,bool downloading,bool &go_ahead_always,filesize_t &peer_max_transfer_bytes);
 
 	// Receive message indicating that the peer is ready to receive the file.
-	bool DoReceiveTransferGoAhead(Stream *s,char const *fname,bool downloading,bool &go_ahead_always,bool &try_again,int &hold_code,int &hold_subcode,MyString &error_desc, int alive_interval);
+	bool DoReceiveTransferGoAhead(Stream *s,char const *fname,bool downloading,bool &go_ahead_always,filesize_t &peer_max_transfer_bytes,bool &try_again,int &hold_code,int &hold_subcode,MyString &error_desc, int alive_interval);
 
 	// Obtain permission to receive a file download and then tell our
 	// peer to go ahead and send it.
@@ -393,6 +441,8 @@ class FileTransfer {
 
 	bool DoObtainAndSendTransferGoAhead(DCTransferQueue &xfer_queue,bool downloading,Stream *s,char const *full_fname,bool &go_ahead_always,bool &try_again,int &hold_code,int &hold_subcode,MyString &error_desc);
 
+	std::string GetTransferQueueUser();
+
 	// Report information about completed transfer from child thread.
 	bool WriteStatusToTransferPipe(filesize_t total_bytes);
 	ClassAd jobAd;
@@ -419,6 +469,8 @@ class FileTransfer {
 		// Returns true if specified path points into the spool directory.
 		// This does not do an existence check for the file.
 	bool outputFileIsSpooled(char const *fname);
+
+	void callClientCallback();
 };
 
 // returns 0 if no expiration
