@@ -191,12 +191,12 @@ HistorySnapshot::printResults(SQLQuery *queryhor,
 			  MyString owner, ad_str, temp;
 			  int compl_date;
 
-			  ad->sPrint(ad_str);
+                       sPrintAd(ad_str, *ad);
                        if (!ad->LookupString(ATTR_OWNER, owner))
                                  owner = "NULL";
                          if (!ad->LookupInteger(ATTR_COMPLETION_DATE, compl_date))
                                  compl_date = 0;
-                         temp.sprintf("*** Offset = %ld ClusterId = %d ProcId = %d Owner = \"%s\" CompletionDate = %d\n",
+                         temp.formatstr("*** Offset = %ld ClusterId = %d ProcId = %d Owner = \"%s\" CompletionDate = %d\n",
                                         offset - last_line, curClusterId_hor, curProcId_hor, owner.Value(), compl_date);
 
                          offset += ad_str.Length() + temp.Length();
@@ -204,7 +204,7 @@ HistorySnapshot::printResults(SQLQuery *queryhor,
                          fprintf(stdout, "%s", ad_str.Value());
                          fprintf(stdout, "%s", temp.Value());
                  }     else if (longformat) {
-                         ad->fPrint(stdout);
+                         fPrintAd(stdout, *ad);
                          printf("\n");
                  }
 
