@@ -27,19 +27,62 @@
 #include <dirent.h>
 #endif
 
+/** is it a *.old file? */
+static int isOldString(const char *str);
+
+/** is the argument an ISO timestamp? */
+static int isTimestampString(const char *str);
+
+/** is the argument a valid log filename? */
+static int isLogFilename(const char *filename);
+
+/** find the oldest file in the directory matching the 
+  *current base name according to iso time ending 
+ */
+static char *findOldest(char *dirName, int *count);
 
 #ifdef WIN32
-char searchLogName[PATH_MAX];
+char * searchLogName = NULL;
 #endif
  
-char logBaseName[PATH_MAX]; 
-char baseDirName[PATH_MAX];
+char * logBaseName = NULL;
+char * baseDirName = NULL;
 int isInitialized = 0;
 
 int numLogs = 0;
 
+// max size of an iso timestamp extenstion 4+2+2+1+2+2+2
+#define MAX_ISO_TIMESTAMP 16
+
 /** create an ISO timestamp string */
-static char *createTimestampString();
+static void createTimestampString(std::string & timestamp, time_t tt)
+{
+	struct tm *tm;
+	tm = localtime(&tt);
+
+	char buf[80];
+	strftime(buf, sizeof(buf), "%Y%m%dT%H%M%S", tm);
+
+	timestamp = buf;
+}
+
+long long quantizeTimestamp(time_t tt, long long secs)
+{
+	if ( ! secs) return tt;
+
+	static int leap_sec = -1;
+	if (leap_sec < 0) {
+		struct tm * ptm = localtime(&tt);
+		ptm->tm_hour  = 0;
+		ptm->tm_min = 0;
+		ptm->tm_sec = 0;
+		time_t today = mktime(ptm);
+		leap_sec = today % 3600;
+	}
+
+	return tt - (tt % secs);
+}
+
 
 #ifndef WIN32
 int scandirectory(const char *dir, struct dirent ***namelist,
@@ -82,8 +125,8 @@ int scandirectory(const char *dir, struct dirent ***namelist,
 }
 
 int doalphasort(const void *a, const void *b) {
-        const struct dirent **d1 = (const struct dirent**)a;
-        const struct dirent **d2 = (const struct dirent**)b;		
+        const struct dirent **d1 = (const struct dirent**)const_cast<void*>(a);
+        const struct dirent **d2 = (const struct dirent**)const_cast<void*>(b);
         return(strcmp(const_cast<char*>((*d1)->d_name),
 		const_cast<char*>((*d2)->d_name)));
 }
@@ -99,11 +142,18 @@ void setBaseName(const char *baseName) {
 	if (isInitialized == 0) {
 		char *tmpDir;
 
-		sprintf(logBaseName, "%s", baseName);
+		if (logBaseName)
+			free(logBaseName);
+		logBaseName = strdup(baseName);
 		tmpDir = condor_dirname(logBaseName);
-		sprintf(baseDirName, "%s", tmpDir);
+		if (baseDirName)
+			free(baseDirName);
+		baseDirName = strdup(tmpDir);
 		free(tmpDir);
 #ifdef WIN32
+		if (searchLogName)
+			free(searchLogName);
+		searchLogName = (char *)malloc(strlen(logBaseName)+3);
 		sprintf(searchLogName, "%s.*", (const char*)logBaseName); 		
 #endif
 		isInitialized = 1;
@@ -113,30 +163,31 @@ void setBaseName(const char *baseName) {
 int
 rotateSingle()
 {
-	return rotateTimestamp("old", 1);
+	return rotateTimestamp("old", 1, 0);
 } 
 
 
 const char *
-createRotateFilename(const char *ending, int maxNum)
+createRotateFilename(const char *ending, int maxNum, time_t tt)
 {
-	const char *timeStamp;
+	static std::string timeStamp;
 	if (maxNum <= 1)
 		timeStamp = "old";
 	else 	
 	if (ending == NULL) {
-		timeStamp = createTimestampString();
+		createTimestampString(timeStamp, tt);
 	} else {
 		timeStamp = ending;
 	}
-	return timeStamp;
+	return timeStamp.c_str();
 }
 
+// rotate the current file to file.timestamp
 int
-rotateTimestamp(const char *timeStamp, int maxNum)
+rotateTimestamp(const char *timeStamp, int maxNum, time_t tt)
 {
 	int save_errno;
-	const char *ts = createRotateFilename(timeStamp, maxNum);
+	const char *ts = createRotateFilename(timeStamp, maxNum, tt);
 
 	// First, select a name for the rotated history file
 	char *rotated_log_name = (char*)malloc(strlen(logBaseName) + strlen(ts) + 2) ;
@@ -147,7 +198,14 @@ rotateTimestamp(const char *timeStamp, int maxNum)
 	return save_errno; // will be 0 in case of success.
 }
 
-int cleanUp(int maxNum) {
+
+/*
+tj : change to delete files rather than 'rotating' them when the number exceeds 1
+this code currently will take the oldest .timestamp file and rename it to .old
+which seems wrong.
+*/
+
+int cleanUpOldLogFiles(int maxNum) {
 	int count;
 	char *oldFile = NULL;
 	char empty[BUFSIZ];
@@ -177,16 +235,13 @@ int cleanUp(int maxNum) {
 }
 
 
-
-#ifndef WIN32
-
-int isOldString(char *str){
+static int isOldString(const char *str){
 	if (strcmp(str, "old") == 0)
 		return 1;
 	return 0;
 }
 
-int isTimestampString(char *str) {
+static int isTimestampString(const char *str) {
 	int len = strlen(str);
 	if (len != 15) {
 		return 0;
@@ -207,10 +262,13 @@ int isTimestampString(char *str) {
 	return 1;
 } 
 
-int isLogFilename( char *filename) {
+static int isLogFilename(const char *filename) {
 	int dirLen = strlen(baseDirName);
-	if (baseDirName[dirLen-1] != DIR_DELIM_CHAR)
-		++dirLen;
+#ifdef WIN32
+	if (baseDirName[dirLen-1] != DIR_DELIM_CHAR && baseDirName[dirLen-1] != '/') ++dirLen;
+#else
+	if (baseDirName[dirLen-1] != DIR_DELIM_CHAR) ++dirLen;
+#endif
     int fLen = strlen(logBaseName);
     if (strncmp(filename, (logBaseName+dirLen), fLen - dirLen) == 0 ) {
     	if (  (strlen(filename) > unsigned(fLen-dirLen)) && 
@@ -224,10 +282,10 @@ int isLogFilename( char *filename) {
 	return 0;
 }
 
+#ifndef WIN32
+
 int file_select(const struct dirent *entry) {
-	
-	char *entryData = const_cast<char*>(entry->d_name);
-	return isLogFilename(entryData) ? 1 : 0;
+	return isLogFilename(entry->d_name);
 }
 
 
@@ -246,11 +304,14 @@ char *findOldest(char *dirName, int *count) {
 }
 #else
 
+// return a count of files matching the logfile name pattern,
+// and the filename of the oldest of these files.
 char *findOldest(char *dirName, int *count) {
-	char *oldFile;
+	char *oldFile = NULL;
 	WIN32_FIND_DATA ffd;
 	HANDLE hFind = INVALID_HANDLE_VALUE;
 	int result;
+	int cchBase = strlen(logBaseName);
 	
 	*count = 0;
 	
@@ -259,34 +320,31 @@ char *findOldest(char *dirName, int *count) {
 		return NULL;
 	}
 	
-	result = FindNextFile(hFind, &ffd);
-	if (result == 0) {
-		return NULL;
-	} else {
-		oldFile = (char*)malloc(strlen(logBaseName) + 16);
-		ASSERT( oldFile );
-		strcpy(oldFile, ffd.cFileName);
-	}
-	while (result != 0) {
+	// the filenames returned from FindNextFile aren't necessarily sorted
+	// so scan the list, and return the one that sorts the lowest
+	// this should work correctly for .old (because there is only one)
+	// and for .{TIMESTAMP} because the timestamps will sort by date.
+
+	while ((result = FindNextFile(hFind, &ffd)) != 0) {
+		if ( ! isLogFilename(ffd.cFileName))
+			continue;
 		++(*count);
-		result = FindNextFile(hFind, &ffd);
+		if ( ! oldFile) {
+			oldFile = (char*)malloc(strlen(logBaseName) + 2 + MAX_ISO_TIMESTAMP);
+			ASSERT( oldFile );
+			strcpy(oldFile, ffd.cFileName);
+		} else {
+			int cch = strlen(ffd.cFileName);
+			if (cch > cchBase && strcmp(ffd.cFileName+cchBase, oldFile+cchBase) < 0) {
+				strcpy(oldFile, ffd.cFileName);
+			}
+		}
 	}
-	
+
+	FindClose(hFind);
 	return oldFile;
 }
 
 #endif
 
-char *createTimestampString() {
-	time_t clock_now;
-	struct tm *tm;
-	static char timebuf[80];
-	static char *timeFormat = 0;
-	timeFormat = strdup("%Y%m%dT%H%M%S");
-	memset((void*)&clock_now,0,sizeof(time_t)); 
-	(void)time(  &clock_now );
-	tm = localtime( &clock_now );	
-	strftime(timebuf, 80, timeFormat, tm);
-	return timebuf;
-}
 
