@@ -1030,7 +1030,7 @@ bool SetPrivilege(
 
 #ifdef PERM_OBJ_DEBUG
 
-extern void set_debug_flags( const char *strflags );
+extern void set_debug_flags( const char *strflags, int flags );
 extern "C" FILE	*DebugFP;
 
 // Main method for testing the Perm functions
@@ -1041,7 +1041,7 @@ main(int argc, char* argv[]) {
 	perm* foo = new perm();
 
 	DebugFP = stdout;
-	set_debug_flags( "D_ALL" );
+	set_debug_flags( "D_ALL", 0 );
 	//char p_ntdomain[80];
 	//char buf[100];
 	
