@@ -44,7 +44,7 @@ bool operator>=(const MyString& L, const std::string& R) { return R <= L.Value()
 bool operator>=(const std::string& L, const MyString& R) { return L >= R.Value(); }
 
 
-int vsprintf(std::string& s, const char* format, va_list pargs) {
+int vformatstr(std::string& s, const char* format, va_list pargs) {
     char fixbuf[STL_STRING_UTILS_FIXBUF];
     const int fixlen = sizeof(fixbuf)/sizeof(fixbuf[0]);
 	int n;
@@ -102,45 +102,74 @@ int vsprintf(std::string& s, const char* format, va_list pargs) {
     return nn;
 }
 
-int sprintf(std::string& s, const char* format, ...) {
+int formatstr(std::string& s, const char* format, ...) {
     va_list args;
     va_start(args, format);
-    int r = vsprintf(s, format, args);
+    int r = vformatstr(s, format, args);
     va_end(args);
     return r;
 }
 
-int sprintf(MyString& s, const char* format, ...) {
+int formatstr(MyString& s, const char* format, ...) {
     va_list args;
     std::string t;
     va_start(args, format);
     // this gets me the sprintf-standard return value (# chars printed)
-    int r = vsprintf(t, format, args);
+    int r = vformatstr(t, format, args);
     va_end(args);
     assign(s, t);
     return r;
 }
 
-int sprintf_cat(std::string& s, const char* format, ...) {
+int formatstr_cat(std::string& s, const char* format, ...) {
     va_list args;
     std::string t;
     va_start(args, format);
-    int r = vsprintf(t, format, args);
+    int r = vformatstr(t, format, args);
     va_end(args);
     s += t;
     return r;
 }
 
-int sprintf_cat(MyString& s, const char* format, ...) {
+int formatstr_cat(MyString& s, const char* format, ...) {
     va_list args;
     std::string t;
     va_start(args, format);
-    int r = vsprintf(t, format, args);
+    int r = vformatstr(t, format, args);
     va_end(args);
     s += t.c_str();
     return r;
 }
 
+// to replace MyString with std::string we need a compatible read-line function
+bool readLine(std::string& str, FILE *fp, bool append)
+{
+	bool first_time = true;
+
+	ASSERT( fp );
+
+	while( 1 ) {
+		char buf[1024];
+		if( ! fgets(buf, 1024, fp) ) {
+			if (first_time) {
+				return false;
+			}
+			return true;
+		}
+		if (first_time && !append) {
+			str = buf;
+			first_time = false;
+		} else {
+			str += buf;
+		}
+		if ((str.size() > 0) && (str[str.size()-1] == '\n')) {
+				// we found a newline, return success
+			return true;
+		}
+	}
+}
+
+
 bool chomp(std::string &str)
 {
 	bool chomped = false;
@@ -187,7 +216,7 @@ void trim( std::string &str )
 
 void upper_case( std::string &str )
 {
-	for ( int i = 0; str[i]; i++ ) {
+	for ( unsigned int i = 0; i<str.length(); i++ ) {
 		if ( str[i] >= 'a' && str[i] <= 'z' ) {
 			str[i] = _toupper( str[i] );
 		}
@@ -196,13 +225,71 @@ void upper_case( std::string &str )
 
 void lower_case( std::string &str )
 {
-	for ( int i = 0; str[i]; i++ ) {
+	for ( unsigned int i = 0; i<str.length(); i++ ) {
 		if ( str[i] >= 'A' && str[i] <= 'Z' ) {
 			str[i] = _tolower( str[i] );
 		}
 	}
 }
 
+void title_case( std::string &str )
+{
+	bool upper = true;
+	for ( unsigned int i = 0; i<str.length(); i++ ) {
+		if (upper) {
+			if ( str[i] >= 'a' && str[i] <= 'z' ) {
+				str[i] = _toupper( str[i] );
+			}
+		} else {
+			if ( str[i] >= 'A' && str[i] <= 'Z' ) {
+				str[i] = _tolower( str[i] );
+			}
+		}
+		upper = isspace(str[i]);
+	}
+}
+
+// returns true if pre is non-empty and str is the same as pre up to pre.size()
+bool starts_with(const std::string& str, const std::string& pre)
+{
+	size_t cp = pre.size();
+	if (cp <= 0)
+		return false;
+
+	size_t cs = str.size();
+	if (cs < cp)
+		return false;
+
+	for (size_t ix = 0; ix < cp; ++ix) {
+		if (str[ix] != pre[ix])
+			return false;
+	}
+	return true;
+}
+
+bool starts_with_ignore_case(const std::string& str, const std::string& pre)
+{
+	size_t cp = pre.size();
+	if (cp <= 0)
+		return false;
+
+	size_t cs = str.size();
+	if (cs < cp)
+		return false;
+
+	// to optimize for the case where str and pre are the same case,
+	// we first do a case-sensitive compare, and do case conversion only
+	// if the sensitive compare fails. this avoids branches when
+	// str and pre of the same case, which is the most likely scenario.
+	for (size_t ix = 0; ix < cp; ++ix) {
+		if (str[ix] != pre[ix]) {
+			if (_tolower(str[ix]) != _tolower(pre[ix]))
+				return false;
+		}
+	}
+	return true;
+}
+
 static char *tokenBuf = NULL;
 static char *nextToken = NULL;
 
