@@ -49,6 +49,12 @@ THREAD_LOCAL_STORAGE static priv_state CurrentPrivState = PRIV_UNKNOWN;
 static priv_state CurrentPrivState = PRIV_UNKNOWN;
 #endif
 
+priv_state
+get_priv_state(void)
+{
+	return CurrentPrivState;
+}
+
 #if !defined(WIN32)
 /*
    supplementary group used to track process families. if nonzero,
@@ -1292,12 +1298,6 @@ set_file_owner_ids( uid_t uid, gid_t gid )
 }
 
 
-priv_state
-get_priv_state(void)
-{
-	return CurrentPrivState;
-}
-
 priv_state
 _set_priv(priv_state s, const char *file, int line, int dologging)
 {
@@ -1394,6 +1394,19 @@ get_condor_gid()
 	return CondorGid;
 }
 
+bool
+get_condor_uid_if_inited(uid_t &uid,gid_t &gid)
+{
+	if( !CondorIdsInited ) {
+		uid = 0;
+		gid = 0;
+		return false;
+	}
+
+	uid = CondorUid;
+	gid = CondorGid;
+	return true;
+}
 
 /* This returns the string containing the username of whatever uid
    priv_state condor gives you. */
