@@ -353,7 +353,7 @@ write_to_daemoncore_pipe(const char* fmt, ... )
 	MyString output;
 	va_list args;
 	va_start(args, fmt);
-	output.vsprintf(fmt, args);
+	output.vformatstr(fmt, args);
 	write_to_daemoncore_pipe(vmgahp_stdout_pipe, 
 			output.Value(), output.Length());
 	va_end(args);
@@ -377,6 +377,7 @@ write_stderr_to_pipe()
 	}
 }
 
+#ifndef vmprintf
 void vmprintf( int flags, const char *fmt, ... ) 
 {
 	int saved_flags = 0;
@@ -425,7 +426,7 @@ void vmprintf( int flags, const char *fmt, ... )
 	}
 	oriDebugFlags = saved_flags;
 }
-
+#endif
 
 void 
 initialize_uids(void)
@@ -659,12 +660,24 @@ int systemCommand( ArgList &args, priv_state priv, StringList *cmd_out, StringLi
 	      }
 
 
+			/* to be safe, we want to switch our real uid/gid to our
+			   effective uid/gid (shedding any privledges we've got).
+			   we also want to drop any supplimental groups we're in.
+			   we want to run this popen()'ed thing as our effective
+			   uid/gid, dropping the real uid/gid.  all of these calls
+			   will fail if we don't have a ruid of 0 (root), but
+			   that's harmless.  also, note that we have to stash our
+			   effective uid, then switch our euid to 0 to be able to
+			   set our real uid/gid.
+			   We wrap some of the calls in if-statements to quiet some
+			   compilers that object to us not checking the return values.
+			*/
 	    uid_t euid = geteuid();
 	    gid_t egid = getegid();
-	    seteuid( 0 );
+	    if ( seteuid( 0 ) ) { }
 	    setgroups( 1, &egid );
-	    setgid( egid );
-	    setuid( euid );
+	    if ( setgid( egid ) ) { }
+	    if ( setuid( euid ) ) _exit(ENOEXEC); // Unsafe?
 	    
 	    install_sig_handler(SIGPIPE, SIG_DFL);
 	    sigset_t sigs;
