@@ -18,5 +18,9 @@ bool dcloud_action_worker(int argc, char **argv, std::string &output_string);
 bool dcloud_info_worker(int argc, char **argv, std::string &output_string);
 bool dcloud_statusall_worker(int argc, char **argv, std::string &output_string);
 bool dcloud_find_worker(int argc, char **argv, std::string &output_string);
+bool dcloud_max_name_length_worker(int argc, char **argv,
+                                   std::string &output_string);
+bool dcloud_start_auto_worker(int argc, char **argv,
+                              std::string &output_string);
 
 #endif
